Package: BradleyTerry2
Title: Bradley-Terry Models
Version: 1.1.3
Authors@R: c(
    person("Heather", "Turner", , "ht@heatherturner.net", role = c("aut", "cre")),
    person("David", "Firth", role = "aut")
  )
Description: Specify and fit the Bradley-Terry model, including structured
    versions in which the parameters are related to explanatory variables
    through a linear predictor and versions with contest-specific effects,
    such as a home advantage.
License: GPL (>= 2)
URL: https://github.com/hturner/BradleyTerry2
BugReports: https://github.com/hturner/BradleyTerry2/issues
Depends: R (>= 2.10)
Imports: brglm, gtools, lme4 (>= 1.0), qvcalc, stats
Suggests: bookdown, knitr, litedown, prefmod, testthat
Enhances: gnm
VignetteBuilder: knitr, litedown
Encoding: UTF-8
Language: en-GB
LazyData: yes
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-04-10 10:20:56 UTC; stspao
Author: Heather Turner [aut, cre],
  David Firth [aut]
Maintainer: Heather Turner <ht@heatherturner.net>
Repository: CRAN
Date/Publication: 2025-04-10 19:20:44 UTC
Built: R 4.3.3; ; 2025-04-10 21:55:07 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(add1,BTm)
S3method(anova,BTm)
S3method(coef,BTabilities)
S3method(drop1,BTm)
S3method(formula,BTm)
S3method(model.matrix,BTm)
S3method(predict,BTglmmPQL)
S3method(predict,BTm)
S3method(print,BTabilities)
S3method(print,BTglmmPQL)
S3method(print,BTm)
S3method(print,summary.BTglmmPQL)
S3method(qvcalc,BTabilities)
S3method(residuals,BTm)
S3method(summary,BTglmmPQL)
S3method(vcov,BTabilities)
S3method(vcov,BTglmmPQL)
export(BTabilities)
export(BTm)
export(GenDavidson)
export(countsToBinomial)
export(glmmPQL)
export(glmmPQL.control)
export(plotProportions)
export(qvcalc)
importFrom(brglm,brglm)
importFrom(graphics,curve)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(gtools,combinations)
importFrom(lme4,findbars)
importFrom(lme4,nobars)
importFrom(qvcalc,qvcalc)
importFrom(qvcalc,qvcalc.default)
importFrom(stats,.checkMFClasses)
importFrom(stats,.getXlevels)
importFrom(stats,C)
importFrom(stats,add.scope)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,contrasts)
importFrom(stats,delete.response)
importFrom(stats,drop.scope)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm.control)
importFrom(stats,glm.fit)
importFrom(stats,is.empty.model)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,napredict)
importFrom(stats,naprint)
importFrom(stats,optimize)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,printCoefmat)
importFrom(stats,reformulate)
importFrom(stats,relevel)
importFrom(stats,runif)
importFrom(stats,symnum)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,vcov)
importFrom(utils,flush.console)

			R-Forge SVN README


(See "http://download.r-forge.r-project.org/manuals/R-Forge_Manual.pdf"
       for detailed information on registering a new project.)

1. Introduction
-----------------------------------------------------------------------
R is free software distributed under a GNU-style copyleft. R-Forge is
a central platform for the development of R packages, R-related 
software and further projects. Among many other web-based features it 
provides facilities for collaborative source code management via 
Subversion (SVN).

2. The directory you're in
-----------------------------------------------------------------------
This is the repository of your project. It contains two important
pre-defined directories namely 'pkg' and 'www'. These directories must 
not be deleted otherwise R-Forge's core functionality will not be 
available (i.e., daily checking and building of your package or the 
project websites).
'pkg' and 'www' are standardized and therefore are going to be
described in this README. The rest of your repository can be used as
you like.

3. 'pkg' directory
-----------------------------------------------------------------------
To make use of the package building and checking feature the package 
source code has to be put into the 'pkg' directory of your repository 
(i.e., 'pkg/DESCRIPTION', 'pkg/R', 'pkg/man', etc.) or, alternatively,
a subdirectory of 'pkg'. The latter structure allows for having more 
than one package in a single project, e.g., if a project consists of 
the packages foo and bar then the source code will be located in 
'pkg/foo' and 'pkg/bar', respectively.

R-Forge automatically examines the 'pkg' directory of every repository 
and builds the package sources as well as the package binaries on a
daily basis for Mac OSX and Windows (if applicable). The package builds
are provided in the 'R Packages' tab for download or can be installed
directly in R from a CRAN-style repository using 
'install.packages("foo", repos="http://R-Forge.R-project.org")'. 
Furthermore, in the 'R Packages' tab developers can examine logs 
generated by the build and check process on different platforms.

4. 'www' directory
-----------------------------------------------------------------------
Developers may present their work on a subdomain of R-Forge, e.g.,
'http://foo.R-Forge.R-project.org', or via a link to an external
website.

This directory contains the project homepage which gets updated hourly
on R-Forge, so please take into consideration that it will not be 
available right after you commit your changes or additions. 

5. Help
-----------------------------------------------------------------------
If you need help don't hesitate to contact us at R-Forge@R-project.org

<!-- README.md is generated from README.Rmd. Please edit that file -->

# BradleyTerry2

[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/BradleyTerry2)](https://cran.r-project.org/package=BradleyTerry2)

Specify and fit the Bradley-Terry model, including structured versions
in which the parameters are related to explanatory variables through a
linear predictor and versions with contest-specific effects, such as a
home advantage.

## Installation

You can install **BradleyTerry2** from github with:

``` r
# install.packages("devtools")
devtools::install_github("hturner/BradleyTerry2")
```

## Code of conduct

Please note that this project is released with a [Contributor Code of
Conduct](https://github.com/hturner/BradleyTerry2/blob/master/CONDUCT.md).
By participating in this project you agree to abide by its terms.

## ----include=FALSE------------------------------------------------------------
library <- function(...) suppressPackageStartupMessages(base::library(...))
library(knitr)
opts_chunk$set(
tidy=FALSE
)

## ----set_options, echo = FALSE--------------------------------------
options(prompt = "R> ", continue = "+  ", width = 70,
        useFancyQuotes = FALSE, digits = 7)

## ----LoadBradleyTerry2----------------------------------------------
library("BradleyTerry2")

## ----CitationData---------------------------------------------------
data("citations", package = "BradleyTerry2")

## ----CitationData2--------------------------------------------------
citations

## ----countsToBinomial-----------------------------------------------
citations.sf <- countsToBinomial(citations)
names(citations.sf)[1:2] <- c("journal1", "journal2")
citations.sf

## ----citeModel------------------------------------------------------
citeModel <- BTm(cbind(win1, win2), journal1, journal2, ~ journal,
    id = "journal", data = citations.sf)
citeModel

## ----citeModelupdate------------------------------------------------
update(citeModel, refcat = "JASA")

## ----citeModelupdate2-----------------------------------------------
update(citeModel, br = TRUE)

## ----lizModel-------------------------------------------------------
options(show.signif.stars = FALSE)
data("flatlizards", package = "BradleyTerry2")
lizModel <- BTm(1, winner, loser, ~ SVL[..] + (1|..),
                data = flatlizards)

## ----summarize_lizModel---------------------------------------------
summary(lizModel)

## ----lizModel2------------------------------------------------------
lizModel2 <- BTm(1, winner, loser,
            ~ throat.PC1[..] + throat.PC3[..] +
            head.length[..] + SVL[..] + (1|..),
            data = flatlizards)
summary(lizModel2)

## ----baseball-------------------------------------------------------
data("baseball", package = "BradleyTerry2")
head(baseball)

## ----baseballModel--------------------------------------------------
baseballModel1 <- BTm(cbind(home.wins, away.wins), home.team, away.team,
                      data = baseball, id = "team")
summary(baseballModel1)

## ----baseballDataUpdate---------------------------------------------
baseball$home.team <- data.frame(team = baseball$home.team, at.home = 1)
baseball$away.team <- data.frame(team = baseball$away.team, at.home = 0)

## ----baseballModelupdate--------------------------------------------
baseballModel2 <- update(baseballModel1, formula = ~ team + at.home)
summary(baseballModel2)

## ----CEMSmodel------------------------------------------------------
data("CEMS", package = "BradleyTerry2")
table8.model <-  BTm(outcome = cbind(win1.adj, win2.adj),
    player1 = school1, player2 = school2, formula = ~ .. +
    WOR[student] * LAT[..] +  DEG[student] * St.Gallen[..] +
    STUD[student] * Paris[..] + STUD[student] * St.Gallen[..] +
    ENG[student] * St.Gallen[..] + FRA[student] * London[..] +
    FRA[student] * Paris[..] + SPA[student] * Barcelona[..] +
    ITA[student] * London[..] + ITA[student] * Milano[..] +
    SEX[student] * Milano[..],
    refcat = "Stockholm", data = CEMS)

## ----BTabilities----------------------------------------------------
BTabilities(baseballModel2)

## ----BTabilities2---------------------------------------------------
head(BTabilities(lizModel2), 4)

## ----figqvplot, echo=FALSE , fig.cap="Estimated relative abilities of baseball teams.", fig.alt="The ability for Baltimore is fixed at zero, with an interval ranging from -0.5 to 0.5. Boston has a relative ability near 1.2; Cleveland around 0.7. The remaining teams have relative abilities around 1.3 to 1.6. The intervals are based on quasi standard errors and all have length of approximately 1. Therefore, aside from Cleveland, all teams are clearly significantly stronger than Baltimore as the intervals do not overlap.", fig.show='hold', fig.align="center", out.width="67.0%"----
knitr::include_graphics(c("baseball-qvplot.png"))

## ----residuals------------------------------------------------------
res.pearson <- round(residuals(lizModel2), 3)
head(cbind(flatlizards$contests, res.pearson), 4)

## ----BTresiduals----------------------------------------------------
res <- residuals(lizModel2, type = "grouped")
#  with(flatlizards$predictors, plot(throat.PC2, res))
#  with(flatlizards$predictors, plot(head.width, res))

## ----residualWLS----------------------------------------------------
lm(res ~ throat.PC1, weights = attr(res, "weights"),
   data = flatlizards$predictors)
lm(res ~ head.length, weights = attr(res, "weights"),
   data = flatlizards$predictors)

## ----figresiduals, echo=FALSE , fig.cap="Lizard residuals for the simple model lizModel, plotted against throat.PC3.", fig.alt="The residuals are quite spread out over the range -2 to 2, but the distribution is clearly not uniform over the range of the predictor variable, throat.PC3. Residuals between -2 and -1 range correspond to values throat.PC3 between -6 and 4; residuals between -1 and 1 correspond to throat.PC3 values of -4 to 4, and residuals from 1 to 2 correspond to throat.PC3 values between -3 and 6. Thus there is an overall positive correlation bewteen the residuals and throat.PC3.", fig.show='hold', fig.align="center", out.width="69.0%"----
knitr::include_graphics(c("residuals.png"))

## ----baseballModel2_call--------------------------------------------
 baseballModel2$call

## ----str_baseball---------------------------------------------------
str(baseball, vec.len = 2)

## ----first_comparison-----------------------------------------------
baseball$home.team[1,]
baseball$away.team[1,]

## ----first_outcome--------------------------------------------------
 baseball[1, c("home.wins", "away.wins")]

## ----str_CEMS-------------------------------------------------------
str(CEMS, vec.len = 2)

## ----student-specific_data------------------------------------------
library("prefmod")
student <- cemspc[c("ENG", "SEX")]
student$ENG <- factor(student$ENG, levels = 1:2,
                      labels = c("good", "poor"))
student$SEX <- factor(student$SEX, levels = 1:2,
                      labels = c("female", "male"))

## ----student_factor-------------------------------------------------
cems <- list(student = student)
student <- gl(303, 1, 303 * 15) #303 students, 15 comparisons
contest <- data.frame(student = student)

## ----binomial_response----------------------------------------------
win <- cemspc[, 1:15] == 0
lose <- cemspc[, 1:15] == 2
draw <- cemspc[, 1:15] == 1
contest$win.adj <- c(win + draw/2)
contest$lose.adj <- c(lose + draw/2)

## ----school_factors-------------------------------------------------
lab <- c("London", "Paris", "Milano", "St. Gallen", "Barcelona",
         "Stockholm")
contest$school1 <- factor(sequence(1:5), levels = 1:6, labels = lab)
contest$school2 <- factor(rep(2:6, 1:5), levels = 1:6, labels = lab)

## ----cems_data------------------------------------------------------
cems$contest <- contest

## ----functions, echo = FALSE------------------------------
## cf. prompt
options(width = 55)
for (fn in getNamespaceExports("BradleyTerry2")) {
    name <- as.name(fn)
    args <- formals(fn)
    n <- length(args)
    arg.names <- arg.n <- names(args)
    arg.n[arg.n == "..."] <- "\\dots"
    is.missing.arg <- function(arg) typeof(arg) == "symbol" &&
        deparse(arg) == ""
    Call <- paste(name, "(", sep = "")
        for (i in seq_len(n)) {
        Call <- paste(Call, arg.names[i], if (!is.missing.arg(args[[i]]))
            paste(" = ", paste(deparse(args[[i]]),
                collapse = "\n"), sep = ""), sep = "")
        if (i != n)
            Call <- paste(Call, ", ", sep = "")
    }
    Call <- paste(Call, ")", sep = "")
    cat(deparse(parse(text = Call)[[1]], width.cutoff = 50), fill = TRUE)
}
options(width = 60)

GenDavidson <- function(win, # TRUE/FALSE
                        tie, # TRUE/FALSE
                        loss, # TRUE/FALSE
                        player1, # player1 in each contest
                        player2, # ditto player2
                        home.adv = NULL,
                        tie.max = ~1,
                        tie.mode = NULL,
                        tie.scale = NULL,
                        at.home1 = NULL,
                        at.home2 = NULL){
    call <- as.expression(sys.call()[c(1,5:6)])
    extra <- NULL
    if (is.null(tie.max)) stop("a formula must be specified for tie.max")
    if (!is.null(home.adv) & is.null(at.home1))
        stop("at.home1 and at.home2 must be specified")
    has.home.adv <- !is.null(home.adv)
    has.tie.mode <- !is.null(tie.mode)
    has.tie.scale <- !is.null(tie.scale)
    if (has.home.adv) extra <- c(extra, list(home.adv = home.adv))
    if (has.tie.mode) extra <- c(extra, list(tie.mode = tie.mode))
    if (has.tie.scale) extra <- c(extra, list(tie.scale = tie.scale))
    i <- has.home.adv + has.tie.mode + has.tie.scale
    a <- match("home.adv", names(extra), 1)
    b <- match("tie.mode", names(extra), 1)
    c <- match("tie.scale", names(extra), 1)
    adv <- has.home.adv | has.tie.mode
    list(predictors = {c(extra,
                         list(tie.max = tie.max,
                              substitute(player1), # player1 & 2 are homogeneous
                              substitute(player2)))},
         ## substitutes "result" for "outcome", but also substitutes all of code vector
         variables = {c(list(loss = substitute(loss),
                             tie = substitute(tie),
                             win = substitute(win)),
                        list(at.home1 = substitute(at.home1),
                             at.home2 = substitute(at.home2))[adv])},
         common =  c(1[has.home.adv], 2[has.tie.mode], 3[has.tie.scale], 4, 5, 5),
         term = function(predLabels, varLabels){
             if (has.home.adv) {
                 ability1 <- paste("(", predLabels[a], ") * ", varLabels[4],
                                   " + ", predLabels[i + 2], sep = "")
                 ability2 <- paste("(", predLabels[a], ") * ", varLabels[5],
                                   " + ", predLabels[i + 3], sep = "")
             }
             else {
                 ability1 <- predLabels[i + 2]
                 ability2 <- predLabels[i + 3]
             }
             tie.scale <- ifelse(has.tie.scale, predLabels[c], 0)
             scale <- paste("exp(", tie.scale, ")", sep = "")
             if (has.tie.mode) {
                 psi1 <- paste("exp((", predLabels[b], ") * ",  varLabels[4],
                               ")", sep = "")
                 psi2 <- paste("exp((", predLabels[b], ") * ",  varLabels[5],
                               ")", sep = "")
                 weight1 <- paste(psi1, "/(", psi1, " + ", psi2, ")", sep = "")
                 weight2 <- paste(psi2, "/(", psi1, " + ", psi2, ")", sep = "")
             }
             else {
                 weight1 <- weight2 <- "0.5"
             }
             nu <- paste(predLabels[i + 1], " - ", scale, " * (",
                         weight1, " * log(", weight1, ") + ",
                         weight2, " * log(", weight2, "))", sep = "")
             paste(varLabels[1], " * (", ability2, ") + ",
                   varLabels[2], " * (", nu, " + ",
                   scale, " * ", weight1, " * (", ability1, ") + ",
                   scale, " * ", weight2, " * (", ability2, ") + ",
                   "(1 - ", scale, ") * ",
                   "log(exp(", ability1, ") + exp(", ability2, "))) + ",
                   varLabels[3], " * (", ability1, ")", sep = "")
         },
         start = function(theta) {
             init <- runif(length(theta)) - 0.5
             init[c] <- 0.5
         }
         )
}
class(GenDavidson) <- "nonlin"

BTabilities <-  function (model)
{
    if (!inherits(model, "BTm"))
        stop("model is not of class BTm")

    X0 <- model.matrix(model)
    player1 <- model$player1[, model$id]
    player.names <- levels(player1)
    factors <- attr(terms(model$formula), "factors")
    if (!(model$id %in% rownames(factors))) {
        players <- data.frame(factor(seq(player.names), labels = player.names))
        names(players) <- model$id
        ## assume player covariates indexed by id
        fixed <- nobars(model$formula)
        factors <- attr(terms(fixed), "factors")
        vars <- rownames(factors)
        by.id <- grep(paste("[", model$id, "]", sep = ""), vars,
                      fixed = TRUE)
        drop <- setdiff(seq(length(vars)), by.id)
        ## following will only work for linear terms
        ## (drop any term involving non-player covariate)
        keep <- colSums(factors[drop, , drop = FALSE]) == 0
        formula <- reformulate(names(keep)[keep])
        mf <- model.frame(terms(formula), data = c(players, model$data),
                          na.action = na.pass)
        players <- players[, model$id]
        offset <- model.offset(mf)
        if (is.null(offset)) offset <- 0
        predvars <- setdiff(seq(ncol(mf)),
                            attr(attr(mf, "terms"), "offset"))
        predvars <- terms(~ . ,data = mf[, predvars, drop = FALSE])
        X <- model.matrix(predvars, mf)
        Xmiss <- is.na(rowSums(X)) |  players %in% model$separate.ability
        X[Xmiss, ] <- 0
        X <- X[, -1, drop = FALSE]
        separate.ability <- unique(union(players[Xmiss],
                                         model$separate.ability))
        ns <- length(separate.ability)
        if (ns) {
            S <- matrix(0, nrow = nrow(X), ncol = ns)
            S[cbind(which(players %in% separate.ability), seq(ns))] <- 1
            X <- cbind(S, X)
        }
        ## remove inestimable coef
        est <- !is.na(model$coef)
        X <- X[, est, drop = FALSE]
        ## keep coef of player covariates
        kept <- model$assign[est] %in% c(0, which(keep))

        sqrt.vcov <- chol(vcov(model)[kept, kept])
        V <- crossprod(sqrt.vcov %*% t(X))
        se <- sqrt(diag(V))
        abilities <- cbind(X %*% coef(model)[est][kept] + offset, se)
        attr(abilities, "vcov") <- V
        if (length(separate.ability)) {
            attr(abilities, "separate") <- separate.ability
        }
    }
    else {
        ## get ability coef and corresponding vcov
        asgn <- model$assign
        if (is.null(asgn))
            abilities <- TRUE
        else {
            idterm <- attr(terms(model$formula), "term.labels") == model$id
            if (!any(idterm))
               stop("abilities not uniquely defined for this parameterization")
            coefs.to.include <- asgn == which(idterm)
            vcov.to.include <- asgn[!is.na(coef(model))] == which(idterm)
        }
        coef <- na.exclude(coef(model)[coefs.to.include])
        vc <- vcov(model)[names(coef), names(coef), drop = FALSE]
        ## setup factor reflecting contrasts used ..
        fac <- factor(player.names, paste0(model$id, player.names))
        if (!is.null(model$refcat)) {
            fac <- C(relevel(fac, model$refcat), "contr.treatment")
        } else fac <- C(fac, model$contrasts[[model$id]])
        contr <- contrasts(fac)
        ## calc abilities and s.e., fill in NA as necessary
        if (!is.null(attr(coef, "na.action"))) {
            contr <- contr[, -attr(coef, "na.action"), drop = FALSE]
        }
        est <- contr %*% coef
        se <- sqrt(diag(contr %*% vc %*% t(contr)))
        if (!is.null(attr(coef, "na.action"))){
            id <- match(names(attr(coef, "na.action")), rownames(contr))
            est[id] <- se[id] <- NA
        }
        abilities <- cbind(est, se)
        attr(abilities, "vcov") <- vc
    }
    colnames(abilities) <- c("ability", "s.e.")
    rownames(abilities) <- player.names
    attr(abilities, "modelcall") <- model$call
    attr(abilities, "factorname") <- model$id
    class(abilities) <- c("BTabilities", "matrix")
    abilities
}

print.BTabilities <- function(x, ...) {
    attr(x, "vcov") <- attr(x, "modelcall") <- attr(x, "factorname") <- NULL
    class(x) <- "matrix"
    print(x)      ## ie, print without showing the messy attributes
}

vcov.BTabilities <- function(object, ...) {
    attr(object, "vcov")
}

coef.BTabilities <- function(object, ...) {
    object[, "ability"]
}

## some awkward cases for predict
## (in response to Arthur Spirling's bug report)

options(digits = 4) ## only applies to this file

## The final model in example(flatlizards)
library(BradleyTerry2)
data(flatlizards, package = "BradleyTerry2")
attach(flatlizards)
Whiting.model3 <- BTm(1, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
                      head.length[..] + SVL[..] + (1|..),
                      family = binomial(link = "probit"),
                      data = list(contests, predictors), trace = TRUE)


## new lizard with original lizards with NAs
newdata  <- list(contests = data.frame(winner = factor(c("lizard096", "lizard059"),
                                       levels = c("lizard048", "lizard052", "lizard096", "lizard059")),
                 loser = factor(c("lizard048", "lizard052"),
                 levels = c("lizard048", "lizard052", "lizard096", "lizard059"))),
                 predictors = rbind(flatlizards$predictors[c(27, 29, 55),-c(1,18) ],
                 c(NA, 1.5, 1.5, -.2, 3, 1, -1, -1.5, -1.5, 250, 2000, 1, 0.1, 0.2, 0.5, -0.2)))

predict(Whiting.model3, level = 1, se.fit = TRUE, newdata = newdata)

## new lizard with NAs - can't predict, go by na.action
newdata  <- list(contests = data.frame(winner = factor(c("lizard048", "lizard059"),
                                       levels = c("lizard006", "lizard011", "lizard048", "lizard059")),
                 loser = factor(c("lizard006", "lizard011"),
                 levels = c("lizard006", "lizard011", "lizard048", "lizard059"))),
                 predictors = rbind(flatlizards$predictors[c(3, 6, 27),-c(1,18) ],
                 c(NA, 1.5, 1.5, -.2, 3, 1, -1, -1.5, -1.5, 250, 2000, 1, 0.1, 0.2, 0.5, -0.2)))

predict(Whiting.model3, level = 0:1, se.fit = TRUE, newdata = newdata, na.action = na.pass)

predict(Whiting.model3, level = 0:1, se.fit = TRUE, newdata = newdata, na.action = na.omit)

## newdata = original data
tmp <- predict(Whiting.model3)
tmp2 <- predict(Whiting.model3, newdata = list(contests, predictors))
identical(tmp, tmp2)

## new data with separate effects as in original

newdata  <- list(contests = data.frame(winner = factor(c("lizard096", "lizard099"),
                                       levels = c("lizard048", "lizard052", "lizard096", "lizard099")),
                 loser = factor(c("lizard048", "lizard052"),
                 levels = c("lizard048", "lizard052", "lizard096", "lizard099"))),
                 predictors = flatlizards$predictors[c(27, 29, 55, 56),-c(1,18) ])

predict(Whiting.model3, level = 1, se.fit = TRUE, newdata = newdata)#[31, 34]

predict(Whiting.model3, level = 1, se.fit = TRUE)$fit[c(31, 34)]
predict(Whiting.model3, level = 1, se.fit = TRUE)$se.fit[c(31, 34)]

print.summary.BTglmmPQL <- function(x, digits = max(3, getOption("digits") - 3),
                                   symbolic.cor = x$symbolic.cor,
                                   signif.stars = getOption("show.signif.stars"),
                                   ...)  {
    cat("\nCall:\n", deparse(x$call), sep = "", fill = TRUE)
    p <- length(x$aliased)
    tidy.zeros <- function(vec) ifelse(abs(vec) < 100 * .Machine$double.eps,
                                       0, vec)
    if (p == 0) {
        cat("\nNo Fixed Effects\n")
    }
    else {
        if (nsingular <- p - x$rank) {
            cat("\nFixed Effects: (", nsingular,
                " not defined because of singularities)\n", sep = "")
            cn <- names(x$aliased)
            pars <- matrix(NA, p, 4, dimnames = list(cn, colnames(x$fixef)))
            pars[!x$aliased, ] <- tidy.zeros(x$fixef)
        }
        else {
            cat("\nFixed Effects:\n")
            pars <- tidy.zeros(x$fixef)
        }
        printCoefmat(pars, digits = digits, signif.stars = signif.stars,
            na.print = "NA", ...)
    }
    cat("\n(Dispersion parameter for ", x$family$family,
        " family taken to be 1)\n", sep = "")
    cat("\nRandom Effects:\n")
    pars <- tidy.zeros(x$ranef)
    printCoefmat(pars, digits = digits, signif.stars = signif.stars,
            na.print = "NA", ...)
    if (nzchar(mess <- naprint(x$na.action)))
        cat("\n", mess, "\n", sep = "")
    cat("\nNumber of iterations: ", x$iter, "\n", sep = "")
    correl <- x$correlation
    if (!is.null(correl)) {
        if (x$rank > 1) {
            cat("\nCorrelation of Coefficients:\n")
            if (is.logical(symbolic.cor) && symbolic.cor) {
                print(symnum(correl, abbr.colnames = NULL))
            }
            else {
                correl <- format(round(correl, 2), nsmall = 2,
                  digits = digits)
                correl[!lower.tri(correl)] <- ""
                print(correl[-1, -x$rank, drop = FALSE], quote = FALSE)
            }
        }
    }
    cat("\n")
    invisible(x)
}

summary.BTglmmPQL <- function(object, dispersion = NULL, correlation = FALSE,
                            symbolic.cor = FALSE, ...) {
    if (identical(object$sigma, 0)){
        cat("PQL algorithm converged to fixed effects model\n")
        return(NextMethod("summary"))
    }
    aliased <- is.na(coef(object))
    coefs <- coef(object)[!aliased]
    cov.scaled <- cov.unscaled <- object$varFix # when dispersion != 1?
    dn <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")
    if (object$rank > 0) {
        sterr <- sqrt(diag(cov.scaled))
        tvalue <- coefs/sterr
        pvalue <- 2 * pnorm(-abs(tvalue))
        fixef.table <- cbind(coefs, sterr, tvalue, pvalue)
        dimnames(fixef.table) <- list(names(coefs), dn)
    }
    else {
        fixef.table <- matrix(, 0, 4)
        dimnames(fixef.table) <- list(NULL, dn)
    }
    sterr <- sqrt(object$varSigma)
    tvalue <- object$sigma/sterr
    pvalue <- 2 * pnorm(-abs(tvalue))
    ranef.table <- cbind(object$sigma, sterr, tvalue, pvalue)
    dimnames(ranef.table) <- list("Std. Dev.", dn)
    ans <- c(object[c("call", "family", "iter", "rank", "na.action")],
             list(fixef = fixef.table, ranef = ranef.table,
                  aliased = aliased, dispersion = 1,
                  cov.unscaled = cov.unscaled))
    if (correlation & object$rank > 0) {
        dd <- sqrt(diag(cov.unscaled))
        ans$correlation <- cov.unscaled/outer(dd, dd)
        ans$symbolic.cor <- symbolic.cor
    }
    class(ans) <- "summary.BTglmmPQL"
    ans
}

drop1.BTm <- function(object, scope, scale = 0, test = c("none", "Chisq", "F"),
                      ...) {
    x <- model.matrix(object)

    ## Pass on if no random effects
    if (is.null(object$random)){
        object$x <- x
        attr(object$x, "assign") <- object$assign
        object$terms <- terms(object$formula)
        return(NextMethod())
    }

    form <- formula(object)

    if (missing(scope))
        scope <- drop.scope(nobars(form))
     else {
        if (!is.character(scope)) {
            srandom <- findbars(scope[[2]])
            if (length(srandom))
                stop("Scope should not include random effects.")
            scope <- attr(terms(update.formula(form, scope)),
                          "term.labels")
        }
        if (!all(match(scope, terms(form), 0L) > 0L))
            stop("scope is not a subset of term labels")
    }

    asgn <- object$assign

    coefs <- coef(object)
    if (scale == 0) dispersion <- 1
    else dispersion <- scale
    vc <- vcov(object, dispersion = dispersion) #vcov should deal with dispersion != 1

    sTerms <- sapply(strsplit(scope, ":", fixed = TRUE),
                     function(x) paste(sort(x), collapse = ":"))
    stat <- df <- numeric(length(scope))
    names(stat) <- names(df) <- as.character(sapply(scope, as.name))
    tryerror <- FALSE
    for (i in seq(scope)) {
        stt <- paste(sort(strsplit(scope[i], ":")[[1]]), collapse = ":")
        usex <- match(asgn, match(stt, sTerms), 0) > 0
        trystat <- try(t(coefs[usex]) %*% chol2inv(chol(vc[usex, usex])) %*%
                       coefs[usex], silent = TRUE)
        if (inherits(trystat, "try-error")) {
            stat[i] <- df[i] <- NA
            tryerror <- TRUE
        }
        else {
            stat[i] <- trystat
            df[i] <- sum(usex)
        }
    }
    table <- data.frame(stat, df)
    dimnames(table) <- list(names(df), c("Statistic", "Df"))
    title <- "Single term deletions\n"
    topnote <- gsub("\\s+", " ", paste("Model: ",
                                     paste(deparse(as.vector(formula(object))),
                                           collapse = ""),
                     if (scale > 0) paste("\nscale: ", format(scale), "\n"),
                     if (tryerror)
                     "\n\nTest statistic unestimable for at least one term"),
                    perl = TRUE)
    test <- match.arg(test)
    if (test == "Chisq") {
        dfs <- table[, "Df"]
        vals <- table[, "Statistic"]
        vals[dfs %in% 0] <- NA
        table <- cbind(table, `P(>|Chi|)` = pchisq(vals, abs(dfs),
                              lower.tail = FALSE))
    }
    else if (test == "F") {
        ## Assume dispersion fixed at one - if dispersion estimated, would use
        ## "residual" df from larger model in each comparison
        df.dispersion <- Inf
        if (df.dispersion == Inf) {
            fam <- object[[1]]$family$family
            if (fam == "binomial" || fam == "poisson")
                warning(gettextf("using F test with a '%s' family is inappropriate",
                                 fam), domain = NA, call. = FALSE)
            else warning("using F test with a fixed dispersion is inappropriate")
        }
        dfs <- table[, "Df"]
        Fvalue <- table[, "Statistic"]/abs(dfs)
        Fvalue[dfs %in% 0] <- NA
        table <- cbind(table, F = Fvalue, `Pr(>F)` =
                       pf(Fvalue, abs(dfs), df.dispersion,
                          lower.tail = FALSE))
    }
    structure(table, heading = c(title, topnote), class = c("anova",
        "data.frame"))
}

anova.BTm <- function (object, ..., dispersion = NULL, test = NULL)
{
    ## Only list models in ...
    dotargs <- list(...)
    named <- if (is.null(names(dotargs)))
        rep(FALSE, length(dotargs))
    else (names(dotargs) != "")
    if (any(named))
        warning("the following arguments to 'anova.BTm' are invalid and dropped: ",
            paste(deparse(dotargs[named]), collapse = ", "))
    dotargs <- dotargs[!named]
    is.BTm <- unlist(lapply(dotargs, function(x) inherits(x, "BTm")))
    dotargs <- dotargs[is.BTm]

    ## Compare list of models
    models <- c(list(object), dotargs)
    if (length(dotargs) > 0){
        fixed <- unlist(lapply(models, function(x) is.null(x$random)))
        if (all(fixed)) {
            variables <- lapply(models, function(x) paste(deparse(formula(x)),
                                                          collapse = "\n"))
            models <- lapply(models, function(x) {
                x$formula <- formula(x$terms)
                class(x) <- setdiff(class(x), "BTm")
                x})
            call <- match.call()
            anova.table <- do.call("anova", c(models, call$dispersion, call$test))
            attr(anova.table, "heading") <-
                c(paste("Analysis of Deviance Table\n\n",
                        "Response: ", deparse(object$call$outcome, 500), "\n", sep = ""),
                  paste("Model ", format(seq(models)), ": ", variables,
                        sep = "", collapse = "\n"))
            return(anova.table)
        }
        else
            return(anova.BTmlist(c(list(object), dotargs),
                                 dispersion = dispersion,  test = test))
    }

    X <- model.matrix(object)
    Z <- object$random

    sep <- 0 %in% object$assign

    ## Passing on to glm when no random effects
    if (is.null(Z)) {
        object$x <- X
        attr(object$x, "assign") <- object$assign + sep
        attr(object$terms, "term.labels") <- c("[sep]"[sep], object$term.labels)
        anova.table <- NextMethod()
        attr(anova.table, "heading") <-
            paste("Analysis of Deviance Table", "\n\nModel: ",
                  object$family$family, ", link: ", object$family$link,
                  "\n\nResponse: ", deparse(object$call$outcome, 500),
                  "\n\nTerms added sequentially (first to last)\n\n",
                  sep = "")
        if (sep) {
            anova.table <- anova.table[-1,]
            rownames(anova.table)[1] <- "NULL"
            anova.table[1, 1:2] <- NA
        }
        return(anova.table)
    }

    varseq <- object$assign

    nvars <- max(0, varseq)
    stat <- df <- numeric(nvars)
    tryerror <- FALSE
    if (nvars > 1) {
        y <- object$y
        ## Extension to further methods
        method <- object$method
        if (!is.function(method))
            method <- get(method, mode = "function")
        control <- object$control
        control$trace <- FALSE
        for (i in 1:(nvars - 1)) {
            fit <- method(X = X[, varseq <= i, drop = FALSE], y = y, Z = Z,
                          weights = object$prior.weights, start = object$start,
                          offset = object$offset, family = object$family,
                          control = control,
                          sigma = object$call$sigma,
                          sigma.fixed = object$sigma.fixed)
            class(fit) <- oldClass(object)
            ind <- (varseq == i)[varseq <= i]
            trystat <- try(t(coef(fit)[ind]) %*%
                           chol2inv(chol(suppressMessages(vcov(fit, dispersion = dispersion))[ind, ind])) %*%
                           coef(fit)[ind], silent = TRUE) #vcov should deal with dispersion != 1
            if (inherits(trystat, "try-error")) {
                stat[i] <- df[i] <- NA
                tryerror <- TRUE
            }
            else {
                stat[i] <- trystat
                df[i] <- sum(ind)
            }
        }
    }
    ind <- varseq == nvars
    trystat <- try(t(coef(object)[ind]) %*% chol2inv(chol(object$varFix[ind, ind])) %*%
                   coef(object)[ind], silent = TRUE)
    if (inherits(trystat, "try-error")) {
        stat[nvars] <- df[nvars] <- NA
        tryerror <- TRUE
    }
    else {
        stat[nvars] <- trystat
        df[nvars] <- sum(ind)
    }
    table <- data.frame(c(NA, stat), c(NA, df))
    dimnames(table) <- list(c("NULL", object$term.labels), c("Statistic", "Df"))
    title <- paste("Sequential Wald Tests", "\n\nModel: ",
                   object$family$family, ", link: ", object$family$link,
                   "\n\nResponse: ", deparse(object$call$outcome, 500),
                   "\n\nPredictor: ", paste(formula(object), collapse = ""),
                   "\n\nTerms added sequentially (first to last)",
                   if (tryerror)
                   "\n\nTest statistic unestimable for at least one term",
                   "\n", sep = "")

    ## Assume dispersion fixed at one - if dispersion estimated, would use
    ## "residual" df from larger model in each comparison
    df.dispersion <- Inf
    if (!is.null(test)) {
        if (test == "F" && df.dispersion == Inf) {
            fam <- object$family$family
            if (fam == "binomial" || fam == "poisson")
                warning(gettextf("using F test with a %s family is inappropriate",
                  fam), domain = NA)
            else warning("using F test with a fixed dispersion is inappropriate")
        }
        table <- switch(test, Chisq = {
            dfs <- table[, "Df"]
            vals <- table[, "Statistic"]
            vals[dfs %in% 0] <- NA
            cbind(table, `P(>|Chi|)` = pchisq(vals, dfs, lower.tail = FALSE))
        }, F = {
            dfs <- table[, "Df"]
            Fvalue <- table[, "Statistic"]/dfs
            Fvalue[dfs %in% 0] <- NA
            cbind(table, F = Fvalue, `Pr(>F)` =
                  pf(Fvalue, dfs, df.dispersion, lower.tail = FALSE))
        })
    }
    structure(table, heading = title, class = c("anova", "data.frame"))
}

anova.BTmlist <- function (object, ..., dispersion = NULL, test = NULL) {
    ## Pass on if no random effects
    fixed <- unlist(lapply(object, function(x) is.null(x$random)))
    if (!all(!fixed))
        stop("Models must have the same random effects structure")

    responses <- as.character(lapply(object, function(x) {
        deparse(formula(terms(x))[[2]])
    }))
    sameresp <- responses == responses[1]
    if (!all(sameresp)) {
        object <- object[sameresp]
        warning("models with response ", deparse(responses[!sameresp]),
                " removed because response differs from model 1")
    }
    ns <- sapply(object, function(x) length(fitted(x)))
    if (any(ns != ns[1]))
        stop("models were not all fitted to the same size of dataset")
    nmodels <- length(object)

    ncoefs <- sapply(object, function(x) length(na.omit(coef(x)))) #omit aliased
    labels <- lapply(object, function(x) x$term.labels)
    stat <- numeric(nmodels)
    for (i in 2:nmodels) {
        descending <- ncoefs[i] < ncoefs[i - 1]
        bigger <- i - descending
        smaller <- i - !descending
        if (!all(labels[[smaller]] %in% labels[[bigger]]))
            stop("models are not nested")
        ind <- !(labels[[bigger]] %in% labels[[smaller]])
        stat[i] <- t(coef(object[[bigger]])[ind]) %*%
            chol2inv(chol(vcov(object[[bigger]], dispersion = dispersion)[ind, ind])) %*%
                coef(object[[bigger]])[ind] #vcov should deal with dispersion != 1
    }
    stat[1] <- NA
    table <- data.frame(stat, c(NA, diff(ncoefs)))
    variables <- lapply(object, function(x) paste(deparse(formula(x)),
                                                  collapse = "\n"))
    dimnames(table) <- list(1:nmodels, c("Statistic", "Df"))
    title <- paste("Sequential Wald Tests\n\n",
                   "Response: ", responses[1], "\n", sep = "")
    topnote <- paste("Model ", format(1:nmodels), ": ", variables,
                     sep = "", collapse = "\n")
    if (!is.null(test)) {
        ## Assume dispersion fixed at one - if dispersion estimated, would use
        ## "residual" df from larger model in each comparison
        df.dispersion <- Inf
        if (test == "F" && df.dispersion == Inf) {
            fam <- object[[1]]$family$family
            if (fam == "binomial" || fam == "poisson")
                warning(gettextf("using F test with a '%s' family is inappropriate",
                  fam), domain = NA, call. = FALSE)
            else warning("using F test with a fixed dispersion is inappropriate")
        }
        table <- switch(test, Chisq = {
            dfs <- table[, "Df"]
            vals <- table[, "Statistic"]
            vals[dfs %in% 0] <- NA
            cbind(table, `P(>|Chi|)` = pchisq(vals, abs(dfs), lower.tail = FALSE))
        }, F = {
            dfs <- table[, "Df"]
            Fvalue <- table[, "Statistic"]/abs(dfs)
            Fvalue[dfs %in% 0] <- NA
            cbind(table, F = Fvalue, `Pr(>F)` =
                  pf(Fvalue, abs(dfs), df.dispersion, lower.tail = FALSE))
        })
    }
    structure(table, heading = c(title, topnote), class = c("anova",
        "data.frame"))
}

glmmPQL.fit <- function(X, y, Z,  weights = rep(1, NROW(y)), start = NULL,
                        etastart = NULL, mustart = NULL,
                        offset = rep(0, NROW(y)), family = gaussian(),
                        control = glmmPQL.control(...),
                        sigma = NULL, sigma.fixed = FALSE, ...) {
    matchCall <- as.list(match.call(expand.dots = FALSE))
    dots <- names(matchCall[["..."]])
    dots <- intersect(dots, names(formals(glm)))
    fit0 <- do.call("glm.fit", c(list(X, y, weights, start = start,
                                      etastart = etastart, mustart = mustart,
                                      offset = offset, family = family),
                                 matchCall[dots]))
    w <- fit0$prior.weights
    # QR missing from glm.fit if ncol(X) = 0
    QR <- qr(X)
    R <- qr.R(QR)
    rank <- QR$rank
    p <- ncol(R)
    nm <- colnames(R)[seq(length = rank)]
    if (rank < p) {
        X0 <- X[,colnames(R)[-seq(length = rank)]]
        X <- X[, nm]
    }
    empty <- !length(X)
    if (empty) {
        alpha <- numeric(0)
        Xa <- matrix(0, length(y), 1)
    }
    eta <- fit0$linear.predictors
    residuals <- fit0$residuals
    Y <- eta + residuals - offset #working response
    wy <- fit0$weights # iterative weights
    wY <- sqrt(wy) * Y
    wZ <- sqrt(wy) * Z
    ZWy <- crossprod(wZ, wY)
    ZWZ <- crossprod(wZ, wZ)
    if (!empty) {
        wX <- sqrt(wy) * X
        XWy <- crossprod(wX, wY)
        XWX <- crossprod(wX, wX)
        ZWX <- crossprod(wZ, wX)
        E <- chol(XWX)
        F <- backsolve(E, t(ZWX), transpose = TRUE)
        f <- backsolve(E, XWy, transpose = TRUE)
        ZSy <- ZWy - crossprod(F, f)
        ZSZ <- ZWZ - crossprod(F, F)
    }
    if (is.null(sigma)) sigma <- 0.1
    logtheta <- log(sigma^2)
    conv <- FALSE
    for (i in 1:control$maxiter) {
        ## Update coefficients
        for (j in 1:control$IWLSiter) {
            IZWZD <- ZWZ * sigma^2
            diag(IZWZD) <- 1 + diag(IZWZD)
            A <- chol(IZWZD)
            if (!empty) {
                IZSZD <- ZSZ * sigma^2
                diag(IZSZD) <- 1 + diag(IZSZD)
                G <- chol(IZSZD)
                g <- backsolve(G, ZSy, transpose = TRUE)
                v <- backsolve(G, g)

                B <- backsolve(A, sigma * ZWX, transpose = TRUE)
                C <- chol(XWX - crossprod(B, B))
                b <- backsolve(A, sigma * ZWy, transpose = TRUE)
                c <- backsolve(C, XWy - t(B) %*% b, transpose = TRUE)

                alpha <- backsolve(C, c)
                Xa <- X %*% alpha
                beta <- sigma^2 * v
            }
            else {
                g <- backsolve(A, ZWy, transpose = TRUE)
                v <- backsolve(A, g)

                beta <- sigma^2 * v
            }

            eta <- c(Xa + Z %*% beta + offset)

            ## Update working response & weights
            mu <- family$linkinv(eta)
            mu.eta.val <- family$mu.eta(eta)
            residuals <- (fit0$y - mu)/mu.eta.val
            Y <- eta + residuals - offset
            wy <- w * mu.eta.val^2/family$variance(mu)
            wY <- sqrt(wy) * Y
            wZ <- sqrt(wy) * Z
            ZWy <- crossprod(wZ, wY)
            ZWZ <- crossprod(wZ, wZ)

            if (!empty) {
                wX <- sqrt(wy) * X
                XWy <- crossprod(wX, wY)
                XWX <- crossprod(wX, wX)
                ZWX <- crossprod(wZ, wX)
                E <- chol(XWX)
                F <- backsolve(E, t(ZWX), transpose = TRUE)
                f <- backsolve(E, XWy, transpose = TRUE)
                ZSy <- ZWy - crossprod(F, f)
                ZSZ <- ZWZ - crossprod(F, F)

                score <- c(crossprod(X, wy * residuals),
                           crossprod(Z, wy * residuals) - v)
                diagInfo <- c(diag(XWX), diag(ZWZ))

                if (all(diagInfo < 1e-20) ||
                    all(abs(score) < control$tol * sqrt(control$tol + diagInfo))) {
                    if (sigma.fixed) conv <- TRUE
                    break
                }
            }
            else {
                score <- crossprod(Z, wy * residuals) - v
                diagInfo <- diag(ZWZ)
                if (all(diagInfo < 1e-20) ||
                    all(abs(score) < control$tol * sqrt(control$tol + diagInfo))) {
                    if (sigma.fixed) conv <- TRUE
                    break
                }
            }
        }

        if (!sigma.fixed){
            ## Update sigma
            ## sigma^2 = exp(logtheta)
            ## One Fisher scoring iteration
            IZWZD <- ZWZ * sigma^2
            diag(IZWZD) <- 1 + diag(IZWZD)
            A <- chol(IZWZD)
            if (!empty) {
                IZSZD <- ZSZ * sigma^2
                diag(IZSZD) <- 1 + diag(IZSZD)
                G <- chol(IZSZD)
                g <- backsolve(G, ZSy, transpose = TRUE)
                v <- backsolve(G, g)
                h <- backsolve(G, ZSZ, transpose = TRUE)
                H <- backsolve(G, h)
            }
            else {
                g <- backsolve(A, ZWy, transpose = TRUE)
                v <- backsolve(A, g)
                h <- backsolve(A, ZWZ, transpose = TRUE)
                H <- backsolve(A, h)
            }

            ## Harville p326
            score <- drop(-0.5 * sum(diag(H)) + 0.5 * crossprod(v, v)) *
                sigma^2
            Info <- 0.5 * sum(H^2) * sigma^4

             if (control$trace) {
                 ##B & C eq 5 - still not consistently increasing
                 cat("Iteration ", i,
                     ". Score = ", abs(score) ,
                     "\n", sep = "")
                 flush.console()
            }

            ## check for overall convergence
            if (Info < 1e-20 ||
                abs(score) < control$tol * sqrt(control$tol + Info)){
                conv <- TRUE
                break
            }

            ## Cannot use beta to update t(YXa) %*% Vinv %*% YXa
            ZWYXa <- crossprod(wZ, sqrt(wy) * (Y - Xa))
            optfun <- function(logtheta) {
                IZWZD <- ZWZ * exp(logtheta)
                diag(IZWZD) <- 1 + diag(IZWZD)
                A <- chol(IZWZD)
                if (!empty) {
                    IZSZD <- ZSZ * exp(logtheta)
                    diag(IZSZD) <- 1 + diag(IZSZD)
                    G <- chol(IZSZD)
                    d <- backsolve(A, sqrt(exp(logtheta)) * ZWYXa, transpose = TRUE)
                    sum(log(diag(G))) - 0.5 * crossprod(d, d)
                }
                else {
                    d <- backsolve(A, sqrt(exp(logtheta)) * ZWy, transpose = TRUE)
                    sum(log(diag(A))) - 0.5 * crossprod(d, d)
                }
            }
            optres <- optimize(optfun, c(-10, 10))
            if (optfun(-10) < optfun(optres$minimum))
                sigma <- 0
            else {
                if (abs(optres$minimum - (logtheta + score/Info)) > 0.1)
                    logtheta <- optres$minimum
                else
                    logtheta <- logtheta + score/Info
                sigma <- sqrt(exp(logtheta))
            }
        }
        else if (conv)
            break
    }
    if (!empty) varFix <- chol2inv(C)
    else varFix <- matrix(, 0, 0)
    rownames(varFix) <- colnames(varFix) <- colnames(X)
    fit0$coef[nm] <- alpha
    if (!sigma.fixed)
        varSigma <- sigma^2/(4 * Info)
    else
        varSigma <- NA
    glm <- identical(sigma, 0)
    if (!empty) {
        if (rank < p) QR <- qr(cbind(wX, sqrt(w) * X0))
        else QR <- qr(wX)
        R <- qr.R(QR)
    }
    list(coefficients = structure(fit0$coef, random = beta),
         residuals = residuals,
         fitted.values = mu,
         #effect = ?
         R = if (!empty) R,
         rank = rank,
         qr = if (!empty) QR,
         family = family,
         linear.predictors = eta,
         deviance = if (glm) sum(family$dev.resids(y, mu, w)),
         aic = if (glm)
         family$aic(y, length(y), mu, w, sum(family$dev.resids(y, mu, w))) + 2 * rank,
         null.deviance = if (glm) {
             wtdmu <- family$linkinv(offset)
             sum(family$dev.resids(y, wtdmu, w))
         },
         iter = ifelse(glm, NA, i),
         weights = wy,
         prior.weights = w,
         df.residual = length(y) - rank,
         df.null = if (glm) length(y) - sum(w == 0),
         y = y,
         sigma = sigma, sigma.fixed = sigma.fixed,
         varFix = varFix, varSigma = varSigma, converged = conv)
}

BTm.setup <- function(outcome = 1, player1, player2, formula = NULL,
                            id = "..", separate.ability = NULL, refcat = NULL,
                            data = NULL, weights = NULL, subset = NULL,
                            offset = NULL, contrasts = NULL, ...){
    if (!is.data.frame(data)){
        keep <- names(data) %in% c(deparse(substitute(player1)),
                                   deparse(substitute(player2)))
        if (!length(keep)) keep <- FALSE
        ## save row names for checking against index variables (in Diff)
        nm <- lapply(data, rownames)
        data <- c(data[keep], unlist(unname(data[!keep]), recursive = FALSE))
        if (any(dup <- duplicated(names(data))))
            warning("'data' argument specifies duplicate variable names: ",
                    paste(names(data)[dup], collapse = " "))
    }
    ## (will take first occurence of replicated names)
    withIfNecessary <- function(x, data, as.data.frame = TRUE) {
        if (as.data.frame)
            expr <- substitute(data.frame(x), list(x = x))
        else expr <- x
        if (!is.null(data))
            with(data, eval(expr))
        else eval(expr)
    }
    player1 <- withIfNecessary(substitute(player1), data)
    player2 <- withIfNecessary(substitute(player2), data)
    if (ncol(player1) == 1) colnames(player1) <- colnames(player2) <- id
    Y <- withIfNecessary(substitute(outcome), c(player1, player2, data),
                         as.data.frame = FALSE)
    weights <- withIfNecessary(substitute(weights), data, FALSE)
    subset1 <- withIfNecessary(substitute(subset),
                               c(player1 = list(player1),
                                 player2 = list(player2), player1, data), FALSE)
    subset2 <- withIfNecessary(substitute(subset),
                               c(player1 = list(player1),
                                 player2 = list(player2), player2, data), FALSE)
    if (is.logical(subset1)) subset <- subset1 | subset2
    else subset <- c(subset1, subset2)
    if (is.null(formula)) formula <- reformulate(id)
    diffModel <- Diff(player1, player2, formula, id, data, separate.ability,
                      refcat, contrasts, nm)
    offset <- withIfNecessary(substitute(offset), data, FALSE) #contest level
    if (!is.null(offset)) {
        if (is.null(diffModel$offset))  diffModel$offset <- offset
        else diffModel$offset <- diffModel$offset + offset
    }
    res <- c(diffModel, list(data = data, player1 = player1, player2 = player2,
                             Y = Y, weights = weights, subset = subset, formula = formula))
}

BTm <- function(outcome = 1, player1, player2, formula = NULL,
                id = "..", separate.ability = NULL, refcat = NULL,
                family = binomial, data = NULL, weights = NULL, subset = NULL,
                na.action = NULL, start = NULL, etastart = NULL, mustart = NULL,
                offset = NULL, br = FALSE, model = TRUE, x = FALSE,
                contrasts = NULL, ...){
    call <- match.call()

    if (is.character(family))
        family <- get(family, mode = "function", envir = parent.frame())
    if (is.function(family))
        family <- family()
    if (is.null(family$family)) {
        print(family)
        stop("`family' not recognized")
    }
    if (family$family != "binomial")
        stop("`family' must be binomial")
    if (!family$link %in% c("logit", "probit", "cauchit"))
        stop("link for binomial family must be one of \"logit\", \"probit\"",
             "or \"cauchit\"")
    fcall <- as.list(match.call(expand.dots = FALSE))
    setup <- match(c("outcome", "player1", "player2", "formula", "id",
                     "separate.ability", "refcat", "data", "weights",
                     "subset", "offset", "contrasts"), names(fcall), 0L)
    if (is.null(formula)) env <- parent.frame()
    else env <- environment(formula)
    setup <- do.call(BTm.setup, fcall[setup], envir = env)
    if (setup$saturated)
        warning("Player ability saturated - equivalent to fitting ",
                "separate abilities.")
    mf <- data.frame(X = setup$player1) #just to get length
    if (!is.null(setup$X)) {
        mf$X <- setup$X
        formula <- Y ~ X - 1
    }
    else formula <- Y ~ 0
    mf$Y <- setup$Y
    argPos <- match(c("na.action", "start", "etastart",
                      "mustart", "control", "model", "x"), names(fcall), 0)
    dotArgs <- fcall$"..."
    if (is.null(setup$random)) {
        method <- get(ifelse(br, "brglm", "glm"), mode = "function")
        fit <- as.call(c(method, fcall[argPos],
                         list(formula = formula, family = family, data = mf,
                              offset = setup$offset, subset = setup$subset,
                              weights = setup$weights), dotArgs))
        fit <- eval(fit, parent.frame())
    }
    else {
        method <- get("glmmPQL", mode = "function")
        fit <- as.call(c(method, fcall[argPos],
                         list(formula, setup$random, family = family,
                              data = mf, offset = setup$offset,
                              subset = setup$subset, weights = setup$weights), dotArgs))
        fit <- eval(fit, parent.frame())
        if (br) {
            if (identical(fit$sigma, 0)){
                argPos <- match(c("na.action", "model", "x"), names(fcall), 0)
                method <- get("brglm", mode = "function")
                fit <- as.call(c(method, fcall[argPos],
                                 list(formula, family = family, data = mf,
                                      offset = setup$offset,
                                      subset = setup$subset,
                                      weights = setup$weights,
                                      etastart = fit$linear.predictors)))
                fit <- eval(fit, parent.frame())
                fit$class <- c("glmmPQL", class(fit))
            }
            else
                warning("'br' argument ignored for models with random effects",
                        call. = FALSE)
        }
    }
    if (length(fit$coefficients)) {
        if (ncol(setup$X) > 1)
            names(fit$coefficients) <- substring(names(fit$coefficients), 2)
        else
            names(fit$coefficients) <- colnames(setup$X)
        fit$assign <- attr(setup$X, "assign")
    }
    fit$call <- call
    fit$id <- id
    fit$separate.ability <- separate.ability
    fit$contrasts <- setup$contrasts
    fit$refcat <- setup$refcat
    fit$formula <- setup$formula
    fit$player1 <- setup$player1
    fit$player2 <- setup$player2
    fit$term.labels <- setup$term.labels
    fit$data <- setup$data
    fit$random <- setup$random
    class(fit) <- c("BTm", class(fit))
    fit
}

Diff <- function(player1, player2, formula = NULL, id = "..", data = NULL,
                 separate.ability = NULL, refcat = NULL, contrasts = NULL,
                 subset = NULL) {
    player.one <- player1[[id]]
    player.two <- player2[[id]]

    if (!is.factor(player.one) || !is.factor(player.two) ||
        !identical(levels(player.one), levels(player.two)))
        stop("'player1$", id, "' and 'player2$", id,
             "' must be factors with the same levels")
    if (!identical(attr(player.one, "contrasts"),
                   attr(player.two, "contrasts")))
        stop("'player1$", id, "' and 'player2$", id,
             "' must have the same contrasts attribute")
    if(is.null(formula)) formula <- reformulate(id)

    players <- levels(player.one)
    nplayers <- nlevels(player.one)
    ncontests <- length(player.one)
    D <- matrix(nrow = ncontests, ncol = nplayers)
    D <- col(D) == as.numeric(player.one)
    D <- D - (col(D) == as.numeric(player.two))
    colnames(D) <- paste(id, players, sep = "")

    fixed <- nobars(formula)
    X <- offset <- missing <- term.labels <- NULL
    saturated <- FALSE
    sep <- list()
    empty <- is.null(fixed) || is.empty.model(mt <- terms(fixed))
    if (!empty) {
        factors <- attr(mt, "factors")
        term.labels <- as.character(colnames(factors))
        vars <- rownames(factors)
        indexed <- grep("[[][^],]+[],]", vars)
        if (length(indexed)) { #set NAs to zero
            indices <- gsub("[^[]*[[]([^],]+)[],].*", "\\1", vars[indexed])
            vars <-  gsub("[[][^]]*[]]", "", vars[indexed])
            ## assumes no overlap, e.g. no age[..]:judge.gender[judge]
            grp <- split(vars, indices)
            for (ind in names(grp)) {
                vars <- model.frame(terms(reformulate(grp[[ind]])),
                                    data = data, na.action = na.pass)
                lev <- levels(eval(as.name(ind), c(player1, data)))
                as.sep <- rowSums(is.na(vars)) | lev %in% separate.ability
                if (any(as.sep)) {
                    sep[[ind]] <- as.sep
                    vars[sep[[ind]], ] <- lapply(vars, function(x)
                                                 max(levels(x)[1], 0))
                    colnames(vars) <- gsub(".*[$[],? ?\"?([^]\"]*).*", "\\1",
                                           grp[[ind]])
                    labels <- gsub("([^[$]*)[[$].*", "\\1", grp[[ind]])
                    for (lab in intersect(labels, grp[[ind]]))
                        data[lab] <- vars[lab]
                    for (lab in setdiff(labels, grp[[ind]]))
                        data[[lab]] <- vars[, labels == lab, drop = FALSE]
                }
            }
            if (length(sep)) {
                fixed <- reformulate(c(names(sep), attr(mt, "term.labels"),
                                       rownames(attr(mt, "factors"))[attr(mt, "offset")]))
                mt <- terms(fixed)
            }
        }

        idterm <- id %in% rownames(attr(mt, "factors"))
        mf1 <- model.frame(mt, data = c(player1, data), na.action = na.pass)
        if (nrow(mf1) != ncontests)
            stop("Predictor variables are not of the correct length --",
                 "they probably need indexing in 'formula'.")
        mf2 <- model.frame(mt, data = c(player2, data), na.action = na.pass)
        if (idterm){
            if (!is.null(refcat)) {
                mf1[[id]] <- relevel(mf1[[id]], refcat)
                mf2[[id]] <- relevel(mf2[[id]], refcat)
                if (!missing(contrasts)) contrasts[[id]] <- "contr.treatment"
            } else {
                ## 'else' defined by contrasts arg/contrasts attr of id factor
                ## leave refcat NULL
                if (is.null(contrasts))
                    contrasts[[id]] <- attr(player.one, "contrasts")
            }
        }
        offset <- model.offset(mf1)
        if (!is.null(offset)) offset <- offset - model.offset(mf2)

        if (length(sep)){ #create separate effect factor
            recode <- function(x, keep){
                lev <- levels(x)
                ext <- make.unique(c(lev[keep], "nosep"))[sum(keep) + 1]
                levels(x)[!keep]  <- ext
                relevel(x, ref = ext)
            }
            for (ind in names(grp)) {
                mf1[ind] <- recode(mf1[[ind]], sep[[ind]])
                mf2[ind] <- recode(mf2[[ind]], sep[[ind]])
            }
        }

        X1 <- model.matrix(fixed, mf1, contrasts = contrasts)
        X2 <- model.matrix(fixed, mf2, contrasts = contrasts)
        X <- X1 - X2
        ## will need to check for saturation in each set of indexed var
        ## - however as only allowing (1|..) just consider player id for now

        saturated <- qr(na.omit(X))$rank == qr(na.omit(cbind(D, X)))$rank && !idterm
        if (all(X[,1] == 0)) X <- X[, -1, drop = FALSE]
        attr(X, "assign") <- attr(X1, "assign")[-1]
    }

    random <- findbars(formula[[2]])
    if (!is.null(random)) {
        if (!is.list(random)) random <- list(random)
        if (length(random) > 1 ||
            random[[1]] != parse(text = paste("1|", id, sep = ""))[[1]])
            stop("Currently '(1 | ", id, ")' is the only random effects",
                 "structure allowed.")
        random <- D
    }
    else if (!empty && (!idterm & !saturated))
        warning("Ability modelled by predictors but no random effects",
                call. = FALSE)

    if (length(sep)) {
        attr(X, "assign") <- attr(X, "assign") - 1
        if (!is.null(random))
            random <- D[,!sep[[id]], drop = FALSE]
    }
    list(X = X, random = random, offset = offset,
         term.labels = term.labels, refcat = refcat, contrasts = contrasts,
         saturated = saturated)
}

model.frame.BTm <- function (formula, ...)
{
    dots <- list(...)
    nargs <- dots[match(c("outcome", "player1", "player2",
                          "separate.ability", "refcat", "data", "weights",
                          "subset", "offset", "contrasts"), names(dots), 0L)]
    mfArgs <- dots[match(c("na.action", "start", "etastart", "mustart"),
                         names(dots), 0L)]
    if (length(nargs) || is.null(formula$model)) {
        fcall <- formula$call[-1]
        fcall[names(nargs)] <- nargs
        env <- environment(formula$terms)
        if (is.null(env))
            env <- parent.frame()
        setup <- do.call(BTm.setup, fcall, envir = env)
        mf <- data.frame(X = setup$X[,1])
        mf$X <- setup$X
        mf$Y <- setup$Y
        mf <- as.call(c(model.frame, mfArgs,
                         list(formula = Y ~ X - 1, data = mf,
                              offset = setup$offset,
                              subset = setup$subset,
                              weights = setup$weights)))
        eval(mf, parent.frame())
    }
    else formula$model
}

print.BTglmmPQL <- function (x, digits = max(3, getOption("digits") - 3), ...)
{
    if (identical(x$sigma, 0)){
        cat("PQL algorithm converged to fixed effects model\n")
        return(NextMethod())
    }
    cat("\nCall: ", deparse(x$call), "\n", sep = "", fill = TRUE)
    if (length(coef(x))) {
        cat("Fixed effects:\n\n")
        print.default(format(x$coefficients, digits = digits),
            print.gap = 2, quote = FALSE)
    }
    else cat("No fixed effects\n\n")
    cat("\nRandom Effects Std. Dev.:", x$sigma, "\n")
    if (nzchar(mess <- naprint(x$na.action)))
        cat("\n", mess, "\n", sep = "")
}

countsToBinomial <- function(xtab) {
    ## make square if necessary
    if (nrow(xtab) != ncol(xtab) || rownames(xtab) != colnames(xtab)) {
        dat <- as.data.frame(xtab)
        lev <- union(rownames(xtab), colnames(xtab))
        dat[,1] <- factor(dat[,1], levels = lev)
        dat[,2] <- factor(dat[,2], levels = lev)
        xtab <- tapply(dat[,3], dat[1:2], sum)
        xtab[is.na(xtab)] <- 0
    }
    ##assumes square
    players <- rownames(xtab)
    comb <- combinations(nrow(xtab), 2)
    won <- xtab[comb]
    lost <- t(xtab)[comb]
    res <- !(won == 0 & lost == 0)
    player1 <- factor(players[comb[,1]], levels = players)[res]
    player2 <- factor(players[comb[,2]], levels = players)[res]
    data.frame(player1, player2, win1 = won[res], win2 = lost[res])
}

options(digits = 4) ## only applies to this file
library(BradleyTerry2)
data(flatlizards, package = "BradleyTerry2")
##
##  Fit the standard Bradley-Terry model, using the bias-reduced
##  maximum likelihood method:
##
attach(flatlizards)
result <- rep(1, nrow(contests))
BTmodel <- BTm(result, winner, loser, br = TRUE, data = contests)
summary(BTmodel)
##
##  That's fairly useless, though, because of the rather small
##  amount of data on each lizard.  And really the scientific
##  interest is not in the abilities of these particular 77
##  lizards, but in the relationship between ability and the
##  measured predictor variables.
##
##  So next fit (by maximum likelihood) a "structured" B-T model in
##  which abilities are determined by a linear predictor.
##
##  This reproduces results reported in Table 1 of Whiting et al. (2006):
##
Whiting.model <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
                     head.length[..] + SVL[..], family = binomial,
                     data = list(contests, predictors))
summary(Whiting.model)
##
##  Equivalently, fit the same model using glmmPQL:
##
Whiting.model <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
                     head.length[..] + SVL[..] + (1|..), sigma = 0,
                     sigma.fixed = TRUE, data = list(contests, predictors))
summary(Whiting.model)
##
##  But that analysis assumes that the linear predictor formula for
##  abilities is _perfect_, i.e., that there is no error in the linear
##  predictor.  This will always be unrealistic.
##
##  So now fit the same predictor but with a normally distributed error
##  term --- a generalized linear mixed model --- by using the BTm
##  function instead of glm.
##
Whiting.model2 <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
                      head.length[..] + SVL[..] + (1|..),
                      data = list(contests, predictors), trace = TRUE)
summary(Whiting.model2)
##
##  The estimated coefficients (of throat.PC1, throat.PC3,
##  head.length and SVL are not changed substantially by
##  the recognition of an error term in the model; but the estimated
##  standard errors are larger, as expected.  The main conclusions from
##  Whiting et al. (2006) are unaffected.
##
##  With the normally distributed random error included, it is perhaps
##  at least as natural to use probit rather than logit as the link
##  function:
##
Whiting.model3 <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
                      head.length[..] + SVL[..] + (1|..),
                      family = binomial(link = "probit"),
                      data = list(contests, predictors), trace = TRUE)
summary(Whiting.model3)
BTabilities(Whiting.model3)
residuals(Whiting.model3, "grouped")
##  Note the "separate" attribute here, identifying two lizards with
##  missing values of at least one predictor variable
##
##  Modulo the usual scale change between logit and probit, the results
##  are (as expected) very similar to Whiting.model2.

library(BradleyTerry2)
data(flatlizards, package = "BradleyTerry2")

attach(flatlizards)
result <- rep(1, nrow(contests))
BTmodel1 <- BTm(result, winner, loser,
                ~ throat.PC1[..] + throat.PC3[..] + (1|..),
                data = list(contests, predictors),
                tol = 1e-4, sigma = 2, trace = TRUE)

drop1(BTmodel1)

add1(BTmodel1, ~ . + head.length[..] + SVL[..], test = "Chisq")

BTmodel2 <- update(BTmodel1, formula = ~ . + head.length[..])

drop1(BTmodel2, test = "Chisq")

predict.BTglmmPQL <- function (object, newdata = NULL, newrandom = NULL,
                             level = 1, type = c("link", "response", "terms"),
                             se.fit = FALSE, terms = NULL,
                             na.action = na.pass, ...) {
    ## only pass on if a glm
    if (object$sigma == 0) {
        if (level != 0) warning("Fixed effects model: setting level to 0")
        return(NextMethod())
    }
    if (!all(level %in% c(0, 1))) stop("Only level %in% c(0, 1) allowed")
    type <- match.arg(type)
    if (!is.null(newdata) || type == "terms") tt <- terms(object)
    if (!is.null(newdata)) {
        ## newdata should give variables in terms formula
        Terms <- delete.response(tt)
        m <- model.frame(Terms, newdata, na.action = na.action,
                         xlev = object$xlevels)
        na.action <- attr(m, "na.action")
        if (!is.null(cl <- attr(Terms, "dataClasses")))
            .checkMFClasses(cl, m)
        D <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
        np <- nrow(D) # n predictions
        offset <- rep(0, np)
        if (!is.null(off.num <- attr(tt, "offset")))
            for (i in off.num) offset <- offset + eval(attr(tt,
                "variables")[[i + 1]], newdata)
        if (!is.null(object$call$offset))
            offset <- offset + eval(object$call$offset, newdata)
    }
    else {
        D <- model.matrix(object)
        newrandom <- object$random
        na.action <- object$na.action
        offset <- object$offset
    }
    if (se.fit == TRUE) {
        sigma <- object$sigma
        w <- sqrt(object$weights)
        if (!is.null(newdata)) {
            wX <- w * model.matrix(object)
            wZ <- w * object$random
        }
        else {
            wX <- w * D
            wZ <- w * newrandom
        }
        XWX <- crossprod(wX)
        XWZ <- crossprod(wX, wZ)
        ZWZ <- crossprod(wZ, wZ)
        diag(ZWZ) <- diag(ZWZ) + 1/sigma^2
        C <- cbind(XWX, XWZ)
        C <- chol(rbind(C, cbind(t(XWZ), ZWZ)))
        if (type == "terms" || level == 0){
            ## work out (chol of inverse of) topleft of C-inv directly
            A <- backsolve(chol(ZWZ), t(XWZ), transpose = TRUE)
            A <- chol(XWX - t(A) %*% A)
        }
    }
    if (type == "terms") { # ignore level
        if (1 %in% level)
            warning("type = \"terms\": setting level to 0", call. = FALSE)
        coef <- coef(object) #fixef
        aa <- attr(D, "assign")
        ll <- attr(tt, "term.labels")
        if (!is.null(terms)) {
            include <- ll %in% terms
            ll <- ll[include]
        }
        hasintercept <- attr(tt, "intercept") > 0L
        if (hasintercept) {
            avx <- colMeans(model.matrix(object))
            termsconst <- sum(avx * coef) #NA coefs?
            D <- sweep(D, 2, avx)
        }
        pred0 <- matrix(ncol = length(ll), nrow = NROW(D))
        colnames(pred0) <- ll
        if (se.fit) {
            A <- chol2inv(A)
            se.pred0 <- pred0
        }
        for (i in seq(length.out = length(ll))){
            ind <- aa == which(attr(tt, "term.labels") == ll[i])
            pred0[, i] <- D[, ind, drop = FALSE] %*% coef[ind]
            if (se.fit) {
                se.pred0[, i] <-  sqrt(diag(D[, ind] %*%
                                            tcrossprod(A[ind, ind], D[, ind])))
            }
        }
        if (hasintercept) attr(pred0, "constant") <- termsconst
        if (se.fit) return(list(fit = pred0, se.fit = se.pred0))
        return(pred0)
    }
    if (0 %in% level) {
        pred0 <- napredict(na.action, c(D %*% coef(object)) + offset)
        if (type == "response")
            pred0 <- family(object)$linkinv(pred0)
        if (se.fit == TRUE) {
            na.act <- attr(na.exclude(pred0), "na.action")
            H <- backsolve(A, t(na.exclude(D)), transpose = TRUE)
            ## se.pred0 <- sqrt(diag(D %*% chol2inv(C)[1:ncol(D), 1:ncol(D)] %*% t(D)))
            se.pred0 <- napredict(na.action, napredict(na.act, sqrt(colSums(H^2))))
           if (type == "response")
               se.pred0 <- se.pred0*abs(family(object)$mu.eta(pred0))
            pred0 <- list(fit = pred0, se.fit = se.pred0)
        }
        if (identical(level, 0)) return(pred0)
    }

    r <- nrow(D)
    ## newrandom should give new design matrix for original random effects
    if (!is.null(newdata)){
        if(is.null(newrandom))
            stop("newdata specified without newrandom")
        if (!is.null(na.action))
            newrandom <- newrandom[-na.action, , drop = FALSE]
    }
    if (!identical(dim(newrandom), c(r, ncol(object$random))))
        stop("newrandom should have ", r, " rows and ",
             ncol(object$random), " columns")
    D <- cbind(D, newrandom)
    coef <- c(coef(object), attr(coef(object), "random"))
    pred <- napredict(na.action, c(D %*% coef) + offset)
    if (type == "response")
        pred <- family(object)$linkinv(pred)
    if (se.fit == TRUE) {
        ##se.pred <- sqrt(diag(D %*% chol2inv(C) %*% t(D)))
        na.act <- attr(na.exclude(pred), "na.action")
        H <- backsolve(C, t(na.exclude(D)), transpose = TRUE)
        se.pred <- napredict(na.action, napredict(na.act, sqrt(colSums(H^2))))
        if (type == "response")
            se.pred <- se.pred*abs(family(object)$mu.eta(pred))
        pred <- list(fit = pred, se.fit = se.pred)
    }

    if (0 %in% level)
        list(population = pred0, individual = pred)
    else pred
}

predict.BTm <- function (object, newdata = NULL, level = 1,
                         type = c("link", "response", "terms"), se.fit = FALSE,
                         dispersion = NULL, terms = NULL,
                         na.action = na.pass, ...) {
    type <- match.arg(type)
    if (!is.null(newdata)) {
        ## need to define X so will work with model terms
        setup <- match(c("player1", "player2", "formula", "id",
                        "separate.ability", "refcat", "weights",
                        "subset", "offset", "contrasts"), names(object$call), 0L)
        setup <- do.call(BTm.setup,
                         c(as.list(object$call)[setup], list(data = newdata)),
                         envir = environment(object$formula))
        nfix <- length(object$coefficients)
        newdata <- data.frame(matrix(, nrow(setup$X), 0))
        keep <- match(names(object$coefficients), colnames(setup$X),
                      nomatch = 0)
        if (0 %in% keep){
            ## new players with missing data - set to NA
            missing <- rowSums(setup$X[,-keep, drop = FALSE]) != 0
            setup$X <- setup$X[, keep]
            setup$X[missing,] <- NA
        }
        if (ncol(setup$X) != nfix) {
            ## newdata does not include original players with missing data
            X <- matrix(0, nrow(setup$X), nfix,
                        dimnames = list(rownames(setup$X),
                        names(object$coefficients)))
            X[, colnames(setup$X)] <- setup$X
            newdata$X <- X
        }
        else newdata$X <- setup$X
        nran <- length(attr(object$coefficients, "random"))
        if (1 %in% level && type != "terms"){
            if (ncol(setup$random) != nran) {
                ## expand to give col for every random effect
                Z <- matrix(0, nrow(setup$random), nran,
                            dimnames = list(rownames(setup$random),
                            colnames(object$random))) #ranef need names!!
                ## set to NA for contests with new players (with predictors present)
                miss <- !colnames(setup$random) %in% colnames(Z)
                Z[, colnames(setup$random)[!miss]] <- setup$random[,!miss]
                if (any(miss)) {
                    miss <- rowSums(setup$random[, miss, drop = FALSE] != 0) > 0
                    Z[miss,] <- NA
                }
                newrandom <- Z
            }
            else newrandom <- setup$random
            return(NextMethod(newrandom = newrandom))
        }
    }
    if (type == "terms") {
        object$x <- model.matrix(object)
        attr(object$x, "assign") <- object$assign
        id <- unique(object$assign)
        terms <- paste("X", id, sep = "")
        object$terms <- terms(reformulate(c(0, terms)))
        splitX <- function(X) {
            newdata <- data.frame(matrix(, nrow(X), 0))
            for (i in seq(id))
                newdata[terms[i]] <- X[,object$assign == id[i]]
            newdata
        }
        if (is.null(newdata)) newdata <- splitX(object$x)
        else newdata <- splitX(newdata$X)
        tmp <- NextMethod(newdata = newdata)
        #tmp$fit[tmp$se.fit == 0] <- NA
        tmp$se.fit[tmp$se.fit == 0] <- NA
        colnames(tmp$fit) <- colnames(tmp$se.fit) <-
            c("(separate)"[0 %in% id], object$term.labels)
        return(tmp)
    }
    else NextMethod()
}

glmmPQL <- function(fixed, random = NULL, family = binomial, data = NULL,
                    subset = NULL, weights = NULL, offset = NULL,
                    na.action = NULL,  start = NULL, etastart = NULL,
                    mustart = NULL, control = glmmPQL.control(...),
                    sigma = 0.1, sigma.fixed = FALSE, model = TRUE,
                    x = FALSE, contrasts = NULL, ...) {
    call <-  match.call()
    nm <- names(call)[-1]

    if (is.null(random)) {
        keep <- is.element(nm, c("family", "data", "subset", "weights",
                                 "offset", "na.action"))
        for (i in nm[!keep]) call[[i]] <- NULL
        call$formula <- fixed
        environment(call$formula) <- environment(fixed)
        call[[1]] <- as.name("glm")
        return(eval.parent(call))
    }

    modelTerms <- terms(fixed, data = data)
    modelCall <- as.list(match.call(expand.dots = FALSE))
    argPos <- match(c("data", "subset", "na.action", "weights", "offset"),
                    names(modelCall), 0)
    modelData <- as.call(c(model.frame, list(formula = modelTerms,
                                             drop.unused.levels = TRUE),
                           modelCall[argPos]))
    modelData <- eval(modelData, parent.frame())

    if (!is.null(modelCall$subset))
        Z <- random[eval(modelCall$subset, data, parent.frame()),]
    else Z <- random

    if (!is.null(attr(modelData, "na.action")))
        Z <- Z[-attr(modelData, "na.action"),]

    nObs <- nrow(modelData)
    y <- model.response(modelData, "numeric")
    if (is.null(y))
        y <- rep(0, nObs)
    weights <- as.vector(model.weights(modelData))
    if (!is.null(weights) && any(weights < 0))
        stop("negative weights are not allowed")
    if (is.null(weights))
        weights <- rep.int(1, nObs)
    offset <- as.vector(model.offset(modelData))
    if (is.null(offset))
        offset <- rep.int(0, nObs)

    if (is.character(family))
        family <- get(family, mode = "function", envir = parent.frame())
    if (is.function(family))
        family <- family()
    if (is.null(family$family)) {
        print(family)
        stop("`family' not recognized")
    }
    if (family$family == "binomial") {
        if (is.factor(y) && NCOL(y) == 1)
            y <- y != levels(y)[1]
        else if (NCOL(y) == 2) {
            n <- y[, 1] + y[, 2]
            y <- ifelse(n == 0, 0, y[, 1]/n)
            weights <- weights * n
        }
    }

    ## Use GLM to estimate fixed effects
    empty <- is.empty.model(modelTerms)
    if (!empty)
        X <- model.matrix(fixed, data = modelData, contrasts)
    else
        X <- matrix(, nObs, 0)
    fit <- glmmPQL.fit(X = X, y = y, Z = Z, weights = weights, start = start,
                       etastart = etastart, mustart = mustart, offset = offset,
                       family = family, control = control, sigma = sigma,
                       sigma.fixed = sigma.fixed, ...)
    if (sum(offset) && attr(modelTerms, "intercept") > 0) {
        fit$null.deviance <- glm.fit(x = X[, "(Intercept)", drop = FALSE],
            y = y, weights = weights, offset = offset, family = family,
            control = control, intercept = TRUE)$deviance
    }
    if (model)
        fit$model <- modelData
    fit$na.action <- attr(modelData, "na.action")
    if (x)
        fit$x <- X
    fit <- c(fit, list(call = call, formula = fixed, random = random,
                       terms = modelTerms,
                       data = data, offset = offset, control = control,
                       method = "glmmPQL.fit", contrasts = attr(X, "contrasts"),
                       xlevels = .getXlevels(modelTerms, modelData)))
    class(fit) <- c("BTglmmPQL", "glm", "lm")
    fit
}

add1.BTm <- function(object, scope, scale = 0, test = c("none", "Chisq", "F"),
                      x = NULL, ...) {
    old.form <- formula(object)
    new.form <- update.formula(old.form, scope)

    if (!is.character(scope)){
        orandom <- findbars(old.form[[2]])
        srandom <- findbars(new.form[[2]])
        if (length(srandom) && !identical(orandom, srandom))
            stop("Random effects structure of object and scope must be identical.")
        scope <- add.scope(old.form, new.form)
    }
    if (!length(scope))
        stop("no terms in scope for adding to object")

    if (is.null(x)) { # create model.matrix for maximum scope
        model <- Diff(object$player1, object$player2, new.form, object$id,
                      object$data, object$separate.ability, object$refcat)
        if (sum(model$offset) > 0)
            warning("ignoring offset terms in scope")
        x <- model$X
        asgn <- attr(x, "assign")
        ## add dummy term for any separate effects
        oTerms <- c("sep"[0 %in% asgn], object$term.labels)
        object$terms <- terms(reformulate(oTerms))
        y <- object$y
        dummy <- y ~ x - 1
        if (!is.null(model$random)) {
            dummy <- update(dummy, .~ . + Z)
            Z <- model$random
        }
        argPos <- match(c("weights", "subset", "na.action"),
                        names(object$call), 0)
        mf <- as.call(c(model.frame, as.list(object$call)[argPos],
                       list(formula = dummy, offset = object$offset)))
        mf <- eval(mf, parent.frame())
        x <- mf$x
        y <- model.response(mf)
        Z <- mf$Z
        wt <- model.weights(mf)
        if (is.null(wt)) wt <- rep.int(1, length(y))
        offset <- model.offset(mf)
    }
    else {
        asgn <- attr(x, "assign")
        y <- object$y
        wt <- object$prior.weights
        offset <- object$offset
        Z <- object$random
    }

    if (is.null(object$random)){
        attr(x, "assign") <- asgn + 1
        object$formula <- formula(object$terms)
        object$x <- x
        object$y <- y
        object$random <- Z
        object$prior.weights <- wt
        object$offset <- offset
        stat.table <- NextMethod(x = x)
        attr(stat.table, "heading")[3] <- deparse(old.form)
        if (newsep <- sum(asgn == 0) - sum(object$assign ==0))
            attr(stat.table, "heading") <- c(attr(stat.table, "heading"),
                                             paste("\n", newsep,
                                                   " separate effects added\n",
                                                   sep = ""))
        attr(stat.table, "separate.abilities") <- colnames(x)[asgn == 0]
        return(stat.table)
    }

    ## use original term labels: no sep effects or backticks (typically)
    oTerms <- attr(terms(nobars(old.form)), "term.labels")
    Terms <- attr(terms(nobars(new.form)), "term.labels")
    ousex <- asgn %in% c(0, which(Terms %in% oTerms))

    sTerms <- sapply(strsplit(Terms, ":", fixed = TRUE),
                     function(x) paste(sort(x), collapse = ":"))

    method <- switch(object$method,
                     glmmPQL.fit)
    control <- object$control
    control$trace <- FALSE

    if (scale == 0) dispersion <- 1
    else dispersion <- scale

    ns <- length(scope)
    stat <- df <- numeric(ns) # don't add in original as don't need for tests
    names(stat) <- names(df) <- as.character(scope)
    tryerror <- FALSE
    for (i in seq(scope)) {
        stt <- paste(sort(strsplit(scope[i], ":")[[1]]), collapse = ":")
        usex <- match(asgn, match(stt, sTerms), 0) > 0 | ousex
        fit <- method(X = x[, usex, drop = FALSE], y = y, Z = Z, weights = wt,
                      offset = offset, family = object$family,
                      control = control,
                      sigma = object$call$sigma,
                      sigma.fixed = object$sigma.fixed)
        class(fit) <- oldClass(object)
        ind <- (usex & !ousex)[usex]
        trystat <- try(t(coef(fit)[ind]) %*%
            chol2inv(chol(vcov(fit, dispersion = dispersion)[ind, ind])) %*%
                coef(fit)[ind], silent = TRUE) #vcov should deal with dispersion != 1
        if (inherits(trystat, "try-error")) {
            stat[i] <- df[i] <- NA
            tryerror <- TRUE
        }
        else {
            stat[i] <- trystat
            df[i] <- sum(ind)
        }
    }
    table <- data.frame(stat, df)
    dimnames(table) <- list(names(df), c("Statistic", "Df"))
    title <- "Single term additions\n"
    topnote <- paste("Model: ", deparse(as.vector(formula(object))),
                     if (scale > 0) paste("\nscale: ", format(scale), "\n"),
                     if (tryerror)
                     "\n\nTest statistic unestimable for at least one term")
    test <- match.arg(test)
    if (test == "Chisq") {
        dfs <- table[, "Df"]
        vals <- table[, "Statistic"]
        vals[dfs %in% 0] <- NA
        table <- cbind(table, `P(>|Chi|)` = pchisq(vals, abs(dfs),
                              lower.tail = FALSE))
    }
    else if (test == "F") {
        ## Assume dispersion fixed at one - if dispersion estimated, would use
        ## "residual" df from larger model in each comparison
        df.dispersion <- Inf
        if (df.dispersion == Inf) {
            fam <- object[[1]]$family$family
            if (fam == "binomial" || fam == "poisson")
                warning(gettextf("using F test with a '%s' family is inappropriate",
                                 fam), domain = NA, call. = FALSE)
            else warning("using F test with a fixed dispersion is inappropriate")
        }
        dfs <- table[, "Df"]
        Fvalue <- table[, "Statistic"]/abs(dfs)
        Fvalue[dfs %in% 0] <- NA
        table <- cbind(table, F = Fvalue, `Pr(>F)` =
                       pf(Fvalue, abs(dfs), df.dispersion,
                          lower.tail = FALSE))
    }
    if (newsep <- sum(asgn == 0) - sum(object$assign ==0))
        heading <- c(heading, paste("\n", newsep,
                                    " separate effects added\n",
                                    sep = ""))
    structure(table, heading = c(title, topnote),
              class = c("anova", "data.frame"),
              separate.abilities = colnames(x)[asgn == 0])
}

library(BradleyTerry2)

##  This reproduces the analysis in Sec 10.6 of Agresti (2002).

data(baseball, package = "BradleyTerry2")

##  Simple Bradley-Terry model, ignoring home advantage:
baseballModel1 <- BTm(cbind(home.wins, away.wins), home.team, away.team,
                      data = baseball, id = "team")
summary(baseballModel1)

##  Now incorporate the "home advantage" effect
baseball$home.team <- data.frame(team = baseball$home.team, at.home = 1)
baseball$away.team <- data.frame(team = baseball$away.team, at.home = 0)
baseballModel2 <- update(baseballModel1, formula = ~ team + at.home)
summary(baseballModel2)

##  Compare the fit of these two models:
anova(baseballModel1, baseballModel2)

## P(win|not tie) in terms of expit(lambda_i - lambda_j)
GenDavidsonTie <- function(p){
    scale <- match("tie.scale", substring(names(coef), 1, 9), 0)
    if (scale != 0) scale <- exp(coef[scale])
    else scale <- 1
    tie.mode <- match("tie.mode", substring(names(coef), 1, 8), 0)
    if (tie.mode != 0) tie.mode <- coef["tie.mode"]
    delta <- coef[match("tie.max", substring(names(coef), 1, 7))]
    ## first player is at home
    weight1 <- plogis(tie.mode)
    weight2 <- 1 - weight1
    ## plogis = expit
    plogis(delta - scale * (weight1 * log(weight1) + weight2 * log(weight2)) +
           scale * (weight1 * log(p) + weight2 * log(1-p)))
}

#tmp <- eval(substitute(player1), data, parent.frame())
plotProportions <- function(win, tie = NULL, loss,
                            player1,
                            player2,
                            abilities = NULL,
                            home.adv = NULL,
                            tie.max = NULL,
                            tie.scale = NULL,
                            tie.mode = NULL,
                            at.home1 = NULL,
                            at.home2 = NULL,
                            data = NULL,
                            subset = NULL,
                            bin.size = 20,
                            xlab = "P(player1 wins | not a tie)",
                            ylab = "Proportion",
                            legend = NULL,
                            col = 1:2,
                            ...){
    call <- as.list(match.call())
    var <- intersect(names(call), c("win", "tie", "loss",
                                    "player1", "player2",
                                    "at.home1", "at.home2"))
    var <- var[!sapply(call[var], is.null)]

    dat <- with(data, do.call("data.frame", call[var]))
    if (!missing(subset)){
        subset <- eval(substitute(subset), data, parent.frame())
        dat <- subset(dat, subset)
    }
    if (!missing(tie) && sum(dat$tie) == 0) dat$tie <- NULL
    if (!is.null(home.adv) && (missing(at.home1) || missing(at.home2)))
        stop("at.home1 and at.home2 must be specified")
    if (!is.null(home.adv)){
        ## exclude neutral contests, make sure home player is first
        dat <- subset(dat, at.home1 | at.home2)
        swap <- which(as.logical(dat$at.home2))
        if (length(swap)) {
            dat$win[swap] <- dat$loss[swap]
            if (is.null(dat$tie)) dat$loss[swap] <- !dat$win[swap]
            else dat$loss[swap] <- !(dat$win[swap] | dat$tie[swap])
            tmp <- dat$player1[swap]
            dat$player1[swap] <- dat$player2[swap]
            dat$player2[swap] <- tmp
            dat$at.home1[swap] <- TRUE
            dat$at.home2[swap] <- FALSE
        }
    } else home.adv <- 0
    ### get proportions
    p <- with(dat, plogis(home.adv + abilities[as.character(player1)] -
                          abilities[as.character(player2)]))
    ## Depending on the distribution of p_ij (across all matches),
    ## divide the range of probabilities p_ij into discrete "bins", each
    ## of which has at least (say) 20 matches in it
    getBins <- function(p, bin.size) {
        ## alternatively estimate bins to same size intervals
        ## at least bin.size - distribute extra evenly over range
        min.size <- bin.size
        n <- length(p)
        r <- n %% min.size
        size <- rep(min.size, n %/% min.size)
        if (r > 0) {
            step <- length(size)/r
            extra <- round(seq(from = step/2 + 0.01,
                               to = step/2 + 0.01 + (r - 1)*step, by = step))
            size[extra] <- min.size + 1
        }
        bin <- factor(rep(seq(length(size)), size))[match(p, sort(p))]
        low <- sort(p)[cumsum(c(1, size[-length(size)]))] #first
        high <- sort(p)[cumsum(size)] #last
        mid <- (high - low)/2 + low
        list(bin = bin, mid = mid)
    }
    winBin <- getBins(p, bin.size)
    ## Within each bin b, calculate
    ## d_b = proportion of matches in that bin that were drawn
    if (!is.null(dat$tie)) {
        tieBin <- winBin
        tri <- with(dat, win - (!win & !tie))
        d_b <- tapply(tri, tieBin$bin, function(x) sum(x == 0)/length(x))
        ## recompute bins omitting ties
        winBin <- getBins(p[!dat$tie], bin.size)
    }

    ## h_b = proportion of *non-drawn* matches in that bin that were won
    ## by the home team
    if (!is.null(dat$tie)) {
        h_b <- tapply(tri[!dat$tie], winBin$bin,
                      function(x) sum(x == 1)/length(x))
    }
    else h_b <- tapply(dat$win, winBin$bin, function(x) sum(x == 1)/length(x))

    ## Plot d_b and h_b against the bin midpoints, in a plot with
    ## axis limits both (0,1)
    plot(h_b ~ winBin$mid, xlim = c(0, 1), ylim = c(0, 1),
         xlab = xlab, ylab = ylab, ...)
    if (missing(legend)) {
        if (is.null(dat$tie)) legend <- "Matches won"
        else legend <- c("Non-tied matches won", "Matches tied")
    }
    legend("topleft", legend, col = col[c(1, 2[!missing(tie)])],
           pch = 1)
    if (!is.null(dat$tie)) points(d_b ~ tieBin$mid, col = col[2])

    ## Add to the plot the lines/curves
    ## y = x
    ## y = expit(log(nu * sqrt(p_ij * (1 - p_ij))))
    ## The d_b should lie around the latter curve, and the h_b should
    ## lie around the former line.  Any clear patterns of departure are
    ## of interest.
    curve(I, 0, 1, add = TRUE)

    env <- new.env()
    environment(GenDavidsonTie) <- env
    coef <- na.omit(c(home.adv = unname(home.adv),
                      tie.max = unname(tie.max),
                      tie.scale = unname(tie.scale),
                      tie.mode = unname(tie.mode)))
    assign("coef", coef, envir=env)
    curve(GenDavidsonTie, 0, 1, col = col[2], add = TRUE)
    out <- list(win = data.frame(prop.win = h_b, bin.win = winBin$mid))
    if (!is.null(dat$tie))
        out <- c(out, tie = data.frame(prop.tie = d_b, bin.tie = tieBin$mid))
    invisible(out)
}

residuals.BTm <- function(object, type = c("deviance", "pearson", "working",
                          "response", "partial", "grouped"), by = object$id,
                          ...) {
    type <- match.arg(type)
    if (type != "grouped") return(NextMethod())

    ## for glm, lm would just be
    ## X <- model.matrix(formula, data = object$data)
    formula <- as.formula(paste("~", by, "- 1"))
    mt <- terms(formula)
    mf1 <- model.frame(mt, data = c(object$player1, object$data))
    X1 <- model.matrix(mt, data = mf1)
    mf2 <- model.frame(mt, data = c(object$player2, object$data))
    X2 <- model.matrix(mt, data = mf2)
    X <- X1 - X2

    r <- object$residuals  ## the "working" residuals
    w <- object$weights
    total.resid <- crossprod(X, r * w)
    total.weight <- crossprod(abs(X), w)
    result <- total.resid / total.weight
    attr(result, "weights") <- total.weight
    result
}

glmmPQL.control <- function (maxiter = 50, IWLSiter = 10, tol = 1e-6,
                             trace = FALSE) {
    call <- as.list(match.call())

    if (length(call) > 1) {
        argPos <- match(c("maxiter", "IWLSiter", "tol"), names(call))
        for (n in argPos[!is.na(argPos)]) {
            if (!is.numeric(call[[n]]) || call[[n]] <= 0)
                stop("value of '", names(call)[n], "' must be > 0")
        }
    }

    list(maxiter = maxiter, IWLSiter = IWLSiter, tol = tol,
         trace = trace)
}

---
title: |-
  Bradley-Terry Models in R
abstract: |
  This is a short overview of the R add-on package **BradleyTerry2**,
  which facilitates the specification and fitting of Bradley-Terry
  logit, probit or cauchit models to pair-comparison data. Included are
  the standard 'unstructured' Bradley-Terry model, structured versions
  in which the parameters are related through a linear predictor to
  explanatory variables, and the possibility of an order or 'home
  advantage' effect or other 'contest-specific' effects. Model fitting
  is either by maximum likelihood, by penalized quasi-likelihood (for
  models which involve a random effect), or by bias-reduced maximum
  likelihood in which the first-order asymptotic bias of parameter
  estimates is eliminated. Also provided are a simple and efficient
  approach to handling missing covariate data, and suitably-defined
  residuals for diagnostic checking of the linear predictor.
date: |-
  For **BradleyTerry2** version 
  `r packageDescription("BradleyTerry2")[["Version"]]`, `r Sys.Date()`
vignette: |-
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{Bradley-Terry Models in R}
  %\VignetteDepends{}
output: function(){
    if (requireNamespace('bookdown', quietly = TRUE)) {
        function(...){
          bookdown::html_document2(...,
                                   base_format = rmarkdown::html_vignette,
                                   number_sections = TRUE,
                                   math_method = "mathjax")
        }
    } else function(...){
          rmarkdown::html_vignette(...,
                                   number_sections = TRUE,
                                   math_method = "mathjax") 
    }}()
link-citations: yes
bibliography: BradleyTerry.bib

---

``` {r include=FALSE}
library <- function(...) suppressPackageStartupMessages(base::library(...))
library(knitr)
opts_chunk$set(
tidy=FALSE
)
```

``` {r set_options, echo = FALSE}
options(prompt = "R> ", continue = "+  ", width = 70,
        useFancyQuotes = FALSE, digits = 7)
```

## Contents { .unnumbered}

\@ref(sec:intro) [Introduction]  
\@ref(sec:BTmodel) [Standard Bradley-Terry model]  
    \@ref(sec:citations) [Example: Analysis of journal citations]  
    \@ref(sec:bias-reduced) [Bias-reduced estimates]  
\@ref(sec:covariates) [Abilities predicted by explanatory variables]  
    \@ref(sec:player-specific) ['Player-specific' predictor variables]  
    \@ref(sec:missing) [Missing values]  
    \@ref(sec:order) [Order effect]  
    \@ref(sec:CEMS) [More general (contest-specific) predictors]  
\@ref(sec:ability) [Ability scores]  
\@ref(sec:residuals) [Residuals]  
\@ref(sec:model) [Model search]  
\@ref(sec:data) [Setting up the data]  
    \@ref(sec:contest) [Contest-specific data]  
    \@ref(sec:non-contest) [Non contest-specific data]  
    \@ref(sec:wide) [Converting data from a 'wide' format]  
    \@ref(sec:BradleyTerry) [Converting data from the format required by the earlier **BradleyTerry** package]  
\@ref(sec:functions) [A list of the functions provided in **BradleyTerry2**]  
\@ref(sec:finalremarks) [Some final remarks]  
    \@ref(sec:ties) [A note on the treatment of ties]  
    \@ref(sec:random-effects) [A note on 'contest-specific' random effects]  
[Acknowledgments]  
[References]  

## Introduction {#sec:intro}

The Bradley-Terry model [@brad:terr:52] assumes that
in a 'contest' between any two 'players', say player $i$ and player $j$
$(i, j \in \{1,\ldots,K\})$, the odds that $i$ beats $j$ are
$\alpha_i/\alpha_j$, where $\alpha_i$ and $\alpha_j$ are positive-valued
parameters which might be thought of as representing 'ability'. A
general introduction can be found in @brad:84 or @agre:02. 
Applications are many, ranging from experimental
psychology to the analysis of sports tournaments to genetics [for
example, the allelic transmission/disequilibrium test of @sham:curt:95
is based on a Bradley-Terry model in which the
'players' are alleles]. In typical psychometric applications the
'contests' are comparisons, made by different human subjects, between
pairs of items.

The model can alternatively be expressed in the logit-linear form

$$\mathop{\rm logit}[\mathop{\rm pr}(i\ \mathrm{beats}\ j)]=\lambda_i-\lambda_j,
\label{eq:unstructured}   (\#eq:unstructured)$$

where $\lambda_i=\log\alpha_i$ for all $i$. Thus, assuming independence
of all contests, the parameters $\{\lambda_i\}$ can be estimated by
maximum likelihood using standard software for generalized linear
models, with a suitably specified model matrix. The primary purpose of
the **BradleyTerry2** package [@turn:12],
implemented in the R statistical computing environment [@ihak:gent:96;@r], 
is to facilitate the specification and fitting of such models and some
extensions.

The **BradleyTerry2** package supersedes the earlier **BradleyTerry**
package [@firt:05], providing a more flexible user interface
to allow a wider range of models to be fitted. In particular,
**BradleyTerry2** allows the inclusion of simple random effects so that
the ability parameters can be related to available explanatory variables
through a linear predictor of the form

$$\lambda_i=\sum_{r=1}^p\beta_rx_{ir} + U_i.
(\#eq:autonumber2) $$

The inclusion of the prediction error $U_i$ allows for variability
between players with equal covariate values and induces correlation
between comparisons with a common player. **BradleyTerry2** also allows
for general contest-specific effects to be included in the model and
allows the logit link to be replaced, if required, by a different
symmetric link function (probit or cauchit).

The remainder of the paper is organised as follows.
Section \@ref(sec:BTmodel) demonstrates how to use the **BradleyTerry2**
package to fit a standard (i.e., unstructured) Bradley-Terry model, with
a separate ability parameter estimated for each player, including the
use of bias-reduced estimation for such models.
Section \@ref(sec:covariates) considers variations of the standard model,
including the use of player-specific variables to model ability and
allowing for contest-specific effects such as an order effect or judge
effects. Sections \@ref(sec:ability) and \@ref(sec:residuals) explain how
to obtain important information about a fitted model, in particular the
estimates of ability and their standard errors, and player-level
residuals, whilst Section \@ref(sec:model) notes the functions available
to aid model search. Section \@ref(sec:data) explains in more detail how
set up data for use with the **BradleyTerry2** package,
Section \@ref(sec:functions) lists the functions provided by the package
and finally Section \@ref(sec:finalremarks) comments on two directions
for further development of the software.

## Standard Bradley-Terry model {#sec:BTmodel}

### Example: Analysis of journal citations {#sec:citations}

The following data come from page 448 of @agre:02,
extracted from the larger table of @stig:94. The data are
counts of citations among four prominent journals of statistics and are
included the **BradleyTerry2** package as the data set `citations`:

``` {r LoadBradleyTerry2}
library("BradleyTerry2")
```

``` {r CitationData}
data("citations", package = "BradleyTerry2")
```

``` {r CitationData2}
citations
```

Thus, for example, *Biometrika* was cited 498 times by papers in
*Journal of the American Statistical Association* (JASA) during the
period under study. In order to fit a Bradley-Terry model to these data
using `BTm` from the **BradleyTerry2** package, the data must first be
converted to binomial frequencies. That is, the data need to be
organised into pairs (`player1`, `player2`) and corresponding
frequencies of wins and losses for `player1` against `player2`. The
**BradleyTerry2** package provides the utility function
`countsToBinomial` to convert a contingency table of wins to the format
just described:

``` {r countsToBinomial}
citations.sf <- countsToBinomial(citations)
names(citations.sf)[1:2] <- c("journal1", "journal2")
citations.sf
```

Note that the self-citation counts are ignored -- these provide no
information on the ability parameters, since the abilities are relative
rather than absolute quantities. The binomial response can then be
modelled by the difference in player abilities as follows:

``` {r citeModel}
citeModel <- BTm(cbind(win1, win2), journal1, journal2, ~ journal,
    id = "journal", data = citations.sf)
citeModel
```

The coefficients here are maximum likelihood estimates of
$\lambda_2, \lambda_3,
\lambda_4$, with $\lambda_1$ (the log-ability for *Biometrika*) set to
zero as an identifying convention.

The one-sided model formula

``` r
  ~ journal
```

specifies the model for player ability, in this case the 'citeability'
of the journal. The `id` argument specifies that `"journal"` is the name
to be used for the factor that identifies the player -- the values of
which are given here by `journal1` and `journal2` for the first and
second players respectively. Therefore in this case a separate
citeability parameter is estimated for each journal.

If a different 'reference' journal is required, this can be achieved
using the optional `refcat` argument: for example, making use of
`update` to avoid re-specifying the whole model,

``` {r citeModelupdate}
update(citeModel, refcat = "JASA")
```

-- the same model in a different parameterization.

The use of the standard Bradley-Terry model for this application might
perhaps seem rather questionable -- for example, citations within a
published paper can hardly be considered independent, and the model
discards potentially important information on self-citation. @stig:94 
provides arguments to defend the model's use despite such
concerns.

### Bias-reduced estimates {#sec:bias-reduced}

Estimation of the standard Bradley-Terry model in `BTm` is by default
computed by maximum likelihood, using an internal call to the `glm`
function. An alternative is to fit by bias-reduced maximum likelihood
[@firt:93]: this requires additionally the **brglm** package
[@kosm:07], and is specified by the optional argument
`br = TRUE`. The resultant effect, namely removal of first-order
asymptotic bias in the estimated coefficients, is often quite small. One
notable feature of bias-reduced fits is that all estimated coefficients
and standard errors are necessarily finite, even in situations of
'complete separation' where maximum likelihood estimates take infinite
values [@hein:sche:02].

For the citation data, the parameter estimates are only very slightly
changed in the bias-reduced fit:

``` {r citeModelupdate2}
update(citeModel, br = TRUE)
```

Here the bias of maximum likelihood is small because the binomial counts
are fairly large. In more sparse arrangements of contests -- that is,
where there is less or no replication of the contests -- the effect of
bias reduction would typically be more substantial than the
insignificant one seen here.

## Abilities predicted by explanatory variables {#sec:covariates}

### 'Player-specific' predictor variables {#sec:player-specific}

In some application contexts there may be 'player-specific' explanatory
variables available, and it is then natural to consider model
simplification of the form

$$\lambda_i=\sum_{r=1}^p\beta_rx_{ir} + U_i,
(\#eq:autonumber3) $$

in which ability of each player $i$ is related to explanatory variables
$x_{i1},\ldots,x_{ip}$ through a linear predictor with coefficients
$\beta_1,\ldots,\beta_p$; the $\{U_i\}$ are independent errors.
Dependence of the player abilities on explanatory variables can be
specified via the `formula` argument, using the standard *S*-language
model formulae. The difference in the abilities of player $i$ and player
$j$ is modelled by

$$\sum_{r=1}^p\beta_rx_{ir} - \sum_{r=1}^p\beta_rx_{jr} + U_i - U_j,
\label{eq:structured}   (\#eq:structured)$$

where $U_i \sim N(0, \sigma^2)$ for all $i$. The Bradley-Terry model is
then a generalized linear mixed model, which the `BTm` function
currently fits by using the penalized quasi-likelihood algorithm of
@bres:93.

As an illustration, consider the following simple model for the
`flatlizards` data, which predicts the fighting ability of Augrabies
flat lizards by body size (snout to vent length):

``` {r lizModel}
options(show.signif.stars = FALSE)
data("flatlizards", package = "BradleyTerry2")
lizModel <- BTm(1, winner, loser, ~ SVL[..] + (1|..),
                data = flatlizards)
```

Here the winner of each fight is compared to the loser, so the outcome
is always 1. The special name '`..`' appears in the formula as the
default identifier for players, in the absence of a user-specified `id`
argument. The values of this factor are given by `winner` for the
winning lizard and `loser` for the losing lizard in each contest. These
factors are provided in the data frame `contests` that is the first
element of the list object `flatlizards`. The second element of
`flatlizards` is another data frame, `predictors`, containing
measurements on the observed lizards, including `SVL`, which is the
snout to vent length. Thus `SVL[..]` represents the snout to vent length
indexed by lizard (`winner` or `loser` as appropriate). Finally a random
intercept for each lizard is included using the bar notation familiar to
users of the **lme4** package [@bate:11]. (Note that a random intercept is the only random effect structure
currently implemented in **BradleyTerry2**.)

The fitted model is summarized below:

``` {r summarize_lizModel}
summary(lizModel)
```

The coefficient of snout to vent length is weakly significant; however,
the standard deviation of the random effect is quite large, suggesting
that this simple model has fairly poor explanatory power. A more
appropriate model is considered in the next section.

### Missing values {#sec:missing}

The contest data may include all possible pairs of players and hence
rows of missing data corresponding to players paired with themselves.
Such rows contribute no information to the Bradley-Terry model and are
simply discarded by `BTm`.

Where there are missing values in player-specific *predictor* (or
*explanatory*) variables which appear in the formula, it will typically
be very wasteful to discard all contests involving players for which
some values are missing. Instead, such cases are accommodated by the
inclusion of one or more parameters in the model. If, for example,
player $1$ has one or more of its predictor values
$x_{11},\ldots,x_{1p}$ missing, then the combination of
Equations \@ref(eq:unstructured) and \@ref(eq:structured) above yields

$$\mathop{\rm logit}[\mathop{\rm pr}(1\ \mathrm{beats}\ j)]=\lambda_1 - \left(\sum_{r=1}^p\beta_rx_{jr} + U_j\right),
(\#eq:autonumber5) $$

for all other players $j$. This results in the inclusion of a 'direct'
ability parameter for each player having missing predictor values, in
addition to the common coefficients $\beta_1,\ldots,\beta_p$ -- an
approach which will be appropriate when the missingness mechanism is
unrelated to contest success. The same device can be used also to
accommodate any user-specified departures from a structured
Bradley-Terry model, whereby some players have their abilities
determined by the linear predictor but others do not.

In the original analysis of the `flatlizards` data [@whit:06], 
the final model included the first and third principal components
of the spectral reflectance from the throat (representing brightness and
UV intensity respectively) as well as head length and the snout to vent
length seen in our earlier model. The spectroscopy data was missing for
two lizards, therefore the ability of these lizards was estimated
directly. The following fits this model, with the addition of a random
intercept as before:

``` {r lizModel2}
lizModel2 <- BTm(1, winner, loser,
            ~ throat.PC1[..] + throat.PC3[..] +
            head.length[..] + SVL[..] + (1|..),
            data = flatlizards)
summary(lizModel2)
```

Note that `BTm` detects that lizards 96 and 99 have missing values in
the specified predictors and automatically includes separate ability
parameters for these lizards. This model was found to be the single best
model based on the principal components of reflectance and the other
predictors available and indeed the standard deviation of the random
intercept is much reduced, but still highly significant. Allowing for
this significant variation between lizards with the same predictor
values produces more realistic (i.e., larger) standard errors for the
parameters when compared to the original analysis of @whit:06. 
Although this affects the significance of the morphological
variables, it does not affect the significance of the principal
components, so in this case does not affect the main conclusions of the
study.

### Order effect {#sec:order}

In certain types of application some or all contests have an associated
'bias', related to the order in which items are presented to a judge or
with the location in which a contest takes place, for example. A natural
extension of the Bradley-Terry model (Equation \@ref(eq:unstructured))
is then

$$\mathop{\rm logit}[\mathop{\rm pr}(i\ \mathrm{beats}\ j)]=\lambda_i-\lambda_j + \delta z,
(\#eq:autonumber6) $$

where $z=1$ if $i$ has the supposed advantage and $z=-1$ if $j$ has it.
(If the 'advantage' is in fact a disadvantage, $\delta$ will be
negative.) The scores $\lambda_i$ then relate to ability in the absence
of any such advantage.

As an example, consider the baseball data given in @agre:02, page 438:

``` {r baseball}
data("baseball", package = "BradleyTerry2")
head(baseball)
```

The data set records the home wins and losses for each baseball team
against each of the 6 other teams in the data set. The `head` function
is used to show the first 6 records, which are the Milwaukee home games.
We see for example that Milwaukee played 7 home games against Detroit
and won 4 of them. The 'standard' Bradley-Terry model without a
home-advantage parameter will be fitted if no formula is specified in
the call to `BTm`:

``` {r baseballModel}
baseballModel1 <- BTm(cbind(home.wins, away.wins), home.team, away.team,
                      data = baseball, id = "team")
summary(baseballModel1)
```

The reference team is Baltimore, estimated to be the weakest of these
seven, with Milwaukee and Detroit the strongest.

In the above, the ability of each team is modelled simply as `  team`
where the values of the factor `team` are given by `home.team` for the
first team and `away.team` for the second team in each game. To estimate
the home-advantage effect, an additional variable is required to
indicate whether the team is at home or not. Therefore data frames
containing both the team factor and this new indicator variable are
required in place of the factors `home.team` and `away.team` in the call
to `BTm`. This is achieved here by over-writing the `home.team` and
`away.team` factors in the `baseball` data frame:

``` {r baseballDataUpdate}
baseball$home.team <- data.frame(team = baseball$home.team, at.home = 1)
baseball$away.team <- data.frame(team = baseball$away.team, at.home = 0)
```

The `at.home` variable is needed for both the home team and the away
team, so that it can be differenced as appropriate in the linear
predictor. With the data organised in this way, the ability formula can
now be updated to include the `at.home` variable as follows:

``` {r baseballModelupdate}
baseballModel2 <- update(baseballModel1, formula = ~ team + at.home)
summary(baseballModel2)
```

This reproduces the results given on page 438 of @agre:02: 
the home team has an estimated odds-multiplier of
$\exp(0.3023) = 1.35$ in its favour.

### More general (contest-specific) predictors {#sec:CEMS}

The 'home advantage' effect is a simple example of a contest-specific
predictor. Such predictors are necessarily interactions, between aspects
of the contest and (aspects of) the two 'players' involved.

For more elaborate examples of such effects, see `?chameleons` and
`?CEMS`. The former includes an 'experience' effect, which changes
through time, on the fighting ability of male chameleons. The latter
illustrates a common situation in psychometric applications of the
Bradley-Terry model, where *subjects* express preference for one of two
*objects* (the 'players'), and it is the influence on the results of
subject attributes that is of primary interest.

As an illustration of the way in which such effects are specified,
consider the following model specification taken from the examples in
`?CEMS`, where data on students' preferences in relation to six European
management schools is analysed.

``` {r CEMSmodel}
data("CEMS", package = "BradleyTerry2")
table8.model <-  BTm(outcome = cbind(win1.adj, win2.adj),
    player1 = school1, player2 = school2, formula = ~ .. +
    WOR[student] * LAT[..] +  DEG[student] * St.Gallen[..] +
    STUD[student] * Paris[..] + STUD[student] * St.Gallen[..] +
    ENG[student] * St.Gallen[..] + FRA[student] * London[..] +
    FRA[student] * Paris[..] + SPA[student] * Barcelona[..] +
    ITA[student] * London[..] + ITA[student] * Milano[..] +
    SEX[student] * Milano[..],
    refcat = "Stockholm", data = CEMS)
```

This model reproduces results from Table 8 of @ditt:01 
apart from minor differences due to the
different treatment of ties. Here the outcome is the binomial frequency
of preference for `school1` over `school2`, with ties counted as half a
'win' and half a 'loss'. The formula specifies the model for school
'ability' or worth. In this formula, the default label '`..`' represents
the school (with values given by `school1` or `school2` as appropriate)
and `student` is a factor specifying the student that made the
comparison. The remaining variables in the formula use
[R]{.sans-serif}'s standard indexing mechanism to include
student-specific variables, e.g., `WOR`: whether or not the student was
in full-time employment, and school-specific variables, e.g., `LAT`:
whether the school was in a 'Latin' city. Thus there are three types of
variables: contest-specific (`school1`, `school2`, `student`),
subject-specific (`WOR`, `DEG`, ...) and object-specific (`LAT`,
`St.Gallen`, ...). These three types of variables are provided in three
data frames, contained in the list object `CEMS`.

## Ability scores {#sec:ability}

The function `BTabilities` extracts estimates and standard errors for
the log-ability scores $\lambda_1, \ldots,\lambda_K$. These will either
be 'direct' estimates, in the case of the standard Bradley-Terry model
or for players with one or more missing predictor values, or
'model-based' estimates of the form
$\hat\lambda_i=\sum_{r=1}^p\hat\beta_rx_{ir}$ for players whose ability
is predicted by explanatory variables.

As a simple illustration, team ability estimates in the home-advantage
model for the `baseball` data are obtained by:

``` {r BTabilities}
BTabilities(baseballModel2)
```

This gives, for each team, the estimated ability when the team enjoys no
home advantage.

Similarly, estimates of the fighting ability of each lizard in the
`flatlizards` data under the model based on the principal components of
the spectral reflectance from the throat are obtained as follows:

``` {r BTabilities2}
head(BTabilities(lizModel2), 4)
```

The ability estimates in an unstructured Bradley-Terry model are
particularly well suited to presentation using the device of
*quasi-variances* [@firt:04]. The **qvcalc**
package [@firt:10,version 0.8-5 or later] contains a
function of the same name which does the necessary work:

``` r
> library("qvcalc")
> baseball.qv <- qvcalc(BTabilities(baseballModel2))
> plot(baseball.qv,
+      levelNames = c("Bal", "Bos", "Cle", "Det", "Mil", "NY", "Tor"))
```

```{r figqvplot, echo=FALSE , fig.cap="Estimated relative abilities of baseball teams.", fig.alt="The ability for Baltimore is fixed at zero, with an interval ranging from -0.5 to 0.5. Boston has a relative ability near 1.2; Cleveland around 0.7. The remaining teams have relative abilities around 1.3 to 1.6. The intervals are based on quasi standard errors and all have length of approximately 1. Therefore, aside from Cleveland, all teams are clearly significantly stronger than Baltimore as the intervals do not overlap.", fig.show='hold', fig.align="center", out.width="67.0%"}
knitr::include_graphics(c("baseball-qvplot.png"))
```

The 'comparison intervals' as shown in Figure \@ref(fig:figqvplot) are
based on 'quasi standard errors', and can be interpreted as if they
refer to *independent* estimates of ability for the journals. This has
the advantage that comparison between any pair of journals is readily
made (i.e., not only comparisons with the 'reference' journal). For
details of the theory and method of calculation see @firt:04.

## Residuals {#sec:residuals}

There are two main types of residuals available for a Bradley-Terry
model object.

First, there are residuals obtained by the standard methods for models
of class `"glm"`. These all deliver one residual for each contest or
type of contest. For example, Pearson residuals for the model
`lizModel2` can be obtained simply by

``` {r residuals}
res.pearson <- round(residuals(lizModel2), 3)
head(cbind(flatlizards$contests, res.pearson), 4)
```

More useful for diagnostics on the linear predictor $\sum\beta_rx_{ir}$
are 'player'-level residuals, obtained by using the function `residuals`
with argument `type = "grouped"`. These residuals can then be plotted
against other player-specific variables.

``` {r BTresiduals}
res <- residuals(lizModel2, type = "grouped")
#  with(flatlizards$predictors, plot(throat.PC2, res))
#  with(flatlizards$predictors, plot(head.width, res))
```

These residuals estimate the error in the linear predictor; they are
obtained by suitable aggregation of the so-called 'working' residuals
from the model fit. The `weights` attribute indicates the relative
information in these residuals -- weight is roughly inversely
proportional to variance -- which may be useful for plotting and/or
interpretation; for example, a large residual may be of no real concern
if based on very little information. Weighted least-squares regression
of these residuals on any variable already in the model is null. For
example:

``` {r residualWLS}
lm(res ~ throat.PC1, weights = attr(res, "weights"),
   data = flatlizards$predictors)
lm(res ~ head.length, weights = attr(res, "weights"),
   data = flatlizards$predictors)
```

As an illustration of evident *non-null* residual structure, consider
the unrealistically simple model `lizModel` that was fitted in
Section \@ref(sec:covariates) above. That model lacks the clearly
significant predictor variable `throat.PC3`, and the plot shown in
Figure \@ref(fig:figresiduals) demonstrates this fact graphically:

``` r
lizModel.residuals <- residuals(lizModel, type = "grouped")
plot(flatlizards$predictors$throat.PC3, lizModel.residuals)
```

```{r figresiduals, echo=FALSE , fig.cap="Lizard residuals for the simple model lizModel, plotted against throat.PC3.", fig.alt="The residuals are quite spread out over the range -2 to 2, but the distribution is clearly not uniform over the range of the predictor variable, throat.PC3. Residuals between -2 and -1 range correspond to values throat.PC3 between -6 and 4; residuals between -1 and 1 correspond to throat.PC3 values of -4 to 4, and residuals from 1 to 2 correspond to throat.PC3 values between -3 and 6. Thus there is an overall positive correlation bewteen the residuals and throat.PC3.", fig.show='hold', fig.align="center", out.width="69.0%"}
knitr::include_graphics(c("residuals.png"))
```

The residuals in the plot exhibit a strong, positive regression slope in
relation to the omitted predictor variable `throat.PC3`.

## Model search {#sec:model}

In addition to `update()` as illustrated in preceding sections, methods
for the generic functions `add1()`, `drop1()` and `anova()` are
provided. These can be used to investigate the effect of adding or
removing a variable, whether that variable is contest-specific, such as
an order effect, or player-specific; and to compare the fit of nested
models.

## Setting up the data {#sec:data}

### Contest-specific data {#sec:contest}

The `outcome` argument of `BTm` represents a binomial response and can
be supplied in any of the formats allowed by the `glm` function. That
is, either a two-column matrix with the columns giving the number of
wins and losses (for `player1` vs. `player2`), a factor where the first
level denotes a loss and all other levels denote a win, or a binary
variable where 0 denotes a loss and 1 denotes a win. Each row represents
either a single contest or a set of contests between the same two
players.

The `player1` and `player2` arguments are either factors specifying the
two players in each contest, or data frames containing such factors,
along with any contest-specific variables that are also player-specific,
such as the `at.home` variable seen in Section \@ref(sec:order). If
given in data frames, the factors identifying the players should be
named as specified by the `id` argument and should have identical
levels, since they represent a particular sample of the full set of
players.

Thus for the model `baseballModel2`, which was specified by the
following call:

``` {r baseballModel2_call}
 baseballModel2$call
```

the data are provided in the `baseball` data frame, which has the
following structure:

``` {r str_baseball}
str(baseball, vec.len = 2)
```

In this case `home.team` and `away.team` are both data frames, with the
factor `team` specifying the team and the variable `at.home` specifying
whether or not the team was at home. So the first comparison

``` {r first_comparison}
baseball$home.team[1,]
baseball$away.team[1,]
```

is Milwaukee playing at home against Detroit. The outcome is given by

``` {r first_outcome}
 baseball[1, c("home.wins", "away.wins")]
```

Contest-specific variables that are *not* player-specific -- for
example, whether it rained or not during a contest -- should only be
used in interactions with variables that *are* player-specific,
otherwise the effect on ability would be the same for both players and
would cancel out. Such variables can conveniently be provided in a
single data frame along with the `outcome`, `player1` and `player2`
data.

An offset in the model can be specified by using the `offset` argument
to `BTm`. This facility is provided for completeness: the authors have
not yet encountered an application where it is needed.

To use only certain rows of the contest data in the analysis, the
`subset` argument may be used in the call to `BTm`. This should either
be a logical vector of the same length as the binomial response, or a
numeric vector containing the indices of rows to be used.

### Non contest-specific data {#sec:non-contest}

Some variables do not vary by contest directly, but rather vary by a
factor that is contest-specific, such as the player ID or the judge
making the paired comparison. For such variables, it is more economical
to store the data by the levels of the contest-specific factor and use
indexing to obtain the values for each contest.

The `CEMS` example in Section \@ref(sec:CEMS) provides an illustration
of such variables. In this example student-specific variables are
indexed by `student` and school-specific variables are indexed by `..`,
i.e., the first or second school in the comparison as appropriate. There
are then two extra sets of variables in addition to the usual
contest-specific data as described in the last section. A good way to
provide these data to `BTm` is as a list of data frames, one for each
set of variables, e.g.,

``` {r str_CEMS}
str(CEMS, vec.len = 2)
```

The names of the data frames are only used by `BTm` if they match the
names specified in the `player1` and `player2` arguments, in which case
it is assumed that these are data frames providing the data for the
first and second player respectively. The rows of data frames in the
list should either correspond to the contests or the levels of the
factor used for indexing.

Player-specific offsets should be included in the formula by using the
`offset` function.

### Converting data from a 'wide' format {#sec:wide}

The `BTm` function requires data in a 'long' format, with one row per
contest, provided either directly as in Section \@ref(sec:contest) or
via indexing as in Section \@ref(sec:non-contest). In studies where the
same set of paired comparisons are made by several judges, as in a
questionnaire for example, the data may be stored in a 'wide' format,
with one row per judge.

As an example, consider the `cemspc` data from the **prefmod** package
[@hatz:12], which provides data from the
CEMS study in a wide format. Each row corresponds to one student; the
first 15 columns give the outcome of all pairwise comparisons between
the 6 schools in the study and the last two columns correspond to two of
the student-specific variables: `ENG` (indicating the student's
knowledge of English) and `SEX` (indicating the student's gender).

The following steps convert these data into a form suitable for analysis
with `BTm`. First a new data frame is created from the student-specific
variables and these variables are converted to factors:

``` {r student-specific_data}
library("prefmod")
student <- cemspc[c("ENG", "SEX")]
student$ENG <- factor(student$ENG, levels = 1:2,
                      labels = c("good", "poor"))
student$SEX <- factor(student$SEX, levels = 1:2,
                      labels = c("female", "male"))
```

This data frame is put into a list, which will eventually hold all the
necessary data. Then a `student` factor is created for indexing the
student data to produce contest-level data. This is put in a new data
frame that will hold the contest-specific data.

``` {r student_factor}
cems <- list(student = student)
student <- gl(303, 1, 303 * 15) #303 students, 15 comparisons
contest <- data.frame(student = student)
```

Next the outcome data is converted to a binomial response, adjusted for
ties. The result is added to the `contest` data frame.

``` {r binomial_response}
win <- cemspc[, 1:15] == 0
lose <- cemspc[, 1:15] == 2
draw <- cemspc[, 1:15] == 1
contest$win.adj <- c(win + draw/2)
contest$lose.adj <- c(lose + draw/2)
```

Then two factors are created identifying the first and second school in
each comparison. The comparisons are in the order 1 vs. 2, 1 vs. 3, 2
vs. 3, 1 vs. 4, ..., so the factors can be created as follows:

``` {r school_factors}
lab <- c("London", "Paris", "Milano", "St. Gallen", "Barcelona",
         "Stockholm")
contest$school1 <- factor(sequence(1:5), levels = 1:6, labels = lab)
contest$school2 <- factor(rep(2:6, 1:5), levels = 1:6, labels = lab)
```

Note that both factors have exactly the same levels, even though only
five of the six players are represented in each case. In other words,
the numeric factor levels refer to the same players in each case, so
that the player is unambiguously identified. This ensures that
player-specific parameters and player-specific covariates are correctly
specified.

Finally the `contest` data frame is added to the main list:

``` {r cems_data}
cems$contest <- contest
```

This creates a single data object that can be passed to the `data`
argument of `BTm`. Of course, such a list could be created on-the-fly as
in `data = list(contest, student)`, which may be more convenient in
practice.

### Converting data from the format required by the earlier **BradleyTerry** package {#sec:BradleyTerry}

The **BradleyTerry** package described in @firt:05 required
contest/comparison results to be in a data frame with columns named
`winner`, `loser` and `Freq`. The following example shows how `xtabs`
and `countsToBinomial` can be used to convert such data for use with the
`BTm` function in **BradleyTerry2**:

``` r
library("BradleyTerry")  ## the /old/ BradleyTerry package
## load data frame with columns "winner", "loser", "Freq"
data("citations", package = "BradleyTerry")
## convert to 2-way table of counts
citations <- xtabs(Freq ~ winner + loser, citations)
## convert to a data frame of binomial observations
citations.sf <- countsToBinomial(citations)
```

The `citations.sf` data frame can then be used with `BTm` as shown in
Section \@ref(sec:citations).

## A list of the functions provided in **BradleyTerry2** {#sec:functions}

The standard R help files provide the definitive reference. Here we
simply list the main user-level functions and their arguments, as a
convenient overview:

``` {r functions, echo = FALSE}
## cf. prompt
options(width = 55)
for (fn in getNamespaceExports("BradleyTerry2")) {
    name <- as.name(fn)
    args <- formals(fn)
    n <- length(args)
    arg.names <- arg.n <- names(args)
    arg.n[arg.n == "..."] <- "\\dots"
    is.missing.arg <- function(arg) typeof(arg) == "symbol" &&
        deparse(arg) == ""
    Call <- paste(name, "(", sep = "")
        for (i in seq_len(n)) {
        Call <- paste(Call, arg.names[i], if (!is.missing.arg(args[[i]]))
            paste(" = ", paste(deparse(args[[i]]),
                collapse = "\n"), sep = ""), sep = "")
        if (i != n)
            Call <- paste(Call, ", ", sep = "")
    }
    Call <- paste(Call, ")", sep = "")
    cat(deparse(parse(text = Call)[[1]], width.cutoff = 50), fill = TRUE)
}
options(width = 60)
```

## Some final remarks {#sec:finalremarks}

### A note on the treatment of ties {#sec:ties}

The present version of **BradleyTerry2** provides no sophisticated
facilities for handling tied contests/comparisons; the well-known models
of @rao:kupp:67 and @davi:70 are
not implemented here. At present the `BTm` function requires a binary or
binomial response variable, the third ('tied') category of response is
not allowed.

In several of the data examples (e.g., `?CEMS`, `?springall`,
`?sound.fields`), ties are handled by the crude but simple device of
adding half of a 'win' to the tally for each player involved; in each of
the examples where this has been done it is found that the result is
very similar, after a simple re-scaling, to the more sophisticated
analyses that have appeared in the literature. Note that this device
when used with `BTm` typically gives rise to warnings produced by the
back-end `glm` function, about non-integer 'binomial' counts; such
warnings are of no consequence and can be safely ignored.

It is likely that a future version of **BradleyTerry2** will have a more
general method for handling ties.

### A note on 'contest-specific' random effects {#sec:random-effects}

The current version of **BradleyTerry2** provides facilities for fitting
models with random effects in 'player-specific' predictor functions, as
illustrated in Section \@ref(#sec:covariates). For more general,
'contest-specific' random-effect structures, such as random 'judge'
effects in psychological studies [e.g., @bock:01],
**BradleyTerry2** provides (through `BTm`) the necessary user interface
but as yet no back-end calculation. It is hoped that this important
generalization can be made successfully in a future version of
**BradleyTerry2**.

## Acknowledgments {#sec:acknowledgments .unnumbered}

This work was supported by the UK Engineering and Physical Sciences
Research Council.

## References {#sec:references .unnumbered}

\name{flatlizards}
\alias{flatlizards}
\docType{data}
\title{ Augrabies Male Flat Lizards: Contest Results and Predictor
  Variables }
\description{
  Data collected at Augrabies Falls National Park (South Africa) in
  September-October 2002, on the contest performance and background
  attributes of 77 male flat lizards (\emph{Platysaurus broadleyi}).  The
  results of exactly 100 contests were recorded, along with various
  measurements made on each lizard.  Full details of the study are in
  Whiting et al. (2006).
}
\usage{flatlizards}
\format{
  This dataset is a list containing two data frames: \code{flatlizards$contests}
  and \code{flatlizards$predictors}.

  The \code{flatlizards$contests} data frame has 100 observations on the following 2 variables:
  \describe{
    \item{\code{winner}}{a factor with 77 levels \code{lizard003} ... \code{lizard189}.}
    \item{\code{loser}}{a factor with the same 77 levels \code{lizard003} ... \code{lizard189}.}
  }

  The \code{flatlizards$predictors} data frame has 77 observations (one
  for each of the 77 lizards) on the following 18 variables:
  \describe{
    \item{\code{id}}{factor with 77 levels (3 5 6 ... 189), the lizard
      identifiers.}
    \item{\code{throat.PC1}}{numeric, the first principal component of
      the throat spectrum.}
    \item{\code{throat.PC2}}{numeric, the second principal component of
      the throat spectrum.}
    \item{\code{throat.PC3}}{numeric, the third principal component of
      the throat spectrum.}
    \item{\code{frontleg.PC1}}{numeric, the first principal component of
      the front-leg spectrum.}
    \item{\code{frontleg.PC2}}{numeric, the second principal component of
      the front-leg spectrum.}
    \item{\code{frontleg.PC3}}{numeric, the third principal component of
      the front-leg spectrum.}
    \item{\code{badge.PC1}}{numeric, the first principal component of
      the ventral colour patch spectrum.}
    \item{\code{badge.PC2}}{numeric, the second principal component of
      the ventral colour patch spectrum.}
    \item{\code{badge.PC3}}{numeric, the third principal component of
      the ventral colour patch spectrum.}
    \item{\code{badge.size}}{numeric, a measure of the area of the
      ventral colour patch.}
    \item{\code{testosterone}}{numeric, a measure of blood testosterone
      concentration.}
    \item{\code{SVL}}{numeric, the snout-vent length of the lizard.}
    \item{\code{head.length}}{numeric, head length.}
    \item{\code{head.width}}{numeric, head width.}
    \item{\code{head.height}}{numeric, head height.}
    \item{\code{condition}}{numeric, a measure of body condition.}
    \item{\code{repro.tactic}}{a factor indicating reproductive tactic; 
      levels are \code{resident} and \code{floater}.}
  }
}
\details{
  There were no duplicate contests (no pair of lizards was seen fighting
  more than once), and there were no tied contests (the result of each
  contest was clear).

  The variables \code{head.length}, \code{head.width},
  \code{head.height} and \code{condition} were all computed as
  residuals (of directly measured head length, head width, head height
  and body mass index, respectively) from simple least-squares regressions
  on \code{SVL}.

  Values of some predictors are missing (\code{NA}) for some lizards,
  \sQuote{at random}, because of instrument problems unconnected with
  the value of the measurement being made.
}
\source{
  The data were collected by Dr Martin Whiting, 
  \url{http://whitinglab.com/?page_id=3380},
  and they appear here with his kind permission.
}
\references{
  Turner, H. and Firth, D. (2012)  Bradley-Terry models in R: The
  BradleyTerry2 package.  \emph{Journal of Statistical
    Software},  \bold{48}(9), 1--21.
  
  Whiting, M. J., Stuart-Fox, D. M., O'Connor, D., Firth, D., Bennett,
  N. C. and Blomberg, S. P. (2006). Ultraviolet signals ultra-aggression
  in a lizard. \emph{Animal Behaviour} \bold{72}, 353--363.
}
\seealso{\code{\link{BTm}}}
\examples{
attach(flatlizards)
##
##  Fit the standard Bradley-Terry model, using the bias-reduced
##  maximum likelihood method:
##
result <- rep(1, nrow(contests))
BTmodel <- BTm(result, winner, loser, br = TRUE, data = contests)
summary(BTmodel)
##
##  That's fairly useless, though, because of the rather small
##  amount of data on each lizard.  And really the scientific
##  interest is not in the abilities of these particular 77
##  lizards, but in the relationship between ability and the
##  measured predictor variables.
##
##  So next fit (by maximum likelihood) a "structured" B-T model in
##  which abilities are determined by a linear predictor.
##
##  This reproduces results reported in Table 1 of Whiting et al. (2006):
##
Whiting.model <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
                     head.length[..] + SVL[..],
                     data = list(contests, predictors))
summary(Whiting.model)
##
##  Equivalently, fit the same model using glmmPQL:
##
Whiting.model <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
                     head.length[..] + SVL[..] + (1|..), sigma = 0,
                     sigma.fixed = TRUE, data = list(contests, predictors))
summary(Whiting.model)
##
##  But that analysis assumes that the linear predictor formula for
##  abilities is _perfect_, i.e., that there is no error in the linear
##  predictor.  This will always be unrealistic.
##
##  So now fit the same predictor but with a normally distributed error
##  term --- a generalized linear mixed model --- by using the BTm
##  function instead of glm.
##
Whiting.model2 <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
                      head.length[..] + SVL[..] + (1|..), 
                      data = list(contests, predictors), trace = TRUE)
summary(Whiting.model2)
##
##  The estimated coefficients (of throat.PC1, throat.PC3,
##  head.length and SVL are not changed substantially by
##  the recognition of an error term in the model; but the estimated
##  standard errors are larger, as expected.  The main conclusions from
##  Whiting et al. (2006) are unaffected.
##
##  With the normally distributed random error included, it is perhaps
##  at least as natural to use probit rather than logit as the link
##  function:
##
Whiting.model3 <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
                      head.length[..] + SVL[..] + (1|..),
                      family = binomial(link = "probit"),
                      data = list(contests, predictors), trace = TRUE)
summary(Whiting.model3)
BTabilities(Whiting.model3)
##  Note the "separate" attribute here, identifying two lizards with
##  missing values of at least one predictor variable 
##
##  Modulo the usual scale change between logit and probit, the results
##  are (as expected) very similar to Whiting.model2.
}
\keyword{datasets}

\name{baseball}
\alias{baseball}
\docType{data}
\title{ Baseball Data from Agresti (2002) }
\description{
  Baseball results for games in the 1987 season between 7 teams
  in the Eastern Division of the American League.
}
\usage{baseball}
\format{
  A data frame with 42 observations on the following 4 variables.
  \describe{
    \item{\code{home.team}}{a factor with levels \code{Baltimore},
      \code{Boston}, \code{Cleveland}, \code{Detroit},
      \code{Milwaukee}, \code{New York}, \code{Toronto}.}
    \item{\code{away.team}}{a factor with levels \code{Baltimore},
      \code{Boston}, \code{Cleveland}, \code{Detroit},
      \code{Milwaukee},  \code{New York}, \code{Toronto}.}
    \item{\code{home.wins}}{a numeric vector.}
    \item{\code{away.wins}}{a numeric vector.}
  }
}
\source{
  Page 438 of Agresti, A. (2002) \emph{Categorical Data Analysis} (2nd
  Edn.). New York: Wiley.
}
\references{
  Firth, D. (2005)  Bradley-Terry models in R.  \emph{Journal of
    Statistical Software},  \bold{12}(1), 1--12.
  
  Turner, H. and Firth, D. (2012)  Bradley-Terry models in R: The
  BradleyTerry2 package.  \emph{Journal of Statistical
    Software},  \bold{48}(9), 1--21.
}
\note{This dataset is in a simpler format than the one described in
  Firth (2005).}
\seealso{\code{\link{BTm}}}
\examples{
##  This reproduces the analysis in Sec 10.6 of Agresti (2002).

##  Simple Bradley-Terry model, ignoring home advantage:
baseballModel1 <- BTm(cbind(home.wins, away.wins), home.team, away.team,
                      data = baseball, id = "team")

##  Now incorporate the "home advantage" effect
baseball$home.team <- data.frame(team = baseball$home.team, at.home = 1)
baseball$away.team <- data.frame(team = baseball$away.team, at.home = 0)
baseballModel2 <- update(baseballModel1, formula = ~ team + at.home)

##  Compare the fit of these two models:
anova(baseballModel1, baseballModel2)

}
\keyword{datasets}

\name{sound.fields}
\alias{sound.fields}
\docType{data}
\title{
Kousgaard (1984) Data on Pair Comparisons of Sound Fields
}
\description{
The results of a series of factorial subjective room acoustic
  experiments carried out at the Technical University of Denmark by A C
  Gade. 
}
\usage{sound.fields}
\format{
A list containing two data frames, \code{sound.fields$comparisons},
and \code{sound.fields$design}.

The \code{sound.fields$comparisons} data frame has 84 observations on the following 8 variables:
 \describe{
   \item{\code{field1}}{a factor with levels \code{c("000", "001", "010", "011", "100", "101", "110", "111")}, the first sound
     field in a comparison}
   \item{\code{field2}}{a factor with the same levels as
     \code{field1}; the second sound field in a comparison}
   \item{\code{win1}}{integer, the number of times that
     \code{field1} was preferred to \code{field2}}
   \item{\code{tie}}{integer, the number of times that no
     preference was expressed when comparing \code{field1} and
     \code{field2}}
   \item{\code{win2}}{integer, the number of times that
     \code{field2} was preferred to \code{field1}}
   \item{\code{win1.adj}}{numeric, equal to \code{win1 + tie/2}}
   \item{\code{win2.adj}}{numeric, equal to \code{win2 + tie/2}}
   \item{\code{instrument}}{a factor with 3 levels,
     \code{c("cello", "flute", "violin")}}
 }

The \code{sound.fields$design} data frame has 8 observations (one for
each of the sound fields compared in the experiment)
on the following 3 variables:
\describe{
   \item{\code{a}}{a factor with levels \code{c("0", "1")}, the \emph{direct sound} factor (0 for \emph{obstructed
       sight line}, 1 for \emph{free sight line}); contrasts are sum contrasts}
   \item{\code{b}}{a factor with levels \code{c("0", "1")}, the \emph{reflection} factor (0 for \emph{-26dB},
     1 for \emph{-20dB}); contrasts are sum contrasts}
   \item{\code{c}}{a factor with levels \code{c("0", "1")}, the \emph{reverberation} factor (0 for \emph{-24dB},
     1 for \emph{-20dB}); contrasts are sum contrasts}
 }
 }
\details{
The variables \code{win1.adj} and \code{win2.adj} are provided in order
 to allow a simple way of handling ties (in which a tie counts as half a
 win and half a loss), which is slightly different numerically from the
 Davidson (1970) method that is used by Kousgaard (1984): see the
 examples.
}
\source{
Kousgaard, N. (1984)  Analysis of a Sound Field Experiment by a Model
for Paired Comparisons with Explanatory Variables.  \emph{Scandinavian
  Journal of Statistics} \bold{11}, 51--57.
 }
\references{
  Davidson, R. R. (1970)  Extending the Bradley-Terry model to
  accommodate ties in paired comparison experiments.  \emph{Journal of
    the American Statistical Association} \bold{65}, 317--328.
}
\author{David Firth}
\examples{
##
##  Fit the Bradley-Terry model to data for flutes, using the simple 'add 0.5'
##  method to handle ties:
##
flutes.model <- BTm(cbind(win1.adj, win2.adj), field1, field2, ~ field,
                    id = "field",
                    subset = (instrument == "flute"),
                    data = sound.fields)
##
##  This agrees (after re-scaling) quite closely with the estimates given
##  in Table 3 of Kousgaard (1984):
##
table3.flutes <- c(-0.581, -1.039, 0.347, 0.205, 0.276, 0.347, 0.311, 0.135)
plot(c(0, coef(flutes.model)), table3.flutes)
abline(lm(table3.flutes ~ c(0, coef(flutes.model))))
##
##  Now re-parameterise that model in terms of the factorial effects, as
##  in Table 5 of Kousgaard (1984):
##
flutes.model.reparam <- update(flutes.model,
                               formula = ~ a[field] * b[field] * c[field]
			       )
table5.flutes <- c(.267, .250, -.088, -.294, .062, .009, -0.070)
plot(coef(flutes.model.reparam), table5.flutes)
abline(lm(table5.flutes ~ coef(flutes.model.reparam)))
}
\keyword{datasets}

\name{anova.BTm}
\alias{anova.BTm}
\title{ Compare Nested Bradley Terry Models }
\description{
  Compare nested models inheriting from class \code{"BTm"}. For models
  with no random effects, compute analysis of deviance table, otherwise
  compute Wald tests of additional terms. 
}
\usage{
\method{anova}{BTm}(object, ..., dispersion = NULL, test = NULL)
}
\arguments{
  \item{object, \dots}{ a fitted object of class inheriting from
    \code{"BTm"}, or a list of such objects. }
  \item{dispersion}{ a value for the dispersion. Not implemented for
    models with random effects. }
  \item{test}{ optional character string (partially) matching one of 
    \code{"Chisq"}, \code{"F"} or \code{"Cp"} to specify that p-values
    should be returned.  The Chisq test is a likelihood ratio
    test for models with no random effects, otherwise a Wald
    test. Options \code{"F"} and \code{"Cp"} are only applicable to
    models with no random effects, see \code{\link{stat.anova}}. }
}
\details{
  For models with no random effects, an analysis of deviance table is
  computed using \code{\link{anova.glm}}. Otherwise, Wald tests are
  computed as detailed here.
  
  If a single object is specified, terms are added sequentially and a
  Wald statistic is computed for the extra parameters. If the full model
  includes player covariates and there are players with missing values
  over these covariates, then the \code{NULL} model will include a
  separate ability for these players. If there are
  missing values in any contest-level variables in the full model, the
  corresponding contests will be omitted throughout. The random effects
  structure of the full model is assumed for all sub-models.

  For a list of objects, consecutive pairs of models are compared by
  computing a Wald statistic for the extra parameters in the larger of
  the two models.

  The Wald statistic is always based on the variance-covariance matrix
  of the larger of the two models being compared.
}
\value{
  An object of class \code{"anova"} inheriting from class \code{"data.frame"}.
}
\section{Warning}{
  The comparison between two or more models will only be valid if they
  are fitted to the same dataset. This may be a problem if there are
  missing values and \R's default of \code{na.action = na.omit} is
  used. An error will be returned in this case.

  The same problem will occur when separate abilities have been
  estimated for different subsets of players in the models being
  compared. However no warning is given in this case.
}
\author{ Heather Turner }
\seealso{ \code{\link{BTm}}, \code{\link{add1.BTm}} }
\examples{
attach(flatlizards)
result <- rep(1, nrow(contests))
BTmodel <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
               head.length[..] + (1|..), data = list(contests, predictors),
               trace = TRUE)
anova(BTmodel)
}
\keyword{ models }

\name{GenDavidson}
\alias{GenDavidson}
\title{ Specify a Generalised Davidson Term in a gnm Model Formula}
\description{
  GenDavidson is a function of class \code{"nonlin"} to specify a
  generalised Davidson term in the formula argument to
  \code{\link[gnm]{gnm}}, providing a model for paired comparison data
  where ties are a possible outcome.
}
\usage{
GenDavidson(win, tie, loss, player1, player2, home.adv = NULL,
            tie.max = ~1, tie.mode = NULL, tie.scale = NULL, 
            at.home1 = NULL, at.home2 = NULL)
}
\arguments{
  \item{win}{ a logical vector: \code{TRUE} if player1 wins,
    \code{FALSE} otherwise.
}
  \item{tie}{ a logical vector: \code{TRUE} if the outcome is a tie,
    \code{FALSE} otherwise.
}
  \item{loss}{ a logical vector: \code{TRUE} if player1 loses,
    \code{FALSE} otherwise.
  }
  \item{player1}{ an ID factor specifying the first player in
    each contest, with the same set of levels as \code{player2}.
}
  \item{player2}{ an ID factor specifying the second player in
    each contest, with the same set of levels as \code{player2}.
  }
  \item{home.adv}{ a formula for the paramter corresponding to the
    home advantage effect. If \code{NULL}, no home advantage effect is
    estimated. 
  }
  \item{tie.max}{ a formula for the parameter corresponding to the
    maximum tie probability.
  }
    \item{tie.scale}{ a formula for the parameter corresponding to the
    scale of dependence of the tie probability on the probability
    that \code{player1} wins, given the outcome is not a draw.
}
  \item{tie.mode}{ a formula for the parameter corresponding to the
    location of maximum tie probability, in terms of the probability
    that \code{player1} wins, given the outcome is not a draw.
}
  \item{at.home1}{ a logical vector: \code{TRUE} if \code{player1} is at
    home, \code{FALSE} otherwise.
}
  \item{at.home2}{ a logical vector: \code{TRUE} if \code{player2} is at
    home, \code{FALSE} otherwise.
}
}
\details{
\code{GenDavidson} specifies a generalisation of the Davidson model
(1970) for paired comparisons where a tie is a possible outcome. It is
designed for modelling trinomial counts corresponding to the win/draw/loss
outcome for each contest, which are assumed Poisson conditional on the
total count for each match. Since this total must be one, the expected
counts are equivalently the probabilities for each possible outcome,
which are modelled on the log scale:
\deqn{\log(p(i \textrm{beats} j)_k) = \theta_{ijk} + \log(\mu\alpha_i}{log(p(i beats j)_k) = theta_{ijk} + log(mu * alpha_i)}
\deqn{\log(p(draw)_k) = \theta_{ijk} + \delta + c +
  \sigma(\pi\log(\mu\alpha_i) - (1 - \pi)log(\alpha_j)) +
  (1 - \sigma)(\log(\mu\alpha_i + \alpha_j))}{
  log(p(draw)_k) = theta_{ijk} + log(delta) + c + sigma * (pi * log(mu *
  alpha_i) + (1 - pi) * log(alpha_j)) + (1 - sigma) * log(mu * alpha_i + alpha_j) }
\deqn{\log(p(j \textrm{beats} i)_k) = \theta_{ijk} +
  log(\alpha_j)}{log(p(j beats i)_k) = theta_{ijk} + log(alpha_j)}
Here \eqn{\theta_{ijk}}{theta_{ijk}} is a structural parameter to fix
the trinomial totals; \eqn{\mu}{mu} is the home advantage parameter;
\eqn{\alpha_i}{alpha_i} and \eqn{\alpha_j}{alpha_j} are the abilities of
players \eqn{i} and \eqn{j} respectively; \eqn{c}{c} is a function of
the parameters such that \eqn{\textrm{expit}(\delta)}{plogis(delta)} is
the maximum probability of a tie, \eqn{\sigma}{sigma} scales the
dependence of the probability of a tie on the relative abilities and
\eqn{\pi}{pi} allows for asymmetry in this dependence.

For parameters that must be positive (\eqn{\alpha_i, \sigma, \mu}{alpha,
  sigma, mu}), the log is estimated, while for parameters that must be
  between zero and one (\eqn{\delta, \pi}), the logit is estimated, as
  illustrated in the example. }
\value{
  A list with the anticipated components of a "nonlin" function:
  \item{ predictors }{ the formulae for the different parameters and the
    ID factors for player 1 and player 2. }
  \item{ variables }{ the outcome variables and the \dQuote{at home}
    variables, if specified. 
    }
  \item{ common }{ an index to specify that common effects are to be
    estimated for the players. }
  \item{ term }{ a function to create a deparsed mathematical expression
    of the term, given labels for the predictors.}
  \item{ start }{ a function to generate starting values for the parameters.}
}
\references{
  Davidson, R. R. (1970). On extending the Bradley-Terry model to
  accommodate ties in paired comparison experiments. \emph{Journal of
    the American Statistical Association}, \bold{65}, 317--328.
}
\author{
  Heather Turner
}
\seealso{\code{\link{football}}, \code{\link{plotProportions}}}
\examples{
### example requires gnm
if (require(gnm)) {
    ### convert to trinomial counts
    football.tri <- expandCategorical(football, "result", idvar = "match")
    head(football.tri)

    ### add variable to indicate whether team playing at home
    football.tri$at.home <- !logical(nrow(football.tri))

    ### fit shifted & scaled Davidson model
    ###  - subset to first and last season for illustration
    shifScalDav <- gnm(count ~
        GenDavidson(result == 1, result == 0, result == -1,
                    home:season, away:season, home.adv = ~1,
                    tie.max = ~1, tie.scale = ~1, tie.mode = ~1,
                    at.home1 = at.home,
                    at.home2 = !at.home) - 1,
        eliminate = match, family = poisson, data = football.tri,
        subset = season \%in\% c("2008-9", "2012-13"))

    ### look at coefs
    coef <- coef(shifScalDav)
    ## home advantage
    exp(coef["home.adv"])
    ## max p(tie)
    plogis(coef["tie.max"])
    ## mode p(tie)
    plogis(coef["tie.mode"])
    ## scale relative to Davidson of dependence of p(tie) on p(win|not a draw)
    exp(coef["tie.scale"])

    ### check model fit
    alpha <- names(coef[-(1:4)])
    plotProportions(result == 1, result == 0, result == -1,
                    home:season, away:season,
                    abilities = coef[alpha], home.adv = coef["home.adv"],
                    tie.max = coef["tie.max"], tie.scale = coef["tie.scale"],
                    tie.mode = coef["tie.mode"],
                    at.home1 = at.home, at.home2 = !at.home,
                    data = football.tri, subset = count == 1)
}

### analyse all five seasons
### - takes a little while to run, particularly likelihood ratio tests
\dontrun{
### fit Davidson model
Dav <- gnm(count ~ GenDavidson(result == 1, result == 0, result == -1,
                               home:season, away:season, home.adv = ~1,
                               tie.max = ~1,
                               at.home1 = at.home,
                               at.home2 = !at.home) - 1,
           eliminate = match, family = poisson, data = football.tri)

### fit scaled Davidson model
scalDav <- gnm(count ~ GenDavidson(result == 1, result == 0, result == -1,
                                  home:season, away:season, home.adv = ~1,
                                  tie.max = ~1, tie.scale = ~1,
                                  at.home1 = at.home,
                                  at.home2 = !at.home) - 1,
               eliminate = match, family = poisson, data = football.tri)

### fit shifted & scaled Davidson model
shifScalDav <- gnm(count ~
    GenDavidson(result == 1, result == 0, result == -1,
                home:season, away:season, home.adv = ~1,
                tie.max = ~1, tie.scale = ~1, tie.mode = ~1,
                at.home1 = at.home,
                at.home2 = !at.home) - 1,
    eliminate = match, family = poisson, data = football.tri)

### compare models
anova(Dav, scalDav, shifScalDav, test = "Chisq")

### diagnostic plots
main <- c("Davidson", "Scaled Davidson", "Shifted & Scaled Davidson")
mod <- list(Dav, scalDav, shifScalDav)
names(mod) <- main

## use football.tri data so that at.home can be found,
## but restrict to actual match results
par(mfrow = c(2,2))
for (i in 1:3) {
    coef <- parameters(mod[[i]])
    plotProportions(result == 1, result == 0, result == -1,
                    home:season, away:season,
                    abilities = coef[alpha],
                    home.adv = coef["home.adv"],
                    tie.max = coef["tie.max"],
                    tie.scale = coef["tie.scale"],
                    tie.mode = coef["tie.mode"],
                    at.home1 = at.home,
                    at.home2 = !at.home,
                    main = main[i],
                    data = football.tri, subset = count == 1)
}
}

}
\keyword{ models }
\keyword{ nonlinear }

\name{citations}
\alias{citations}
\docType{data}
\title{ Statistics Journal Citation Data from Stigler (1994) }
\description{
  Extracted from a larger table in Stigler (1994).  Inter-journal
  citation counts for four journals, \dQuote{Biometrika},
  \dQuote{Comm Statist.}, \dQuote{JASA} and \dQuote{JRSS-B}, as used
  on p448 of Agresti (2002).
}
\usage{citations}
\format{
  A 4 by 4 contingency table of citations, cross-classifed by the factors
  \code{cited} and \code{citing} each with levels \code{Biometrika},
  \code{Comm Statist}, \code{JASA}, and \code{JRSS-B}.
 }
\details{
  In the context of paired comparisons, the \sQuote{winner} is the cited
  journal and the \sQuote{loser} is the one doing the citing.  
}
\source{
  Agresti, A. (2002)  \emph{Categorical Data Analysis} (2nd ed).  
  New York: Wiley.
}
\references{ 
  Firth, D. (2005)  Bradley-Terry models in R.  \emph{Journal of
    Statistical Software} \bold{12}(1), 1--12.

  Turner, H. and Firth, D. (2012)  Bradley-Terry models in R: The
  BradleyTerry2 package.  \emph{Journal of Statistical
    Software},  \bold{48}(9), 1--21.  

  Stigler, S. (1994)  Citation patterns in the journals of statistics 
  and probability.  \emph{Statistical Science} \bold{9}, 94--108.
}
\seealso{\code{\link{BTm}}}
\examples{
##  Data as a square table, as in Agresti p448
citations

##
## Convert frequencies to success/failure data:
##
citations.sf <- countsToBinomial(citations)
names(citations.sf)[1:2] <- c("journal1", "journal2")

## Standard Bradley-Terry model fitted to these data
citeModel <-  BTm(cbind(win1, win2), journal1, journal2,
                  data = citations.sf)
}
\keyword{datasets}

\name{residuals.BTm}
\alias{residuals.BTm}
\title{ Residuals from a Bradley-Terry Model }
\description{
  Computes residuals from a model object of class \code{"BTm"}. In
  additional to the usual options for objects inheriting from class
  \code{"glm"}, a \code{"grouped"} option is implemented to compute
  player-specific residuals suitable for diagnostic checking of a
  predictor involving player-level covariates.
}
\usage{
\method{residuals}{BTm}(object, type = c("deviance", "pearson", "working",
  "response", "partial", "grouped"), by = object$id, ...)
}
\arguments{ 
  \item{object}{a model object for which \code{inherits(model, "BTm")} is
    \code{TRUE}.}
  \item{type}{the type of residuals which should be returned.  The
    alternatives are: \code{"deviance"} (default), \code{"pearson"},
    \code{"working"}, \code{"response"}, and \code{"partial"}.}
  \item{by}{the grouping factor to use when \code{type = "grouped"}.}
  \item{...}{arguments to pass on other methods.}
}
\details{
  For \code{type} other than \code{"grouped"} see
  \code{\link{residuals.glm}}.

  For \code{type = "grouped"} the residuals
  returned are weighted means of working residuals, with weights equal
  to the binomial denominators in the fitted model.  These are suitable
  for diagnostic model checking, for example plotting against candidate
  predictors. 
}
\value{
  A numeric vector of length equal to the number of players, with a
  \code{"weights"} attribute.
}
\author{ David Firth and Heather Turner }
\references{
  Firth, D. (2005)  Bradley-Terry models in R.  \emph{Journal of
    Statistical Software} \bold{12}(1), 1--12.

  Turner, H. and Firth, D. (2012)  Bradley-Terry models in R: The
  BradleyTerry2 package.  \emph{Journal of Statistical
    Software},  \bold{48}(9), 1--21.

}
\seealso{ \code{\link{BTm}}, \code{\link{BTabilities}} }
\examples{
##
##  See ?springall
##
springall.model <- BTm(cbind(win.adj, loss.adj),
                       col, row, 
                       ~ flav[..] + gel[..] + 
                       flav.2[..] + gel.2[..] + flav.gel[..] + (1 | ..),
                       data = springall)
res <- residuals(springall.model, type = "grouped")
with(springall$predictors, plot(flav, res))
with(springall$predictors, plot(gel, res))
##  Weighted least-squares regression of these residuals on any variable
##  already included in the model yields slope coefficient zero:
lm(res ~ flav, weights = attr(res, "weights"),
   data = springall$predictors)
lm(res ~ gel, weights = attr(res, "weights"),
   data = springall$predictors)
}
\keyword{ models }

\name{glmmPQL}
\alias{glmmPQL}
\title{
  PQL Estimation of Generalized Linear Mixed Models
}
\description{
  Fits GLMMs with simple random effects structure via Breslow and
  Clayton's PQL algorithm.
}
\usage{
glmmPQL(fixed, random = NULL, family = binomial, data = NULL,
        subset = NULL, weights = NULL, offset = NULL, na.action = NULL,
        start = NULL, etastart = NULL, mustart = NULL,
        control = glmmPQL.control(...), sigma = 0.1,
        sigma.fixed = FALSE, model = TRUE, x = FALSE, contrasts = NULL,
        ...)  
}
\arguments{
  \item{fixed}{
    a formula for the fixed effects.
  }
  \item{random}{
    a design matrix for the random effects, with number of rows equal to
    the length of variables in \code{formula}.
  }
  \item{family}{
    a description of the error distribution and link function to
    be used in the model. This can be a character string naming a
    family function, a family function or the result of a call to
    a family function.  (See \code{\link{family}} for details of family
    functions.)
  }
  \item{data}{
    an optional data frame, list or environment (or object
    coercible by \code{\link{as.data.frame}} to a data frame) containing the
    variables in the model.  If not found in \code{data}, the
    variables are taken from \code{environment(formula)}, typically
    the environment from which \code{glmmPQL} called.
  }
  \item{subset}{
    an optional logical or numeric vector specifying a 
    subset of observations to be used in the fitting process.
  }
  \item{weights}{
    an optional vector of \sQuote{prior weights} to be used in the
    fitting process. 
  }
  \item{offset}{
    an optional numeric vector to be added to the linear predictor
    during fitting. One or more \code{offset} terms can be included
    in the formula instead or as well, and if more than one is
    specified their sum is used.  See \code{\link{model.offset}}.
}
  \item{na.action}{
    a function which indicates what should happen when the data
    contain \code{NA}s.  The default is set by the \code{na.action} setting
    of \code{\link{options}}, and is \code{\link{na.fail}} if that is unset.
  }
  \item{start}{
    starting values for the parameters in the linear predictor.
  }
  \item{etastart}{
    starting values for the linear predictor.
  }
  \item{mustart}{
    starting values for the vector of means.
  }
  \item{control}{
    a list of parameters for controlling the fitting process.
    See the \code{\link{glmmPQL.control}} for details.
  }
  \item{sigma}{
    a starting value for the standard deviation of the random effects.
  }
  \item{sigma.fixed}{
    logical: whether or not the standard deviation of the random effects
    should be fixed at its starting value.
  }
  \item{model}{
    logical: whether or not the model frame should be returned.
  }
  \item{x}{
    logical: whether or not the design matrix for the fixed effects
    should be returned.
  }
  \item{contrasts}{
    an optional list. See the \code{contrasts.arg} argument of
    \code{\link{model.matrix}}.
  }
  \item{\dots}{
    arguments to be passed to \code{\link{glmmPQL.control}}.
  }
}
\details{
  The GLMM is assumed to be of the form
  \ifelse{html}{\out{g(<b>&mu;</b>) = <b>X&beta;</b> + <b>Ze</b>}}{
    \deqn{g(\boldsymbol{\mu}) = \boldsymbol{X\beta} + \boldsymbol{Ze}}{
    g(mu) = X * beta + Z * e}}
  where \eqn{g} is the link function, \ifelse{html}{\out{<b>&mu;</b>}}{\eqn{\boldsymbol{\mu}}{mu}} is the
  vector of means and \ifelse{html}{\out{<b>X</b>, <b>Z</b>}}{\eqn{\boldsymbol{X}, \boldsymbol{Z}}{X,Z}} are
  design matrices for the fixed effects \ifelse{html}{\out{<b>&beta;</b>}}{\eqn{\boldsymbol{\beta}}{beta}}
  and random effects \ifelse{html}{\out{<b>e</b>}}{\eqn{\boldsymbol{e}}{e}}
  respectively. Furthermore the random effects are assumed to be
  i.i.d. \ifelse{html}{\out{N(0, &sigma;<sup>2</sup>)}}{\eqn{N(0,
      \sigma^2)}{N(0, sigma^2)}}.
}
\value{
  An object of class \code{"BTglmmPQL"} which inherits from \code{"glm"}
  and \code{"lm"}:
  \item{coefficients}{ a named vector of coefficients, with a
    \code{"random"} attribute giving the estimated random effects.}
  \item{residuals}{ the working residuals from the final iteration of
    the IWLS loop.}
  \item{random}{the design matrix for the random effects.}
  \item{fitted.values}{ the fitted mean values, obtained by transforming the
    linear predictors by the inverse of the link function.}
  \item{rank}{the numeric rank of the fitted linear model.}
  \item{family}{the \code{family} object used.}
  \item{linear.predictors}{the linear fit on link scale.}
  \item{deviance}{up to a constant, minus twice the maximized
    log-likelihood.}
  \item{aic}{a version of Akaike's \emph{An Information Criterion}, minus twice
    the maximized log-likelihood plus twice the number of
    parameters, computed by the \code{aic} component of the
    family.}
  \item{null.deviance}{the deviance for the null model, comparable with
    \code{deviance}.}
  \item{iter}{the numer of iterations of the PQL algorithm.}
  \item{weights}{the working weights, that is the weights in the final
    iteration of the IWLS loop.}
  \item{prior.weights}{the weights initially supplied, a vector of
    \code{1}'s if none were.}
  \item{df.residual}{the residual degrees of freedom.}
  \item{df.null}{the residual degrees of freedom for the null model.}
  \item{y}{if requested (the default) the \code{y} vector used. (It is a
    vector even for a binomial model.)}
  \item{x}{if requested, the model matrix.}
  \item{model}{if requested (the default), the model frame.}
  \item{converged}{logical. Was the PQL algorithm judged to have
    converged?}
  \item{call}{the matched call.}
  \item{formula}{the formula supplied.}
  \item{terms}{the \code{terms} object used.}
  \item{data}{the \code{data} argument used.}
  \item{offset}{the offset vector used.}
  \item{control}{the value of the \code{control} argument used.}
  \item{contrasts}{(where relevant) the contrasts used.}
  \item{xlevels}{(where relevant) a record of the levels of the factors used
    in fitting.}
  \item{na.action}{(where relevant) information returned by \code{model.frame} on
          the special handling of \code{NA}s.}
  \item{sigma}{the estimated standard deviation of the random effects}
  \item{sigma.fixed}{logical: whether or not \code{sigma} was fixed}
  \item{varFix}{the variance-covariance matrix of the fixed effects}
  \item{varSigma}{the variance of \code{sigma}}
}
\references{
  Breslow, N. E. and Clayton, D. G. (1993) Approximate inference
  in Generalized Linear Mixed Models. \emph{Journal of the American
    Statistical Association} \bold{88}(421),
  9--25.
  
  Harville, D. A. (1977) Maximum likelihood approaches to variance
  component estimation and to related problems. \emph{Journal of the
    American Statistical Association} \bold{72}(358), 320--338.
}
\author{
Heather Turner
}
\seealso{
  \code{\link{predict.BTglmmPQL}},\code{\link{glmmPQL.control}},\code{\link{BTm}}
}
\examples{
###############################################
## Crowder seeds example from Breslow & Clayton
###############################################
attach(seeds)

summary(glmmPQL(cbind(r, n - r) ~ seed + extract,
        random = diag(length(r)),
        family = binomial, data = seeds))

summary(glmmPQL(cbind(r, n - r) ~ seed*extract,
                random = diag(length(r)),
                family = binomial, data = seeds))

}
\keyword{ models }

\name{chameleons}
\alias{chameleons}
\docType{data}
\title{
Male Cape Dwarf Chameleons: Measured Traits and Contest Outcomes
}
\description{
Data as used in the study by Stuart-Fox et al. (2006).  Physical
measurements made on 35 male Cape dwarf chameleons, and the results of
106 inter-male contests.
}
\usage{chameleons}
\format{
  A list containing three data frames: \code{chameleons$winner},
  \code{chameleons$loser} and \code{chameleons$predictors}.

  The \code{chameleons$winner} and \code{chameleons$loser} data frames
  each have 106 observations (one per contest) on the following 4 variables:
  \describe{
    \item{\code{ID}}{a factor with 35 levels \code{C01}, \code{C02},
      ... , \code{C43}, the identity of the winning (or losing) male in
      each contest}
    \item{\code{prev.wins.1}}{integer (values 0 or 1), did the
      winner/loser of this
      contest win in an immediately previous contest?}
    \item{\code{prev.wins.2}}{integer (values 0, 1 or 2), how many of his
      (maximum) previous 2 contests did each male win?}
    \item{\code{prev.wins.all}}{integer, how many previous contests has
      each male won?}
   }

   The \code{chameleons$predictors} data frame has 35 observations, one
   for each male involved in the contests, on the following 7 variables:
   \describe{
     \item{\code{ch.res}}{numeric, residuals of casque height regression
       on \code{SVL}, i.e. relative height of the bony part on the top
       of the chameleons' heads}
     \item{\code{jl.res}}{numeric, residuals of jaw length regression
       on \code{SVL}}
     \item{\code{tl.res}}{numeric, residuals of tail length regression
       on \code{SVL}}
     \item{\code{mass.res}}{numeric, residuals of body mass regression
       on \code{SVL} (body condition)}
     \item{\code{SVL}}{numeric, snout-vent length (body size)}
     \item{\code{prop.main}}{numeric, proportion (arcsin transformed)
       of area of the flank occupied by the main pink patch on the flank}
     \item{\code{prop.patch}}{numeric, proportion (arcsin transformed)
       of area of the flank occupied by the entire flank patch}
     }
}
\details{
The published paper mentions 107 contests, but only 106 contests are
included here.  Contest number 16 was
deleted from the data used to fit the models, because it involved a male
whose predictor-variables were incomplete (and it was the only contest
involving that lizard, so it is uninformative).
}
\source{
  The data were obtained by Dr Devi Stuart-Fox,
  \url{http://www.zoology.unimelb.edu.au/research/groups/animal/labs/stuart-fox/index.php},
   and they are reproduced here with her kind permission.

  These are the same data that were used in

  Stuart-Fox, D. M., Firth, D., Moussalli, A. and Whiting, M. J. (2006)
  Multiple signals in chameleon contests: designing and analysing animal
  contests as a tournament. \emph{Animal Behaviour} \bold{71},
  1263--1271.
}
\author{David Firth}
\examples{
##
## Reproduce Table 3 from page 1268 of the above paper:
##
summary(chameleon.model <- BTm(player1 = winner, player2 = loser,
  formula = ~ prev.wins.2 + ch.res[ID] + prop.main[ID] + (1|ID), id = "ID",
  data = chameleons))
##
## Note that, although a per-chameleon random effect is specified as in the
## above [the term "+ (1|ID)"], the estimated variance for that random
## effect turns out to be zero in this case.  The "prior experience"
## effect ["+ prev.wins.2"] in this analysis has explained most of the
## variation, leaving little for the ID-specific predictors to do.
## Despite that, two of the ID-specific predictors do emerge as
## significant.
##
## Test whether any of the other ID-specific predictors has an effect:
##
add1(chameleon.model, ~ . + jl.res[ID] + tl.res[ID] + mass.res[ID] +
  SVL[ID] + prop.patch[ID]) 
}
\keyword{datasets}

\name{springall}
\alias{springall}
\docType{data}
\title{
Springall (1973) Data on Subjective Evaluation of Flavour Strength
}
\description{
Data from Section 7 of the paper by Springall (1973) on Bradley-Terry
response surface modelling.  An experiment to assess the effects of gel
and flavour concentrations on the subjective assessment of flavour strength
by pair comparisons.
}
\usage{springall}
\format{
A list containing two data frames, \code{springall$contests} and \code{springall$predictors}.

The \code{springall$contests} data frame has 36 observations (one for each possible pairwise comparison of the 9 treatments) on the following 7 variables:
 \describe{
	\item{\code{row}}{a factor with levels \code{1:9}, the row number in Springall's dataset}
        \item{\code{col}}{a factor with levels \code{1:9}, the column number in Springall's dataset}
        \item{\code{win}}{integer, the number of wins for column treatment over row treatment}
        \item{\code{loss}}{integer, the number of wins for row treatment over column treatment}
        \item{\code{tie}}{integer, the number of ties between row and column treatments}
        \item{\code{win.adj}}{numeric, equal to \code{win + tie/2}}
        \item{\code{loss.adj}}{numeric, equal to \code{loss + tie/2}}
	}

The \code{predictors} data frame has 9 observations (one for each treatment) on the following 5 variables:
 \describe{
	\item{\code{flav}}{numeric, the flavour concentration}
 	\item{\code{gel}}{numeric, the gel concentration}
  	\item{\code{flav.2}}{numeric, equal to \code{flav^2}}
	\item{\code{gel.2}}{numeric, equal to \code{gel^2}}
        \item{\code{flav.gel}}{numeric, equal to \code{flav * gel}}
	}
}
\details{
The variables \code{win.adj} and \code{loss.adj} are provided in order to allow a simple way of handling ties (in which a tie counts as half a win and half a loss), which is slightly different numerically from the Rao and Kupper (1967) model that Springall (1973) uses.
}
\source{
Springall, A (1973)  Response surface fitting using a generalization of the Bradley-Terry paired comparison method.  \emph{Applied Statistics} \bold{22}, 59--68.
}
\references{
  Rao, P. V. and Kupper, L. L. (1967)  Ties in paired-comparison
  experiments: a generalization of the Bradley-Terry model.
  \emph{Journal of the American Statistical Association}, \bold{63},
  194--204. 
}
\author{David Firth}
\examples{
##
## Fit the same response-surface model as in section 7 of 
## Springall (1973).
##
## Differences from Springall's fit are minor, arising from the 
## different treatment of ties.
##
## Springall's model in the paper does not include the random effect.  
## In this instance, however, that makes no difference: the random-effect 
## variance is estimated as zero.
##
summary(springall.model <- BTm(cbind(win.adj, loss.adj), col, row, 
                               ~ flav[..] + gel[..] + 
                                 flav.2[..] + gel.2[..] + flav.gel[..] +
                                 (1 | ..),
                               data = springall))
}
\keyword{datasets}

\name{football}
\alias{football}
\docType{data}
\title{
English Premier League Football Results 2008/9 to 2012/13
}
\description{
The win/lose/draw results for five seasons of the English Premier League
football results, from 2008/9 to 2012/13
}
\usage{football}
\format{
  A data frame with 1881 observations on the following 4 variables.
  \describe{
    \item{\code{season}}{a factor with levels \code{2008-9}, \code{2009-10}, \code{2010-11}, \code{2011-12}, \code{2012-13}}
    \item{\code{home}}{a factor specifying the home team, with 29 levels
      \code{Ars} (Arsenal), ... , \code{Wol} (Wolverhampton)}
    \item{\code{away}}{a factor specifying the away team, with the same
      levels as \code{home}.}
    \item{\code{result}}{a numeric vector giving the result for the home
      team: 1 for a win, 0 for a draw, -1 for a loss.}
  }
}
\details{
  In each season, there are 20 teams, each of which plays one home game
  and one away game against all the other teams in the league. The results
  in 380 games per season.
}
\source{
  These data were downloaded from http://soccernet.espn.go.com in
  2013. The site has since moved and the new site does not appear to
  have an equivalent source. 
}
\references{
  Davidson, R. R. (1970). On extending the Bradley-Terry model to
  accommodate ties in paired comparison experiments. \emph{Journal of
    the American Statistical Association}, \bold{65}, 317--328.
}
\seealso{\code{\link{GenDavidson}}}
\examples{
### example requires gnm
if (require(gnm)) {
    ### convert to trinomial counts
    football.tri <- expandCategorical(football, "result", idvar = "match")
    head(football.tri)

    ### add variable to indicate whether team playing at home
    football.tri$at.home <- !logical(nrow(football.tri))

    ### fit Davidson model for ties
    ###  - subset to first and last season for illustration
    Davidson <- gnm(count ~
        GenDavidson(result == 1, result == 0, result == -1,
                    home:season, away:season,
                    home.adv = ~1, tie.max = ~1,
                    at.home1 = at.home, at.home2 = !at.home) - 1,
        eliminate = match, family = poisson, data = football.tri,
        subset = season \%in\% c("2008-9", "2012-13"))

    ### see ?GenDavidson for further analysis
}
}
\keyword{datasets}

\name{add1.BTm}
\alias{add1.BTm}
\alias{drop1.BTm}
\title{ Add or Drop Single Terms to/from a Bradley Terry Model }
\description{
  Add or drop single terms within the limit specified by the
  \code{scope} argument. For models with no random effects, compute an
  analysis of deviance table, otherwise compute the Wald statistic of
  the parameters that have been added to or dropped from the model.
}
\usage{
\method{add1}{BTm}(object, scope, scale = 0,
  test = c("none", "Chisq", "F"), x = NULL, ...)

\method{drop1}{BTm}(object, scope, scale = 0,
  test = c("none", "Chisq", "F"), ...)
}

\arguments{
  \item{object}{ a fitted object of class inheriting from \code{"BTm"}. }
  \item{scope}{ a formula specifying the model including all terms to be
    considered for adding or dropping. }
  \item{scale}{ an estimate of the dispersion. Not implemented for
    models with random effects.   }
  \item{test}{ should a p-value be returned? The F test is only
    appropriate for models with no random effects for which the
    dispersion has been estimated. The Chisq test is a likelihood ratio
    test for models with no random effects, otherwise a Wald test. }
  \item{x}{ a model matrix containing columns for all terms in the
    scope.  Useful if \code{add1} is to be called repeatedly.
    \bold{Warning:} no checks are done on its validity. }
  \item{\dots}{ further arguments passed to \code{\link{add1.glm}}. }
}
\details{
  The hierarchy is respected when considering terms to be added or
  dropped: all main effects contained in a second-order interaction must
  remain, and so on.

  In a scope formula \samp{.} means \sQuote{what is already there}.

  For \code{drop1}, a missing \code{scope} is taken to mean that all
  terms in the model may be considered for dropping.

  If \code{scope} includes player covariates and there are players
  with missing values over these covariates, then a separate ability
  will be estimated for these players in \emph{all} fitted
  models. Similarly if there are missing values in any contest-level
  variables in \code{scope}, the corresponding contests will be omitted
  from all models.

  If \code{formula} includes random effects, the same random effects
  structure will apply to all models.
}
\value{
  An object of class \code{"anova"} summarizing the differences in fit
  between the models.
}
\author{ Heather Turner }
\seealso{ \code{\link{BTm}}, \code{\link{anova.BTm}} }
\examples{
attach(flatlizards)
result <- rep(1, nrow(contests))
BTmodel1 <- BTm(result, winner, loser,
                ~ throat.PC1[..] + throat.PC3[..] + (1|..),
                data = list(contests, predictors),
                tol = 1e-4, sigma = 2, trace = TRUE)

drop1(BTmodel1)

add1(BTmodel1, ~ . + head.length[..] + SVL[..], test = "Chisq")

BTmodel2 <- update(BTmodel1, formula = ~ . + head.length[..])

drop1(BTmodel2, test = "Chisq")

}
\keyword{ models }

\name{plotProportions}
\alias{plotProportions}
\title{
Plot Proportions of Tied Matches and Non-tied Matches Won
}
\description{
Plot proportions of tied matches and non-tied matches won by the first
player, within matches binned by the relative player ability, as
expressed by the probability that the first player wins, given the match
is not a tie. Add fitted lines for each set of matches, as given by the
generalized Davidson model.
}
\usage{
plotProportions(win, tie = NULL, loss, player1, player2, abilities = NULL,
    home.adv = NULL, tie.max = NULL, tie.scale = NULL, tie.mode = NULL,
    at.home1 = NULL, at.home2 = NULL, data = NULL, subset = NULL,
    bin.size = 20, xlab = "P(player1 wins | not a tie)",
    ylab = "Proportion", legend = NULL, col = 1:2, ...)
}
\arguments{
  \item{win}{ a logical vector: \code{TRUE} if player1 wins,
    \code{FALSE} otherwise.
}
  \item{tie}{ a logical vector: \code{TRUE} if the outcome is a tie,
    \code{FALSE} otherwise (\code{NULL} if there are no ties).
}
  \item{loss}{ a logical vector: \code{TRUE} if player1 loses,
    \code{FALSE} otherwise.
}
  \item{player1}{ an ID factor specifying the first player in
    each contest, with the same set of levels as \code{player2}.
}
  \item{player2}{ an ID factor specifying the second player in
    each contest, with the same set of levels as \code{player2}.
}
  \item{abilities}{ the fitted abilities from a
    generalized Davidson model (or a Bradley-Terry model).
}
  \item{home.adv}{ if applicable, the fitted home advantage parameter from a
    generalized Davidson model (or a Bradley-Terry model).
}
  \item{tie.max}{ the fitted parameter from a generalized Davidson
    model corresponding to the  maximum tie probability.
}
  \item{tie.scale}{ if applicable, the fitted parameter from a
    generalized Davidson model corresponding to the
    scale of dependence of the tie probability on the probability
    that \code{player1} wins, given the outcome is not a draw.
}
  \item{tie.mode}{ if applicable, the fitted parameter from a
    generalized Davidson model corresponding to the
    location of maximum tie probability, in terms of the probability
    that \code{player1} wins, given the outcome is not a draw.
}
  \item{at.home1}{ a logical vector: \code{TRUE} if \code{player1} is at
    home, \code{FALSE} otherwise.
}
  \item{at.home2}{ a logical vector: \code{TRUE} if \code{player2} is at
    home, \code{FALSE} otherwise.
}
  \item{data}{  an optional data frame providing variables required by the
    model, with one observation per match.
}
  \item{subset}{ an optional logical or numeric vector specifying a 
    subset of observations to include in the plot.
}
  \item{bin.size}{ the approximate number of matches in each bin.
}
  \item{xlab}{ the label to use for the x-axis.
}
  \item{ylab}{ the label to use for the y-axis.
  }
  \item{legend}{ text to use for the legend.
  }
  \item{col}{ a vector specifying colours to use for the proportion of
    non-tied matches won and the proportion of tied matches.
    }
  \item{\dots}{ further arguments passed to plot.
}
}
\details{
  If \code{home.adv} is specified, the results are re-ordered if
  necessary so that the home player comes first; any matches played on
  neutral ground are omitted.

  First the probability that the first player wins given that the match
  is not a tie is computed:
  \deqn{expit(home.adv + abilities[player1] - abilities[player2])}
  where \code{home.adv} and \code{abilities} are parameters from a
  generalized Davidson model that have been estimated on the log scale.

  The matches are then binned according to this probability, grouping
  together matches with similar relative ability between the first player and the
  second player. Within each bin, the proportion of tied matches is
  computed and these proportions are plotted against the mid-point of
  the bin. Then the bins are re-computed omitting the tied games and the
  proportion of non-tied matches won by the first player is found and
  plotted against the new mid-point.

  Finally curves are added for the probability of a tie and the
  conditional probability of win given the match is not a tie, under a
  generalized Davidson model with parameters as specified by
  \code{tie.max}, \code{tie.scale} and \code{tie.mode}.

  The function can also be used to plot the proportions of wins along
  with the fitted probability of a win under the Bradley-Terry model. 
}
\note{
  This function is designed for single match outcomes, therefore data
  aggregated over player pairs will need to be expanded.
}
\value{
  A list of data frames:
  \item{win}{ a data frame comprising \code{prop.win}, the proportion of
  non-tied matches won by the first player in each bin and
  \code{bin.win}, the mid-point of each bin. }
  \item{tie}{ (when ties are present) a data frame comprising
  \code{prop.tie}, the proportion of tied matches in each bin and
  \code{bin.tie}, the mid-point of each bin. }
}
\author{
  Heather Turner
}
\seealso{
   \code{\link{GenDavidson}}, \code{\link{BTm}}
}
\examples{
#### A Bradley-Terry example using icehockey data

## Fit the standard Bradley-Terry model, ignoring home advantage
standardBT <- BTm(outcome = result,
                  player1 = visitor, player2 = opponent,
                  id = "team", data = icehockey)

## comparing teams on a "level playing field"
levelBT <- BTm(result,
               data.frame(team = visitor, home.ice = 0),
               data.frame(team = opponent, home.ice = home.ice),
               ~ team + home.ice,
               id = "team", data = icehockey)

## compare fit to observed proportion won
## exclude tied matches as not explicitly modelled here
par(mfrow = c(1, 2))
plotProportions(win = result == 1, loss = result == 0,
                player1 = visitor, player2 = opponent,
                abilities = BTabilities(standardBT)[,1],
                data = icehockey, subset = result != 0.5,
                main = "Without home advantage")

plotProportions(win = result == 1, loss = result == 0,
                player1 = visitor, player2 = opponent,
                home.adv = coef(levelBT)["home.ice"],
                at.home1 = 0, at.home2 = home.ice,
                abilities = BTabilities(levelBT)[,1],
                data = icehockey, subset = result != 0.5,
                main = "With home advantage")

#### A generalized Davidson example using football data
if (require(gnm)) {

    ## subset to first and last season for illustration
    football <- subset(football, season \%in\% c("2008-9", "2012-13"))

    ## convert to trinomial counts
    football.tri <- expandCategorical(football, "result", idvar = "match")

    ## add variable to indicate whether team playing at home
    football.tri$at.home <- !logical(nrow(football.tri))

    ## fit Davidson model
    Dav <- gnm(count ~ GenDavidson(result == 1, result == 0, result == -1,
                                   home:season, away:season, home.adv = ~1,
                                   tie.max = ~1,
                                   at.home1 = at.home,
                                   at.home2 = !at.home) - 1,
               eliminate = match, family = poisson, data = football.tri)

    ## fit shifted & scaled Davidson model
    shifScalDav <- gnm(count ~
        GenDavidson(result == 1, result == 0, result == -1,
                    home:season, away:season, home.adv = ~1,
                    tie.max = ~1, tie.scale = ~1, tie.mode = ~1,
                    at.home1 = at.home,
                    at.home2 = !at.home) - 1,
        eliminate = match, family = poisson, data = football.tri)

    ## diagnostic plots
    main <- c("Davidson", "Shifted & Scaled Davidson")
    mod <- list(Dav, shifScalDav)
    names(mod) <- main
    alpha <- names(coef(Dav)[-(1:2)])

    ## use football.tri data so that at.home can be found,
    ## but restrict to actual match results
    par(mfrow = c(1,2))
    for (i in 1:2) {
        coef <- parameters(mod[[i]])
        plotProportions(result == 1, result == 0, result == -1,
                        home:season, away:season,
                        abilities = coef[alpha],
                        home.adv = coef["home.adv"],
                        tie.max = coef["tie.max"],
                        tie.scale = coef["tie.scale"],
                        tie.mode = coef["tie.mode"],
                        at.home1 = at.home,
                        at.home2 = !at.home,
                        main = main[i],
                        data = football.tri, subset = count == 1)
    }
}
}
\keyword{ models }
\keyword{ nonlinear }

\name{seeds}
\alias{seeds}
\docType{data}
\title{
Seed Germination Data from Crowder (1978)
}
\description{
Data from Crowder(1978) giving the proportion of seeds germinated for
21 plates that were arranged according to a 2x2 factorial layout by seed
variety and type of root extract.
}
\usage{seeds}
\format{
  A data frame with 21 observations on the following 4 variables.
  \describe{
    \item{\code{r}}{the number of germinated seeds.}
    \item{\code{n}}{the total number of seeds.}
    \item{\code{seed}}{the seed variety.}
    \item{\code{extract}}{the type of root extract.}
  }
}
\source{
Crowder, M. (1978) Beta-Binomial ANOVA for proportions. \emph{Applied Statistics}, \bold{27}, 34--37. 
}
\references{
  Breslow, N. E. and Clayton, D. G. (1993) Approximate inference
  in Generalized Linear Mixed Models. \emph{Journal of the American
    Statistical Association}, \bold{88}(421), 9--25.
}
\seealso{\code{\link{glmmPQL}}}
\examples{
attach(seeds)

summary(glmmPQL(cbind(r, n - r) ~ seed + extract,
        random = diag(length(r)),
        family = binomial))
}
\keyword{datasets}

\name{icehockey}
\alias{icehockey}
\docType{data}
\title{
   College Hockey Men's Division I 2009-10 results
}
\description{
  Game results from American College Hockey Men's Division I composite
  schedule 2009-2010.
}
\usage{icehockey}
\format{
  A data frame with 1083 observations on the following 6 variables.
  \describe{
    \item{\code{date}}{a numeric vector}
    \item{\code{visitor}}{a factor with 58 levels \code{Alaska Anchorage} ... \code{Yale}}
    \item{\code{v_goals}}{a numeric vector}
    \item{\code{opponent}}{a factor with 58 levels \code{Alaska Anchorage} ... \code{Yale}}
    \item{\code{o_goals}}{a numeric vector}
    \item{\code{conference}}{a factor with levels \code{AH}, \code{CC},
      \code{CH}, \code{EC}, \code{HE}, \code{NC}, \code{WC}}
    \item{\code{result}}{a numeric vector: 1 if visitor won, 0.5 for a
      draw and 0 if visitor lost}
    \item{\code{home.ice}}{a logical vector: 1 if opponent on home ice,
      0 if game on neutral ground}
  }
}
\details{
  The Division I ice hockey teams are arranged in six conferences:
  Atlantic Hockey, Central Collegiate Hockey Association, College Hockey
  America, ECAC Hockey, Hockey East and the Western Collegiate Hockey
  Association, all part of the National Collegiate Athletic
  Association. The composite schedule includes within conference games
  and between conference games.

  The data set here contains only games from the regular season, the
  results of which determine the teams that play in the NCAA national
  tournament. There are six automatic bids that go to the conference
  tournament champions, the remaining 10 teams are selected based upon
  ranking under the NCAA's system of pairwise comparisons
  (\url{http://www.collegehockeynews.com/info/?d=pwcrpi}). Some have
  argued that Bradley-Terry rankings would be fairer
  (\url{http://www.collegehockeynews.com/info/?d=krach}).
}
\source{
  \url{http://www.collegehockeystats.net/0910/schedules/men}
}
\references{
  Schlobotnik, J. Build your own rankings.
  \url{http://slack.net/~whelan/tbrw/2010/rankings.diy.shtml}

  \url{http://www.collegehockeynews.com}

  Selections for 2010 NCAA tournament.
  \url{http://sports.espn.go.com/ncaa/news/story?id=5012918}
}
\examples{
### Fit the standard Bradley-Terry model
standardBT <- BTm(outcome = result,
    player1 = visitor, player2 = opponent,
    id = "team", data = icehockey)

## Bradley-Terry abilities
abilities <- exp(BTabilities(standardBT)[,1])

## Compute round-robin winning probability and KRACH ratings
## (scaled abilities such that KRACH = 100 for a team with
## round-robin winning probability of 0.5)
rankings <- function(abilities){
    probwin <- abilities/outer(abilities, abilities, "+")
    diag(probwin) <- 0
    nteams <- ncol(probwin)
    RRWP <- rowSums(probwin)/(nteams - 1)
    low <- quantile(abilities, 0.45)
    high <- quantile(abilities, 0.55)
    middling <- uniroot(function(x) {sum(x/(x+abilities)) - 0.5*nteams},
                        lower = low, upper = high)$root
    KRACH <- abilities/middling*100
    cbind(KRACH, RRWP) 
}

ranks <- rankings(abilities)
## matches those produced by Joe Schlobotnik's Build Your Own Rankings 
head(signif(ranks, 4)[order(ranks[,1], decreasing = TRUE),])

## At one point the NCAA rankings gave more credit for wins on
## neutral/opponent's ground. Home ice effects are easily
## incorporated into the Bradley-Terry model, comparing teams
## on a "level playing field"
levelBT <- BTm(result,
               data.frame(team = visitor, home.ice = 0),
               data.frame(team = opponent, home.ice = home.ice),
               ~ team + home.ice,
               id = "team", data = icehockey)

abilities <- exp(BTabilities(levelBT)[,1])
ranks2 <- rankings(abilities)

## Look at movement between the two rankings
change <- factor(rank(ranks2[,1]) - rank(ranks[,1]))
barplot(xtabs(~change), xlab = "Change in Rank", ylab = "No. Teams")

## Take out regional winners and look at top 10
regional <- c("RIT", "Alabama-Huntsville", "Michigan", "Cornell", "Boston College",
              "North Dakota")

ranks <- ranks[!rownames(ranks) \%in\% regional]
ranks2 <- ranks2[!rownames(ranks2) \%in\% regional]

## compare the 10 at-large selections under both rankings
## with those selected under NCAA rankings
cbind(names(sort(ranks, decr = TRUE)[1:10]),
      names(sort(ranks2, decr = TRUE)[1:10]),
      c("Miami", "Denver", "Wisconsin", "St. Cloud State",
        "Bemidji State", "Yale", "Northern Michigan", "New Hampshire",
        "Alsaka", "Vermont"))

}
\keyword{datasets}

\name{BTm}
\alias{BTm}
\title{ Bradley-Terry Model and Extensions }
\description{
  Fits Bradley-Terry models for pair comparison data, including
  models with structured scores, order effect and missing covariate data.  
  Fits by either maximum likelihood or maximum penalized likelihood
  (with Jeffreys-prior penalty) when abilities are modelled exactly, or by
  penalized quasi-likelihood when abilities are modelled by covariates.
}
\usage{
BTm(outcome, player1, player2, formula = NULL, id = "..", 
    separate.ability = NULL, refcat = NULL, family = binomial, 
    data = NULL, weights = NULL, subset = NULL, na.action = NULL, 
    start = NULL, etastart = NULL, mustart = NULL, offset = NULL, 
    br = FALSE, model = TRUE, x = FALSE, contrasts = NULL, ...) 
}
\arguments{
  \item{outcome}{ the binomial response: either a numeric vector, a
    factor in which the first level denotes failure and all others
    success, or a two-column matrix with the columns giving the numbers
    of successes and failures. }
  \item{player1}{ either an ID factor specifying the first player in
    each contest, or a data.frame containing such a factor and possibly
    other contest-level variables that are specific to the first player. If
    given in a data.frame, the ID factor must have the name given in the
    \code{id} argument. If a factor is specified it will be used to
    create such a data.frame. } 
  \item{player2}{ an object corresponding to that given in
    \code{player1} for the second player in each contest, with identical
    structure -- in particular factors must have identical levels. }
  \item{formula}{ a formula with no left-hand-side, specifying the
    model for player ability. See details for more information. }
  \item{id}{ the name of the ID factor. } 
  \item{separate.ability}{(if \code{formula} does not include the ID
    factor as a separate term) a character vector giving the names of
    players whose abilities are to be modelled individually rather than
    using the specification given by \code{formula}. }
  \item{refcat}{(if \code{formula} includes the ID factor as a separate
    term) a character specifying which player to use as a
    reference, with the first level of the ID factor as the
    default. Overrides any other contrast specification for the ID
    factor.} 
  \item{family}{ a description of the error distribution and link
    function to be used in the model. Only the binomial family is
    implemented, with either\code{"logit"}, \code{"probit"} , or \code{"cauchit"} link. (See
    \code{\link{family}} for details of family functions.)}
  \item{data}{ an optional object providing data required by the
    model. This may be a single data frame of contest-level data or a list of
    data frames. Names of data frames are ignored unless they refer to
    data frames specified by \code{player1} and \code{player2}.The rows
    of data frames that do not contain contest-level data must
    correspond to the levels of a factor used for indexing. Objects are
    searched for first in the \code{data} object if provided, then in
    the environment of \code{formula}. If \code{data} is a list, the
    data frames are searched in the order given.} 
  \item{weights}{ an optional numeric vector of \sQuote{prior weights}.}
  \item{subset}{  an optional logical or numeric vector specifying a 
    subset of observations to be used in the fitting process. }
  \item{na.action}{ a function which indicates what should happen when
    any contest-level variables contain \code{NA}s. The default is the
    \code{na.action} setting of \code{options}. See details for the handling
    of missing values in other variables. } 
  \item{start}{ a vector of starting values for the fixed effects.}
  \item{etastart}{ a vector of starting values for the linear
    predictor. }
  \item{mustart}{ a vector of starting values for the vector of means.}
  \item{offset}{ an optional offset term in the model. A vector of
    length equal to the number of contests.}
  \item{br}{ logical.  If \code{TRUE} fitting will be by penalized
    maximum likelihood as in Firth (1992, 1993), using
    \code{\link[brglm]{brglm}}, rather than maximum likelihood using
    \code{\link{glm}}, when abilities are modelled exactly or when the
    abilities are modelled by covariates and the variance of the
    random effects is estimated as zero. }
  \item{model}{logical: whether or not to return the model frame.}
  \item{x}{logical: whether or not to return the design matrix for
    the fixed effects.}
  \item{contrasts}{an optional list specifying contrasts for the factors
    in \code{formula}. See the \code{contrasts.arg} of
    \code{\link{model.matrix}}.}  
  \item{\dots}{other arguments for fitting function (currently either
    \code{\link{glm}}, \code{\link[brglm]{brglm}}, or \code{\link{glmmPQL}}) }
}
\details{
  In each comparison to be modelled there is a 'first player' and a
  'second player' and it is assumed that one player wins while the other
  loses (no allowance is made for tied comparisons).

  The \code{\link{countsToBinomial}} function is provided to convert a
  contingency table of wins into a data frame of wins and losses for
  each pair of players.

  The \code{formula} argument specifies the model for player ability and
  applies to both the first player and the second player in each
  contest. If \code{NULL} a separate ability is estimated for each
  player, equivalent to setting \code{formula = reformulate(id)}.

  Contest-level variables can be specified in the formula in the usual
  manner, see \code{\link{formula}}. Player covariates should
  be included as variables indexed by \code{id}, see examples. Thus
  player covariates must be ordered according to the levels of the ID
  factor.

  If \code{formula} includes player covariates and there are players
  with missing values over these covariates, then a separate ability
  will be estimated for those players.

  When player abilities are modelled by covariates, then random player
  effects should be added to the model. These should be specified in the
  formula using the vertical bar notation of \code{\link[lme4]{lmer}},
  see examples.

  When specified, it is assumed that random player effects arise from a \eqn{N(0,
    \sigma^2)}{N(0, sigma^2)} distribution and model parameters,
  including \eqn{\sigma}{sigma}, are estimated using PQL (Breslow and
  Clayton, 1993) as implemented in the \code{\link{glmmPQL}} function.
}
\value{
  An object of class \code{c("BTm", "x")}, where \code{"x"} is the class
  of object returned by the model fitting function (e.g. \code{glm}).
  Components are as for objects of class \code{"x"}, with additionally
  \item{id}{the \code{id} argument.}
  \item{separate.ability}{the \code{separate.ability} argument.}
  \item{refcat}{the \code{refcat} argument.}
  \item{player1}{a data frame for the first player containing the ID
    factor and any player-specific contest-level variables.}
  \item{player2}{a data frame corresponding to that for \code{player1}.}
  \item{assign}{a numeric vector indicating which coefficients
    correspond to which terms in the model.}
  \item{term.labels}{labels for the model terms.}
  \item{random}{for models with random effects, the design matrix for the
   random effects. }
}
\seealso{
  \code{\link{countsToBinomial}}, \code{\link{glmmPQL}},
    \code{\link{BTabilities}}, \code{\link{residuals.BTm}},
    \code{\link{add1.BTm}}, \code{\link{anova.BTm}}
  }
\references{

  Agresti, A. (2002)  \emph{Categorical Data Analysis} (2nd ed).  New
  York: Wiley.
  
  Firth, D. (1992)  Bias reduction, the Jeffreys prior and GLIM. In 
  \emph{Advances in GLIM and Statistical Modelling}, Eds. Fahrmeir, L.,
  Francis, B. J., Gilchrist, R. and Tutz, G., pp91--100. New York:
  Springer. 
  
  Firth, D. (1993)  Bias reduction of maximum likelihood estimates.
  \emph{Biometrika} \bold{80}, 27--38.
  
  Firth, D. (2005)  Bradley-Terry models in R.  \emph{Journal of
    Statistical Software},  \bold{12}(1), 1--12.
  
  Stigler, S. (1994)  Citation patterns in the journals of statistics 
  and probability.  \emph{Statistical Science} \bold{9}, 94--108.

  Turner, H. and Firth, D. (2012)  Bradley-Terry models in R: The
  BradleyTerry2 package.  \emph{Journal of Statistical
    Software},  \bold{48}(9), 1--21.
}
\author{ Heather Turner, David Firth }
\examples{
########################################################
##  Statistics journal citation data from Stigler (1994)
##  -- see also Agresti (2002, p448)
########################################################

## Convert frequencies to success/failure data
citations.sf <- countsToBinomial(citations)
names(citations.sf)[1:2] <- c("journal1", "journal2")

##  First fit the "standard" Bradley-Terry model
citeModel <- BTm(cbind(win1, win2), journal1, journal2, data = citations.sf)

##  Now the same thing with a different "reference" journal
update(citeModel, refcat = "JASA")

##################################################################
##  Now an example with an order effect -- see Agresti (2002) p438
##################################################################

##  Simple Bradley-Terry model, ignoring home advantage:
baseballModel1 <- BTm(cbind(home.wins, away.wins), home.team, away.team,
                      data = baseball, id = "team")

##  Now incorporate the "home advantage" effect
baseball$home.team <- data.frame(team = baseball$home.team, at.home = 1)
baseball$away.team <- data.frame(team = baseball$away.team, at.home = 0)
baseballModel2 <- update(baseballModel1, formula = ~ team + at.home)

##  Compare the fit of these two models:
anova(baseballModel1, baseballModel2)

##
## For a more elaborate example with both player-level and contest-level
## predictor variables, see help(chameleons).
##

}
\keyword{ models }

\name{predict.BTglmmPQL}
\alias{predict.BTglmmPQL}
\title{
Predict Method for BTglmmPQL Objects
}
\description{
Obtain predictions and optionally standard errors of
those predictions from a \code{"BTglmmPQL"} object.
}
\usage{
\method{predict}{BTglmmPQL}(object, newdata = NULL, newrandom = NULL,
    level = 1, type = c("link", "response", "terms"), se.fit = FALSE,
    terms = NULL, na.action = na.pass, ...)
}
\arguments{
  \item{object}{
a fitted object of class \code{"BTglmmPQL"}
}
  \item{newdata}{
(optional) a data frame in which to look for variables
    with which to predict.  If omitted, the fitted linear predictors are
    used.
}
  \item{newrandom}{
    if \code{newdata} is provided, a corresponding design matrix for the
    random effects, will columns corresponding to the random effects
    estimated in the original model.
}
  \item{level}{
    an integer vector giving
    the level(s) at which predictions are required. Level zero corresponds
    to population-level predictions (fixed effects only), whilst level one
    corresponds to the individual-level predictions (full model) which are
    NA for contests involving individuals not in the original data.
}
  \item{type}{
    the type of prediction required.  The default is on the scale
    of the linear predictors; the alternative \code{"response"} is on
    the scale of the response variable. Thus for a default binomial
    model the default predictions are of log-odds (probabilities on
    logit scale) and \code{type = "response"} gives the predicted
    probabilities. The \code{"terms"} option returns a matrix giving the
    fitted values of each term in the model formula on the linear
    predictor scale (fixed effects only).
}
  \item{se.fit}{
    logical switch indicating if standard errors are required.
}
  \item{terms}{
    with \code{type ="terms"} by default all terms are returned.  A
    character vector specifies which terms are to be returned.
}
  \item{na.action}{
    function determining what should be done with missing
    values in \code{newdata}.  The default is to predict \code{NA}.
}
  \item{\dots}{
    further arguments passed to or from other methods.
}
}
\details{
     If \code{newdata} is omitted the predictions are based on the data used
     for the fit.  In that case how cases with missing values in the
     original fit are treated is determined by the \code{na.action}
     argument of that fit.  If \code{na.action = na.omit} omitted cases
     will not appear in the residuals, whereas if \code{na.action =
       na.exclude} they will appear (in predictions and standard
     errors), with residual value \code{NA}.  See also \code{napredict}.

     Standard errors for the predictions are approximated assuming the
     variance of the random effects is known, see Booth and Hobert
     (1998). 
}
\value{
  If \code{se.fit = FALSE}, a vector or matrix of predictions.  If
  \code{se = TRUE}, a list with components 
  \item{fit }{Predictions}
  \item{se.fit }{Estimated standard errors}
}
\references{
  Booth, J. G. and Hobert, J. P. (1998). Standard errors of prediction
  in Generalized Linear Mixed Models. \emph{Journal of the American
    Statistical Association} \bold{93}(441), 262 -- 272.
}
\author{
  Heather Turner
}
\seealso{
  \code{\link{predict.glm}},  \code{\link{predict.BTm}}
}
\examples{
attach(seeds)

seedsModel <- glmmPQL(cbind(r, n - r) ~ seed + extract,
                      random = diag(length(r)),
                      family = binomial)

pred <- predict(seedsModel, level = 0)
predTerms <- predict(seedsModel, type = "terms")

all.equal(pred, rowSums(predTerms) + attr(predTerms, "constant"))
}
\keyword{ models }

\name{countsToBinomial}
\alias{countsToBinomial}
\title{ Convert Contingency Table of Wins to Binomial Counts }
\description{
  Convert a contingency table of wins to a four-column data frame
  containing the number of wins and losses for each pair of players. 
}
\usage{
countsToBinomial(xtab)
}
\arguments{
  \item{xtab}{ a contingency table of wins cross-classified by
    \dQuote{winner} and \dQuote{loser} }
}
\value{
  A data frame with four columns
  \item{player1 }{ the first player in the contest. }
  \item{player2 }{ the second player in the contest. }
  \item{win1 }{ the number of times \code{player1} won. }
  \item{win2 }{ the number of times \code{player2} won. }
}
\author{ Heather Turner }
\seealso{ \code{\link{BTm}} }
\examples{
########################################################
##  Statistics journal citation data from Stigler (1994)
##  -- see also Agresti (2002, p448)
########################################################
citations

## Convert frequencies to success/failure data
citations.sf <- countsToBinomial(citations)
names(citations.sf)[1:2] <- c("journal1", "journal2")
citations.sf
}
\keyword{ models }

\name{BTabilities}
\alias{BTabilities}
\alias{print.BTabilities}
\alias{coef.BTabilities}
\alias{vcov.BTabilities}
\title{ Estimated Abilities from a Bradley-Terry Model }
\description{
   Computes the (baseline) ability of each player from a model object of
   class \code{"BTm"}. 
}
\usage{
BTabilities(model)
}
\arguments{
  \item{model}{a model object for which \code{inherits(model, "BTm")} is
    \code{TRUE}}
}
\value{
  A two-column numeric matrix (of class \code{c("BTabilities", "matrix")}, with columns named \code{"ability"} and \code{"se"}; has
  one row for each player; has attributes named \code{"vcov"},
  \code{"modelcall"}, \code{"factorname"} and (sometimes --- see below)
  \code{"separate"}.  The first three attributes are not printed
  by the method \code{print.BTabilities}.

  The player abilities are either directly estimated by the model, in
  which case the appropriate parameter estimates are returned, otherwise
  the abilities are computed from the terms of the fitted model that
  involve player covariates only (those indexed by \code{model$id} in the model
  formula). Thus parameters in any other terms are assumed to be zero.   

  If the abilities are structured according to a linear predictor, and
  if there are player covariates with missing values, the
  abilities for the corresponding players are estimated as separate
  parameters.  In this event the resultant matrix has an attribute, named
  \code{"separate"}, which identifies those players whose ability was
  estimated separately.  For an example, see \code{\link{flatlizards}}. 
}
\author{ David Firth and Heather Turner }
\references{
  Firth, D. (2005)  Bradley-Terry models in R.  \emph{Journal of
    Statistical Software},  \bold{12}(1), 1--12.

  Turner, H. and Firth, D. (2012)  Bradley-Terry models in R: The
  BradleyTerry2 package.  \emph{Journal of Statistical
    Software},  \bold{48}(9), 1--21.
}
\seealso{ \code{\link{BTm}}, \code{\link{residuals.BTm}} }
\examples{
### citations example

## Convert frequencies to success/failure data
citations.sf <- countsToBinomial(citations)
names(citations.sf)[1:2] <- c("journal1", "journal2")

##  Fit the "standard" Bradley-Terry model
citeModel <- BTm(cbind(win1, win2), journal1, journal2, data = citations.sf)
BTabilities(citeModel)

### baseball example

baseball$home.team <- data.frame(team = baseball$home.team, at.home = 1)
baseball$away.team <- data.frame(team = baseball$away.team, at.home = 0)
baseballModel2 <- BTm(cbind(home.wins, away.wins), home.team, away.team,
                      formula = ~ team + at.home, id = "team",
                      data = baseball)
##  Estimated abilities for each team, relative to Baltimore, when
##  playing away from home:  
BTabilities(baseballModel2)
}
\keyword{ models }

\name{glmmPQL.control}
\alias{glmmPQL.control}
\title{ Control Aspects of the glmmPQL Algorithm }
\description{
  Set control variables for the glmmPQL algorithm.
}
\usage{
glmmPQL.control(maxiter = 50, IWLSiter = 10, tol = 1e-06, trace = FALSE)
}
\arguments{
  \item{maxiter}{ the maximum number of outer iterations. }
  \item{IWLSiter}{ the maximum number of iterated weighted least squares
  iterations used to estimate the fixed effects, given the standard
  deviation of the random effects. }
  \item{tol}{ the tolerance used to determine convergence in the
    IWLS iterations and over all (see details). }
  \item{trace}{ logical: whether or not to print the score for the
    random effects variance at the end of each iteration. } 
}
\details{
  This function provides an interface to control the PQL algorithm used
  by \code{\link{BTm}} for fitting Bradley Terry models with random
  effects.

  The algorithm iterates between a series of iterated weighted least
  squares iterations to update the fixed effects and a single Fisher
  scoring iteration to update the standard deviation of the random
  effects.

  Convergence of both the inner and outer iterations are
  judged by comparing the squared components of the relevant score vector 
  with corresponding elements of the diagonal of the Fisher information
  matrix. If, for all components of the relevant score vector, the ratio
  is less than \code{tolerance^2}, or the corresponding diagonal element
  of the Fisher information matrix is less than 1e-20, iterations cease. 
}
\value{
  A list with the arguments as components.
}
\references{
  Breslow, N. E. and Clayton, D. G. (1993), Approximate inference in
  Generalized Linear Mixed Models. \emph{Journal of the American
    Statistical Association} \bold{88}(421), 9--25.
}
\author{ Heather Turner }
\seealso{\code{\link{glmmPQL}}, \code{\link{BTm}} }
\examples{
## Variation on example(flatlizards)
attach(flatlizards)
result <- rep(1, nrow(contests))

## BTm passes arguments on to glmmPQL.control()
args(BTm)
BTmodel <- BTm(result, winner, loser, ~ throat.PC1[..] + throat.PC3[..] +
               head.length[..] + SVL[..] + (1|..),
               data = list(contests, predictors), tol = 1e-3, trace = TRUE)
summary(BTmodel)
}
\keyword{ models }

\name{CEMS}
\alias{CEMS}
\docType{data}
\title{
Dittrich, Hatzinger and Katzenbeisser (1998, 2001) Data on Management School Preference in Europe
}
\description{
\emph{Community of European management schools} (CEMS) data as used in the paper by Dittrich et al. (1998, 2001), re-formatted for use with \code{\link{BTm}}
}
\usage{CEMS}
\format{
A list containing three data frames, \code{CEMS$preferences},
\code{CEMS$students} and \code{CEMS$schools}.

The \code{CEMS$preferences} data frame has \code{303 * 15 = 4505} observations (15 possible comparisons, for each of 303 students) on the following 8 variables:
 \describe{
	\item{\code{student}}{a factor with levels \code{1:303}}
        \item{\code{school1}}{a factor with levels \code{c("Barcelona",
 "London", "Milano", "Paris", "St.Gallen", "Stockholm")}; the first management school in a comparison}
        \item{\code{school2}}{a factor with the same levels as \code{school1}; the second management school in a comparison}
        \item{\code{win1}}{integer (value 0 or 1) indicating whether
	  \code{school1} was preferred to \code{school2}}
	\item{\code{win2}}{integer (value 0 or 1) indicating whether
 \code{school2} was preferred to \code{school1}}
        \item{\code{tied}}{integer (value 0 or 1) indicating whether no
 preference was expressed}
        \item{\code{win1.adj}}{numeric, equal to \code{win1 + tied/2}}
        \item{\code{win2.adj}}{numeric, equal to \code{win2 + tied/2}}
	}

The \code{CEMS$students} data frame has 303 observations (one for each student) on the following 8 variables:
\describe{
   \item{\code{STUD}}{a factor with levels \code{c("other",
       "commerce")}, the student's main discipline of study}
   \item{\code{ENG}}{a factor with levels \code{c("good, poor")},
     indicating the student's knowledge of English}
   \item{\code{FRA}}{a factor with levels \code{c("good, poor")},
     indicating the student's knowledge of French}
   \item{\code{SPA}}{a factor with levels \code{c("good, poor")},
     indicating the student's knowledge of Spanish}
   \item{\code{ITA}}{a factor with levels \code{c("good, poor")},
     indicating the student's knowledge of Italian}
   \item{\code{WOR}}{a factor with levels \code{c("no", "yes")},
     whether the student was in full-time employment while studying}
   \item{\code{DEG}}{a factor with levels \code{c("no", "yes")}, whether
     the student intended to take an international degree}
   \item{\code{SEX}}{a factor with levels \code{c("female", "male")}
   }
 }

The \code{CEMS$schools} data frame has 6 observations (one for each
management school) on the following 7 variables:
\describe{
  \item{\code{Barcelona}}{numeric (value 0 or 1)}
  \item{\code{London}}{numeric (value 0 or 1)}
  \item{\code{Milano}}{numeric (value 0 or 1)}
  \item{\code{Paris}}{numeric (value 0 or 1)}
  \item{\code{St.Gallen}}{numeric (value 0 or 1)}
  \item{\code{Stockholm}}{numeric (value 0 or 1)}
  \item{\code{LAT}}{numeric (value 0 or 1) indicating a 'Latin' city}
}
}
\details{
The variables \code{win1.adj} and \code{win2.adj} are provided in order
 to allow a simple way of handling ties (in which a tie counts as half a
 win and half a loss), which is slightly different numerically from the
 Davidson (1970) method that is used by Dittrich et al. (1998): see the
 examples.
}
\source{
Royal Statistical Society datasets website, at
\url{http://onlinelibrary.wiley.com/journal/10.1111/(ISSN)1467-9876/homepage/47_4.htm}.
 }


\references{
  Davidson, R. R. (1970)  Extending the Bradley-Terry model to
  accommodate ties in paired comparison experiments.  \emph{Journal of
  the American Statistical Association} \bold{65}, 317--328.
  
  Dittrich, R., Hatzinger, R. and Katzenbeisser, W. (1998)  Modelling
  the effect of subject-specific covariates in paired comparison studies
  with an application to university rankings.  \emph{Applied Statistics}
  \bold{47}, 511--525.

  Dittrich, R., Hatzinger, R. and Katzenbeisser, W. (2001)  Corrigendum:
  Modelling the effect of subject-specific covariates in paired
  comparison studies with an application to university
  rankings. \emph{Applied Statistics} \bold{50}, 247--249.
  
  Turner, H. and Firth, D. (2012)  Bradley-Terry models in R: The
  BradleyTerry2 package.  \emph{Journal of Statistical
    Software},  \bold{48}(9), 1--21.
}
\author{David Firth}
\examples{
##
##  Fit the standard Bradley-Terry model, using the simple 'add 0.5'
##  method to handle ties:
##
table3.model <-  BTm(outcome = cbind(win1.adj, win2.adj),
                     player1 = school1, player2 = school2,
                     formula = ~.. , refcat = "Stockholm",
                     data = CEMS)
##  The results in Table 3 of Dittrich et al (2001) are reproduced
##  approximately by a simple re-scaling of the estimates:
table3 <- summary(table3.model)$coef[, 1:2]/1.75
print(table3)
##
##  Now fit the 'final model' from Table 6 of Dittrich et al.:
##
table6.model <-  BTm(outcome = cbind(win1.adj, win2.adj),
                     player1 = school1, player2 = school2,
                     formula = ~ .. +
                         WOR[student] * Paris[..] +
                         WOR[student] * Milano[..] +
                         WOR[student] * Barcelona[..] +
                         DEG[student] * St.Gallen[..] +
                         STUD[student] * Paris[..] +
                         STUD[student] * St.Gallen[..] +
                         ENG[student] * St.Gallen[..] +
                         FRA[student] * London[..] +
                         FRA[student] * Paris[..] +
                         SPA[student] * Barcelona[..] +
                         ITA[student] * London[..] +
                         ITA[student] * Milano[..] +
                         SEX[student] * Milano[..],
                     refcat = "Stockholm",
                     data = CEMS)
##
##  Again re-scale to reproduce approximately Table 6 of Dittrich et
##  al. (2001):
##
table6 <- summary(table6.model)$coef[, 1:2]/1.75
print(table6)
##
\dontrun{
##  Now the slightly simplified model of Table 8 of Dittrich et al. (2001):
##
table8.model <-  BTm(outcome = cbind(win1.adj, win2.adj),
                     player1 = school1, player2 = school2,
                     formula = ~ .. +
                         WOR[student] * LAT[..] +
                         DEG[student] * St.Gallen[..] +
                         STUD[student] * Paris[..] +
                         STUD[student] * St.Gallen[..] +
                         ENG[student] * St.Gallen[..] +
                         FRA[student] * London[..] +
                         FRA[student] * Paris[..] +
                         SPA[student] * Barcelona[..] +
                         ITA[student] * London[..] +
                         ITA[student] * Milano[..] +
                         SEX[student] * Milano[..],
                     refcat = "Stockholm",
                     data = CEMS)
table8 <- summary(table8.model)$coef[, 1:2]/1.75
##
##  Notice some larger than expected discrepancies here (the coefficients
##  named "..Barcelona", "..Milano" and "..Paris") from the results in
##  Dittrich et al. (2001).  Apparently a mistake was made in Table 8 of
##  the published Corrigendum note (R. Dittrich personal communication,
##  February 2010).
##
print(table8)
}
}
\keyword{datasets}
